package sieve.beanbox;

import java.util.*;
import java.beans.*;
import java.lang.reflect.*;
import sieve.beanbox.encapsulatedEvents.*;

public class EventAdapter implements EncapsulatedEventListener,
	BeanBoxComponent {
    private NewEventManager em = null;
    private Object source = null;
    private Object dest = null;
    private PropertyChangeSupport listeners = null;

    private MethodDescriptor listenerMethod = null;
    private String listenerInterface = null;

    private MethodDescriptor methodToInvoke = null;

    public EventAdapter() {
	em = new NewEventManager();
	listeners = new PropertyChangeSupport(this);
    }

    public void encapsulatedEvent(EncapsulatedEvent ee) {
        Object      s  = ee.getEventSource();
        EventObject eo = ee.getEvent();

	Method method = ee.getListenerMethod();
	String listener = ee.getListenerInterfaceName();

	if (same(listener, listenerInterface) && 
	    same(method, listenerMethod.getMethod()) &&
	    (methodToInvoke != null)) {
	    try {
// System.out.println("  Trying to invoke method: " + methodToInvoke.getName());
		Method m = methodToInvoke.getMethod();
		Object[] args = new Object[0];
		if (m.getParameterTypes().length != 0)
		    args = ee.getEventArguments();
	        m.invoke(dest, args);
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}

	
    }

    public void setSource(Object source) {
	Object oldSource = this.source;
	EventSetDescriptor[] esd = null;
        try {
            esd = Introspector.getBeanInfo(
		source.getClass()).getEventSetDescriptors();
        } catch (IntrospectionException ie) {
            System.err.println("Failed to introspect");
        }

	em.addEncapsulatedEventListener(source, this);
	this.source = source;
	listeners.firePropertyChange("source", oldSource, source);
    }

    public Object getSource() {
	return source;
    }

    private static boolean same(Object s1, Object s2) {
	return (s1 == s2) || ((s1 != null) && (s2 != null) && (s2.equals(s1)));
    }

    public void setListenerMethod(MethodDescriptor method) {
	if (! same(listenerMethod, method)) {
	    MethodDescriptor oldMethod = listenerMethod;
	    this.listenerMethod = method;
	    listeners.firePropertyChange("listenerMethod", oldMethod, 
		method);
	}
    }
    public MethodDescriptor getListenerMethod() {
	return listenerMethod;
    }

    public void setListenerInterface(String interfaceName) {
	if (! same(listenerInterface, interfaceName)) {
	    String oldInterface = listenerInterface;
	    this.listenerInterface = interfaceName;
	    listeners.firePropertyChange("listenerInterface", oldInterface, 
		interfaceName);
	}
    }
    public String getListenerInterface() {
	return listenerInterface;
    }
 
    public void setDest(Object dest) {
	Object oldDest = this.dest;
	this.dest = dest;
	listeners.firePropertyChange("dest", oldDest, dest);
    }

    public Object getDest() {
	return dest;
    }

    public void setMethodToInvoke(MethodDescriptor method) {
	if (! same(methodToInvoke, method)) {
	    MethodDescriptor oldMethod = listenerMethod;
	    this.methodToInvoke = method;
	    listeners.firePropertyChange("methodToInvoke", oldMethod, 
		method);
	}
    }
    public MethodDescriptor getMethodToInvoke() {
	return methodToInvoke;
    }


    public void addPropertyChangeListener(PropertyChangeListener l) {
	listeners.addPropertyChangeListener(l);
    }
 
    public void removePropertyChangeListener(PropertyChangeListener l) {
	listeners.removePropertyChangeListener(l);
    }
}

