package sieve.beanbox;

import java.awt.*;
import java.beans.*;
import java.lang.reflect.*;
import sieve.workbench.*;

public class PropertyBinder implements PropertyChangeListener, 
    BeanBoxComponent {

    static {
	BeanBoxLink.setup();
    }

    private Object source = null;
    private Object dest = null;
    private String sourceProperty = null;
    private String destProperty = null;
    private PropertyDescriptor[] sourceProperties = null;
    private PropertyChangeSupport listeners = null;

    public PropertyBinder() { 
	listeners = new PropertyChangeSupport(this);
    }

    private static boolean same(Object s1, Object s2) {
        return (s1 == s2) || ((s1 != null) && (s2 != null) && (s2.equals(s1)));
    }

    public String getSourceProperty() {
	return sourceProperty;
    }

    public void setSourceProperty(String property) {
	if (! same(property, sourceProperty)) {
	    String oldProp = sourceProperty;
	    sourceProperty = property;
	    listeners.firePropertyChange("sourceProperty", oldProp, property);
   	}
    }

    public String getDestProperty() {
	return destProperty;
    }

    public void setDestProperty(String property) {
	if (! same(property, destProperty)) {
	    String oldProp = destProperty;
	    destProperty = property;
	    listeners.firePropertyChange("destProperty", oldProp, property);
   	}
    }

    public PropertyDescriptor[] getSourceProperties() {
	return sourceProperties;
    }

    public void propertyChange(PropertyChangeEvent evt) {
	if ((sourceProperty != null) && (destProperty != null) &&
	    (dest != null) && (sourceProperty.equals(evt.getPropertyName()))) {
	    try {
		PropertyDescriptor pd = 
		    new PropertyDescriptor(destProperty, dest.getClass());
		Method m = pd.getWriteMethod();
		Object[] args = {evt.getNewValue()};
		m.invoke(dest, args);
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
    }

    public void setSource(Object source) {
	if (this.source != source) {
	    Object oldSource = this.source;
	    try {
	        // Make sure we can register as a property change listener
	        // and that the there is at least one property before accepting
                // this source
	        Class cl = source.getClass();
	        Class parms[] = {PropertyChangeListener.class};
	        Method reg = cl.getMethod("addPropertyChangeListener", parms);
	        BeanInfo bi = Introspector.getBeanInfo(cl);

	        PropertyDescriptor[] props = bi.getPropertyDescriptors();

	        if (props.length > 0) {
	            // Store a ref to our source and register ourselves 
		    // as a listener
	            this.source = source;
	            this.sourceProperties = props;
	            Object[] me = {this};
	            reg.invoke(source, me);
	            listeners.firePropertyChange("source", oldSource, source);

	            // Default to listening for the first property 
	            this.sourceProperty = props[0].getName();
	        }
	    }
	    catch(Exception e) {
	        e.printStackTrace();
	    }
	}
    }

    public Object getSource() {
	return source;
    }

    public void setDest(Object dest) {
	if (dest != this.dest) {
	    Object oldDest = this.dest;
	    this.dest = dest;
	    listeners.firePropertyChange("dest", oldDest, dest);
	}
    }

    public Object getDest() {
	return dest;
    }

    public void addPropertyChangeListener(PropertyChangeListener l) {
        listeners.addPropertyChangeListener(l);
    }
 
    public void removePropertyChangeListener(PropertyChangeListener l) {
        listeners.removePropertyChangeListener(l);
    }

}


