package sieve.beanbox;

import java.beans.*;
import java.awt.*;
import java.util.*;
import java.lang.reflect.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

public class PropertyBinderCustomizer extends Container implements Customizer {
    private static Dimension zero = new Dimension(0,0);

    private JList sProps = null;
    private JList dProps = null;

    private PropertyBinder binder = null;
    private PropertyChangeSupport listeners = null;
    private PropertyChangeListener spcl = null;

    private PropertyDescriptor[] sourcePDs = null;
    private PropertyDescriptor[] destPDs = null;

    private boolean processing = false;

    public PropertyBinderCustomizer() {
	setLayout(new GridLayout(1, 2));
	sProps = new JList();
	sProps.addListSelectionListener(new SourcePropListener());
	JScrollPane sScroll = new JScrollPane(sProps);
	add(sScroll);

	dProps = new JList();
	dProps.addListSelectionListener(new DestPropListener());
	JScrollPane dScroll = new JScrollPane(dProps);
	add(dScroll);

	listeners = new PropertyChangeSupport(this);
	spcl = new Reseter();
	setSize(new Dimension(200, 120));
    }
    private class Reseter implements PropertyChangeListener {
	public void propertyChange(PropertyChangeEvent e) {
	    reset();
	}
    }
    private class SourcePropListener implements ListSelectionListener {
	public void valueChanged(ListSelectionEvent e) {
	    PropertyWrapper pw = (PropertyWrapper) sProps.getSelectedValue();
	    if (pw != null) 
		binder.setSourceProperty(pw.getPropertyDescriptor().getName());
	    resetDestProps();
	}
    }
    private class DestPropListener implements ListSelectionListener {
	public void valueChanged(ListSelectionEvent e) {
	    PropertyWrapper pw = (PropertyWrapper) dProps.getSelectedValue();
	    if (pw != null) 
		binder.setDestProperty(pw.getPropertyDescriptor().getName());
	}
    }

    private void setupPropertyList(Object source, JList list, 	
	String selectedProp, Class compatClass) {
	PropertyDescriptor[] pds = null;
        try {
           pds = Introspector.getBeanInfo(
		source.getClass()).getPropertyDescriptors();
        } 
	catch (IntrospectionException ie) {
            System.err.println("Failed to introspect");
	    return;
        }
	Vector pdVector = new Vector(pds.length);
	PropertyWrapper selected = null;
	for (int i = 0; i < pds.length; i++) {
	    Class propClass = pds[i].getPropertyType();
	    if ((compatClass == null) || ((propClass != null) && 
		propClass.isAssignableFrom(compatClass))) {
	  	PropertyWrapper pw = new PropertyWrapper(pds[i]); 
		pdVector.addElement(pw);
	        if ((selectedProp != null) && 
		    selectedProp.equals(pds[i].getName()))
		    selected = pw;
	    }
	}
	list.setListData(pdVector);
	if (selected != null) 
	    list.setSelectedValue(selected, true);
    }

    private synchronized void reset() {
	if (! processing && (binder != null)) {
	    processing = true;
	    Object source = binder.getSource();
	    if (source != null) {
	        setupPropertyList(source, sProps, binder.getSourceProperty(),
		    null);
		if (binder.getSourceProperty() != null)
		    resetDestProps();
	    }
	    processing = false;
	}
	else {
	    Object[] empty = new Object[0];
	    sProps.setListData(empty);
	    dProps.setListData(empty);
	}
    }

    private void resetDestProps() {
	// Should we allow multiple selection here?
	PropertyWrapper pw = (PropertyWrapper) sProps.getSelectedValue();
	Object dest = binder.getDest();
	if ((pw != null) && (dest != null)) {
	    setupPropertyList(dest, dProps, binder.getDestProperty(),
		pw.getPropertyDescriptor().getPropertyType());
	}
    }

    // This is here to get toString() to call getName() (for JList)
    private class PropertyWrapper {
	private PropertyDescriptor pd = null;
	public PropertyWrapper(PropertyDescriptor pd) {
	    this.pd = pd;
	}
	public PropertyDescriptor getPropertyDescriptor() {
	    return pd;
	}
	public String toString() {
	    return pd.getName();
	}
    }

    // ------------------- Customizer methods -------------------
    public void setObject(Object bean) {
	if (binder != null) 
	    binder.removePropertyChangeListener(spcl);
	binder = (PropertyBinder)bean;
	if (binder != null) 
	    binder.addPropertyChangeListener(spcl);
	reset();
    }

    public void addPropertyChangeListener(PropertyChangeListener l) {
	listeners.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
	listeners.removePropertyChangeListener(l);
    }

    /**
     * Return true if class a is either equivalent to class b, or
     * if class a is a subclass of class b.
     * Note tht either or both "Class" objects may represent interfaces.
     */
    static boolean isSubclass(Class a, Class b) {
        // We rely on the fact that for any given java class or
        // primtitive type there is a unqiue Class object, so
        // we can use object equivalence in the comparisons.
        if (a == b) {
            return true;
        }
        if (a == null || b == null) {
            return false;
        }
        for (Class x = a; x != null; x = x.getSuperclass()) {
            if (x == b) {
                return true;
            }
            if (b.isInterface()) {
                Class interfaces[] = x.getInterfaces();
                for (int i = 0; i < interfaces.length; i++) {
                    if (interfaces[i] == b) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public Dimension getPreferredSize() {
        Dimension current = getSize();
        return (current.equals(zero) ? super.getPreferredSize() : current);
    }

}


