/*
 *
 * Written By:   James "Bo" Begole
 * Date:         Feb 20 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package sieve.crew;

import sieve.quarry.*;
import java.awt.Rectangle;
import java.util.Hashtable;

/**
  Container of enough information to instantiate a new bean (nugget).
 * @version $Revision  $
 */

public class NuggetInfo implements java.io.Serializable {
    String className = null;
    String identifier = null;
    Rectangle bounds = null;
    String creatorId = null;
    transient Wrapper wrapper = null;
    transient Object nugget = null;

    private static Hashtable objIndex = new Hashtable();
    private static Hashtable idIndex = new Hashtable();

  public static NuggetInfo getNuggetInfo(Object nugget) {
    NuggetInfo ni = (NuggetInfo)objIndex.get(nugget);
    if (ni == null) {
      ni = new NuggetInfo(nugget.getClass().getName(),
           "id:"+System.currentTimeMillis(),  // should prepend hostname 
			  new Rectangle(0,0,0,0),
			  nugget);
      registerNuggetInfo(nugget, ni);
    }
    return ni;
  }

  /* 
     static so it is callable from the class reference instead of from an instance reference
     */
  public static void registerNuggetInfo(Object nugget, NuggetInfo ni) {
    //    System.out.println("registerNuggetInfo "+ni);
    objIndex.put(nugget, ni);
    idIndex.put(ni.getIdentifier(), ni);
  }


  public static NuggetInfo getNuggetInfo(String identifier){
    return (NuggetInfo)idIndex.get(identifier);
  }

  protected NuggetInfo( String className, 
			String identifier,
			Rectangle bounds,
			Object nugget) {
    this.className = className;
    this.identifier = identifier;
    this.bounds = bounds;
    this.nugget = nugget;
  }

  public String getClassName() {
    return className;
  }
  public void setClassName(String className) {
    this.className = className;
  }
  public String getIdentifier() {
    return identifier;
  }
  public void setIdentifier(String identifier) {
    this.identifier = identifier;
  }
  public Rectangle getBounds() {
    return bounds;
  }
  public void setBounds(Rectangle bounds) {
    this.bounds = bounds;
  }

    public void setCreatorId(String creatorId) {
	this.creatorId = creatorId;
    }
    public String getCreatorId() {
	return creatorId;
    }

  public Object getNugget() {
    return nugget;
  }
  public void setNugget(Object nugget) {
    this.nugget = nugget;
  }
  public Wrapper getWrapper() {
    return wrapper;
  }
  public void setWrapper(Wrapper wrapper) {
    this.wrapper = wrapper;
  }

  public String toString(){
    String parmString = " Class Name = "+className+
      " Identifier = "+identifier+
      " Bounds = "+bounds;
    return "["+getClass().getName()+":"+parmString+"]";
  }


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
}
