/*
 *
 * Written By:   James "Bo" Begole
 * Date:         Feb 20 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package sieve.crew;

import sieve.quarry.*;
import SharedData.*;
import vt.cs.widgets.*;
import java.util.*;

/**
 * Consumer for a channel that recieves control 
 * event messages.
 * These messages will be converted to calls to a
 *  object.
 *
 * @version $Revision  $
 */
public class ParticipantControl extends RemoteControl {
  public static final String ChannelName = "ParticipantControl";
  private QuarryBox beanBox = null;
  private MURadarPane muRadarPane = null;
  private Participant localParticipant = null;
  private Participant localEcho = null;
  private Telepointer localTelepointer = null;

  public ParticipantControl(Session session, QuarryBox beanBox) {
    super(session, ChannelName);

    this.beanBox = beanBox;

  }
 
  public void shutdown() {
    notifyParticipants(ParticipantMessage.REMOVE, 
	new Participant(localParticipant)); // PLI -- copying participant
    super.shutdown();
  }

  public void setLocalParticipant(Participant localParticipant) {
    if (localParticipant != null) {
      beanBox.removeTelepointer(localParticipant.getIdentifier());
    }
    participants.put(localParticipant.getIdentifier(), localParticipant);
    this.localParticipant = localParticipant;
    if (muRadarPane == null)
      muRadarPane = beanBox.getFrame().scroller;

    muRadarPane.addLocalViewPort(localParticipant);
  }

    public Participant getLocalParticipant() {
	return localParticipant;
    }

    public synchronized Participant getParticipant(String id) {
	return (Participant)participants.get(id);
    }

  public synchronized void  notifyParticipants(int msgType, Object data) {
    debug("notifyParticipants: "+data);
    if (channel != null) {
      try {
	channel.uniformSendToOthers(this, 
       	       new Data(ParticipantMessage.toByteArray(
	       new ParticipantMessage(msgType, data))));
      } catch (Exception sdex) {
	sdex.printStackTrace();
      }
    }
  }

  boolean sendCursorChange = true;
  int changeCounter = 0;
  Date lastSent = new Date();

  public synchronized void notifyCursor() {
    debug("notifyCursor");
    if (((++changeCounter % 5) == 0) || 
	((System.currentTimeMillis() - lastSent.getTime()) > 10000))
      sendCursorChange = true;

    if (!sendCursorChange) 
      return;
    if (channel != null) {
      try {
	channel.uniformSendToAll(this, 
       	       new Data(ParticipantMessage.toByteArray(
	       new ParticipantMessage(ParticipantMessage.MOVE, 
	          new Participant(localParticipant))))); // PLI -- copying part
      } catch (SharedDataException sdex) {
	sdex.printStackTrace();
      }
      lastSent = new Date();
      sendCursorChange = false;
    }
  }


  Hashtable participants = new Hashtable();


  public void setLocalEcho(Participant localEcho) {
    this.localEcho = localEcho;
  }
  public Participant getLocalEcho() {
    return localEcho;
  }


  /* ------- ChannelConsumer Interface methods ------ */
  // Bad things happen when I do this... PLI
  //public synchronized void dataReceived(Data data) {

  public void dataReceived(Data data) {
    Message msg = null;
    
    if (muRadarPane == null)
      muRadarPane = beanBox.getFrame().scroller;

    msg = (Message)Message.messageFromData(data);
    Participant tempParticipant = (Participant)(msg.data);
    debug("Message type = "+msg.type+"  participant ="+tempParticipant);
    if (tempParticipant == null)
      return;
    switch(msg.type) {
    case ParticipantMessage.CREATE:
      if (localParticipant != null) {
	notifyParticipants(ParticipantMessage.MODIFY, 
	    new Participant(localParticipant)); // PLI -- copying participant
      }
      // fall through...
    case ParticipantMessage.MODIFY:
      debug("ParticipantMessage.MODIFY");
    case ParticipantMessage.MOVE:
      if (localParticipant != null) {
	if (tempParticipant.getIdentifier().equals(
			      localParticipant.getIdentifier())) {

	  debug("message from self "+tempParticipant);
	  // this is the return of our last notifyCursor
	  // allow future cursor updates
	  sendCursorChange = true;

	  if (localEcho == null)
	    localEcho = tempParticipant;
	  else
	    localEcho.setParticipant(tempParticipant);

	  if (beanBox.getCursorMoved()) {
	    // there could be a concurrency race here, but
	    // since cursors aren't crucial, we won't synchronize it.
	    notifyCursor();
	    beanBox.setCursorMoved(false);
	    beanBox.repaint();
	  }
	  
	  return;
	}
      }
      synchronized(participants) {
	debug("participant = "+tempParticipant);
	Participant updateParticipant = (Participant)participants.get(
								      tempParticipant.getIdentifier());
	if (updateParticipant != null) {
	  debug("participant found "+tempParticipant.getIdentifier());
	  updateParticipant.setParticipant(tempParticipant);
	} else {
	  debug("participant *not* found "+tempParticipant.getIdentifier());
	  participants.put(tempParticipant.getIdentifier(), tempParticipant);
	  Telepointer telepointer = new Telepointer(tempParticipant);
	  telepointer.setShowName(beanBox.showNames.getState());
	  beanBox.addTelepointer(tempParticipant.getIdentifier(), telepointer);
	  muRadarPane.addParticipant(tempParticipant.getIdentifier(), tempParticipant);
	}
	beanBox.repaint();
      }
      break;

    case ParticipantMessage.REMOVE:
      Participant participant = (Participant)participants.remove(tempParticipant.getIdentifier());
      participant.deleteObservers();
      Telepointer telepointer = beanBox.removeTelepointer(tempParticipant.getIdentifier());
      muRadarPane.removeParticipant(tempParticipant.getIdentifier());
      break;

    default:
      System.err.println("Unknown message type received.  Ignoring.");
    }
  }
  /* ------------------------------------------------ */





  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    System.out.println("Sieve: athenticate: challenge="+challenge);
    return null;
  }


  public String getName() {
    debug("getName() "+name);
    return name;
  }
  public void clientChanged(ClientEvent event) {
        System.out.println("Sieve: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  } 
}
