/*
 *
 * Written By:   James "Bo" Begole
 * Date:         Feb 20 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package sieve.crew;

import sieve.quarry.*;
import SharedData.*;
import java.net.*;

/**
 * Consumer for a channel that recieves control 
 * event messages.
 * These messages will be converted to calls to a
 *  object.
 *
 * @version $Revision  $
 */
public abstract class RemoteControl implements ChannelConsumer, Client {
  protected Session session = null;
  protected Channel channel = null;
  protected String name = null;
  protected String channelName = null;

  public RemoteControl(Session session, String channelName) {
    debug("Contructor begin");
    this.session = session;
    this.channelName = channelName;
    
    String localHostName = "LocalHost";
    try {
      localHostName = InetAddress.getLocalHost().getHostName();
    } catch (UnknownHostException uhex) {
      debug("UnknownHostException when trying to get local host.");
    }
    name = "RemoteControl-"+localHostName+"-"+System.currentTimeMillis();
    debug("Contructor - pre joinSession");
    joinSession();
    debug("Contructor - pre joinChannel");
    joinChannel();
    debug("Contructor end");
  }

  private void joinSession() {
    try {
      session.join(this);
      debug("joined session "+session);
      session.createChannel( this, channelName, true, true);
      debug("Channel successfully created");
    } catch (Exception sdex) {
      // this channel may already exist.  If so, great.
      if (sdex instanceof SharedDataException){
	if (((SharedDataException)sdex).getType() 
	    != SharedDataException.CHANNEL_EXISTS){
	  sdex.printStackTrace();
	}
      }
      else
	sdex.printStackTrace();
    }
  }

  private void joinChannel() {
    try {
      debug("Joining Control Channel"); 
      // create controlChannel
      channel = Channel.join(session, channelName, this);
      debug("channel successfully joined.");
    } catch (SharedDataException sdex) {
      System.out.println("Error joining channel: "+sdex);
      sdex.printStackTrace();
    }
    debug("end of shared data initilization.");

    try {
      channel.addConsumer(this, this);
    } catch (SharedDataException e) {
      e.printStackTrace();
    }
  }
 

  public void  notifyParticipants(int msgType, Object data) {
    debug("notifyParticipants: "+data);
      if (channel != null) {
	try {
	  channel.uniformSendToOthers(this, 
                  new Data(Message.toByteArray(
			     new Message(msgType, data))));
	} catch (Exception sdex) {
	  sdex.printStackTrace();
	}
      }
  }


  public void shutdown() {
    try {
      channel.leave(this);
      session.leave(this);
    } catch (SharedDataException sdex) {
      System.out.println("Error shutting down control");
      sdex.printStackTrace();
    }
  }


  /* ----------- Client Interface methods ----------- */
  public String authenticate(AuthenticationInfo info) {
    debug("Sieve: athenticate: challenge="+info);
    return null;
  }

  public String getName() {
    debug("getName() "+name);
    return name;
  }
  public void clientChanged(ClientEvent event) {
        debug("Sieve: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */



  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  } 
}

