/*
 * @(#)Chart.java	1.6f 95/03/27 Sami Shaio
 *
 * Copyright (c) 1994-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 */

package sieve.nuggets;

import sieve.sluice.*;
import sieve.util.*;
import sieve.sluice.views.*;
import sieve.sluice.event.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.net.URL;

//**********************************************************************
// this originally extended TableViewApplet. Changed to extend 
// TableViewJComponent, 1-15-98. All other methods same (lwjones)
//********************************************************************** 

public class BarChart extends TableViewJComponent implements TableView, 
    TableViewListener, MouseListener, MouseMotionListener { 

    static final int	VERTICAL = 0;
    static final int 	HORIZONTAL = 1;

    static final int	SOLID = 0;
    static final int	STRIPED = 1;

    static final String MEAN_LABEL = " Mean ";
    static final String MEDIAN_LABEL = " Median ";
    static final String MODE_LABEL = " Mode ";

    int			orientation = HORIZONTAL;
    String		title;
    Font		titleFont;
    FontMetrics		titleFontMetrics;
    int			titleHeight = 15;
    int			columns;
    int			values[];
    Object		colors[];
    String		labels[];
    String		abbrvs[];
    boolean		staggerLabels = false;
    int			styles[];
    double		scale = 10;
    int			maxLabelWidth = 0;
    int			barWidth;
    int			barSpacing = 10;
    int			max = 0;
    int			baseLine = 0;  // The location of the base of the chart
    int			firstBar = 0;  // The location of the first bar
    boolean		drawShadow = true;
    boolean		showFrequencies = true;
    boolean		outlineOnly = false;
    boolean		showMeanMedianMode = false;

    String		labelField	= null;
    Object[]		valueObjects	= null;
    String		freqField	= null;
    Object[]		freqObjects	= null;
    Hashtable		labelStrings	= null;
    boolean		showLabels	= true;
    boolean		showTicks	= false;
    int			numberOfTicks   = 3;

    boolean		okToDraw	= false;
 
    Color		labelColor	= Color.black;
    Color		barColor	= Color.black;

    Vector              modeColumns	= null;
    Color		modeColor	= Color.blue;

    boolean             hasMedian	= true;
    int			medianColumn	= 0;
    Color		medianColor	= Color.red;

    boolean             hasMean	        = false;
    int			meanColumn	= 0;
    double		mean		= 0.0;
    Color		meanColor	= Color.darkGray;
    Vector  		namedBins 	= new Vector();

    int			mouseColumn	= -1; // Column mouse is over
    boolean		hasAbbreviations = false;
    int			lastWidth	= 0;


    // For buffering
    Image offScreen = null;

    public BarChart() {
	super();
	addMouseListener(this);
	addMouseMotionListener(this);
	setName("Bar Chart");
    }

    public synchronized void start() {
    }

    public int getBarSpacing() {
	return barSpacing;
    }
    public void setBarSpacing(int barSpacing) {
	if (barSpacing > this.barSpacing + this.barWidth)
	    this.barSpacing = this.barSpacing + this.barWidth + 1; 
	else
	    this.barSpacing = barSpacing; 
	repaint();
    }

    public boolean getDrawShadow() {
	return drawShadow;
    }
    public void setDrawShadow(boolean drawShadow) {
	this.drawShadow = drawShadow;
	repaint();
    }

    public boolean getShowLabels() {
	return showLabels;
    }
    public void setShowLabels(boolean showLabels) {
	this.showLabels = showLabels;
	repaint();
    }

    public boolean getShowMeanMedianMode() {
	return showMeanMedianMode;
    }
    public void setShowMeanMedianMode(boolean showMeanMedianMode) {
	this.showMeanMedianMode= showMeanMedianMode;
	repaint();
    }

    public boolean getShowFrequencies() {
	return showFrequencies;
    }
    public void setShowFrequencies(boolean showFrequencies) {
	this.showFrequencies = showFrequencies;
	repaint();
    }

    // Should probably be modes: none, hash, full line, reverse line
    public boolean getShowTicks() {
	return showTicks;
    }
    public void setShowTicks(boolean showTicks) {
	this.showTicks = showTicks;
	repaint();
    }

    public int getNumberOfTicks() {
	return numberOfTicks;
    }
    public void setNumberOfTicks(int numberOfTicks) {
	if (numberOfTicks < max)
	    this.numberOfTicks = numberOfTicks; // Use CONSTRAINED prop?
	else
	    this.numberOfTicks = max;
	repaint();
    }

    public boolean getOutlineOnly() {
	return outlineOnly;
    }
    public void setOutlineOnly(boolean outlineOnly) {
	this.outlineOnly= outlineOnly;
	repaint();
    }

    public boolean getDisplayVertical() {
	return (orientation == VERTICAL);
    }

    public void setDisplayVertical(boolean isVertical) {
	boolean orig = getDisplayVertical();
	if (orig != isVertical) {
	    if (isVertical)
		orientation = VERTICAL;
	    else
		orientation = HORIZONTAL;
	    repaint();
	}
    }

    public void tableViewChanged(TableViewEvent event) {
        if (source == null) {
	    repaint();
	}
	else if (event instanceof FieldDescriptorChangeEvent) {
	    FieldDescriptorChangeEvent ev = (FieldDescriptorChangeEvent) event;
	    FieldDescriptor fd = null;
	    try {
                fd = source.getFieldDescriptor(ev.getField());
	    }
	    catch (InterchangeException e) {
		e.printStackTrace();
		return;
	    }

	    if (ev.getField().equals(labelField)) {
	        title = fd.getDescription(); 
                labelColor = fd.getColor();

		// The level of the data should probably not change, but
		// just in case...
	        if (fd.getLevel() == FieldDescriptor.LEVEL_NOMINAL) {
		    labelStrings = fd.getLabels();
		    hasMean = false;
		    hasMedian = false;
		    findCentralTendancy(hasMean, hasMedian);
	        }
	        if (fd.getLevel() == FieldDescriptor.LEVEL_ORDINAL) {
		    labelStrings = fd.getLabels();
		    hasMean = false;
		    findCentralTendancy(hasMean, hasMedian);
	        }
	    }
	    else if (ev.getField().equals(freqField)) {
                barColor = fd.getColor();
		for (int i = 0; i < columns; i++) 
		    colors[i] = barColor;
	    }
	    repaint();
	}
	else {
	    try {
	        if (source.getFieldNames().length > 1)
	            setFields(source.getFieldNames()[0], 
		    source.getFieldNames()[1]);
	    }
	    catch (NoSourceException e) {
	        // WHAT TO DO HERE?
	        System.out.println("Barchart not drawn: No Source");
		okToDraw = false;
	        repaint();
	    }
        }
        super.tableViewChanged(event);
    }

    public void addSourceView(TableView source) throws TooManySourcesException {
		if (source == this)
			throw new TooManySourcesException("Duplicate Source");
		if (this.source != null)
			throw new TooManySourcesException();
		this.source = source;
		try {
			if (source.getFieldNames().length > 1) {
/*
                TableDescriptor td = source.getTableDescriptor();
                if (td.getLabelField() != null)
*/
	        setFields(source.getFieldNames()[0], source.getFieldNames()[1]);
            }
	}
	catch (NoSourceException e) {
	    // WHAT TO DO HERE?
	    System.out.println("Barchart not drawn: No Source");
	}
    }

    public void removeSourceView(TableView source) {
	if (this.source == source) {
	    this.source = null;
	    okToDraw = false;
	    repaint();
	    this.tableViewChanged(new TableViewEvent(this));
	}
    }

    public void setFields(String labelField, String freqField) {
	this.labelField = labelField;
	this.freqField = freqField;
	this.namedBins = new Vector();
        FieldDescriptor fd = null;
        maxLabelWidth = 0;
        max = 0;


    	try {
	    String[] flds = {labelField, freqField};
	    if (source.getRecordCount() < 1) {
		okToDraw = false;
		repaint();
		return;
	    }
	    Object[][] data = source.get(flds, 0);

            fd = source.getFieldDescriptor(freqField);
            barColor = fd.getColor();

            fd = source.getFieldDescriptor(labelField);

	    valueObjects = data[0];
	    freqObjects = data[1];

	    title = fd.getDescription(); 
            labelColor = fd.getColor();

	    if (title == null)
		title = labelField;

	    hasMean = true;
	    hasMedian = true;
	    if (fd.getLevel() == FieldDescriptor.LEVEL_NOMINAL) {
		labelStrings = fd.getLabels();
		hasMean = false;
		hasMedian = false;
	    }
	    if (fd.getLevel() == FieldDescriptor.LEVEL_ORDINAL) {
		labelStrings = fd.getLabels();
		hasMean = false;
	    }
	    okToDraw = true;
    	}
    	catch (Exception e) {
        	e.printStackTrace();
    	}
 
	
	titleFont = getFont(); 
	if (titleFont == null)
	    titleFont = new java.awt.Font("Courier", Font.BOLD, 12);

	titleFontMetrics = getFontMetrics(titleFont);

	columns = valueObjects.length;

	scale = 0.1;

	values = new int[columns];
	colors = new Color[columns];
	labels = new String[columns];
	abbrvs = new String[columns];
	styles = new int[columns];
	for (int i=0; i < columns; i++) {
	    // Get the frequency for this column

	    values[i] = ((Number)(freqObjects[i])).intValue();
	    if (values[i] > max)
		max = values[i];
	
	    // get the label for this column
	    if (labelStrings != null) {
	        labels[i] = (String)(labelStrings.get(valueObjects[i]));
                if (labels[i] == null)
		    labels[i] = valueObjects[i].toString();
	    }
            else
                labels[i] = valueObjects[i].toString();

	    maxLabelWidth = Math.max(titleFontMetrics.stringWidth((String)
		(labels[i])), maxLabelWidth);

	    // parse the bar style
		styles[i] = SOLID;
		//styles[i] = STRIPED;

            colors[i] = barColor;
	}
	findCentralTendancy(hasMean, hasMedian);
	repaint();
    }

    protected void findCentralTendancy(boolean hasMean, boolean hasMedian) {
	if (valueObjects != null) {
	    this.hasMean = hasMean;
	    this.hasMedian = hasMedian;
            int totalObservations = 0;   // Sum of all value[i]'s
	    double weightedSum    = 0.0; // Sum of all (value[i] * i)'s

	    modeColumns = new Vector();
	    medianColumn = -1;
	    meanColumn = -1;

	    for (int i = 0; i < namedBins.size(); ) {
		NamedBin nb = (NamedBin)namedBins.elementAt(i);
		if (nb instanceof MMMNamedBin)
		    namedBins.removeElementAt(i);
		else
		    i++;
	    }

            max = 0;
	    for (int i = 0; i < columns; i++) {
               if (values[i] > max)
		   max = values[i];
            }

	    for (int i = 0; i < columns; i++) {
		totalObservations += values[i];
		weightedSum += (values[i] * (i+1));
		if (values[i] == max)
		    modeColumns.addElement(new Integer(i));
	    }
	    insertNamedBin(namedBins, new MMMNamedBin(MODE_LABEL, 
		((Integer)modeColumns.elementAt(modeColumns.size()/2)
		).intValue(), this, modeColor));

	    int median = totalObservations/2;

	    //meanColumn = ((int)(weightedSum/(columns + 1))) - 1;
	    meanColumn = ((int)(weightedSum/totalObservations)) - 1;
	    if (meanColumn < 0)
		meanColumn = 0;
	    if (hasMean)
		insertNamedBin(namedBins, new MMMNamedBin(MEAN_LABEL, 
		    meanColumn, this, meanColor));

	    if (hasMedian) {
	        totalObservations = 0;
	        for (int i = 0; (i < columns) && (medianColumn < 0); i++) {
		    totalObservations += values[i];
		    if ((totalObservations >= median) && (medianColumn < 0))
		        medianColumn = i;
	        }
		insertNamedBin(namedBins, new MMMNamedBin(MEDIAN_LABEL, 
		    medianColumn, this, medianColor));
	    }
	}
    }

    protected void insertNamedBin(Vector bins, NamedBin nb) {
	int i = 0;
	for (i = 0; i < bins.size() && 
	    ((NamedBin)bins.elementAt(i)).column < nb.column; i++);
	bins.insertElementAt(nb, i);
    }

    public synchronized void update(Graphics appG) {
	int i, j;
	int cx, cy;

	int defaultBorder = 2; // At least 2 pixels on all sides
	int north = defaultBorder;
	int south = defaultBorder;
	int east = defaultBorder;
	int west = defaultBorder;

	// Points to which lines for mean, median, and mode should be drawn
	Point meanPoint = null;
	Point medianPoint = null;
	Vector modePoints = new Vector();

        int tickUnit = max/(numberOfTicks);
	int tickLabelWidth = 0;
	int tickMarkWidth = 3;
 

	// Clear the chart if there's nothing to draw
	if (! okToDraw && (max == 0)) {
	    appG.clearRect(0, 0, size().width, size().height);
	    return;
	}

	titleFont = getFont(); 
	titleFontMetrics = getFontMetrics(titleFont);
        int fontHeight = titleFontMetrics.getHeight();
        Dimension componentSize = size();

	// If we've been resized, then we may have to abbreviate labels
	// where we didn't before. These will be recalculated below
	if (lastWidth != componentSize.width)
	    hasAbbreviations = false;

	if ((offScreen == null) || 
	    (offScreen.getWidth(null) != componentSize.width) ||
	    (offScreen.getHeight(null) != componentSize.height))
	    offScreen = createImage(componentSize.width, componentSize.height);

	Graphics g = offScreen.getGraphics();
	g.setColor(getBackground());
	g.fillRect(0, 0, componentSize.width, componentSize.height);
	g.setFont(getFont());
	g.setColor(getForeground());

        switch (orientation) {
            case VERTICAL:
            default:
		if (showTicks) {
		    tickLabelWidth = titleFontMetrics.stringWidth("" + max);
		    west += tickLabelWidth + tickMarkWidth + 1;
		}

                barWidth = ((componentSize.width - west - east)/columns) - 
		    barSpacing;

		south += fontHeight; // For the title
		if (showLabels) {
		    // Adjust east and west for long labels at the ends
		    int lastLabelDiff = titleFontMetrics.stringWidth(
			(String)labels[columns-1]) - (barWidth + barSpacing);
		    if (lastLabelDiff > 0)
		        east += lastLabelDiff/2;
		    int firstLabelDiff = titleFontMetrics.stringWidth(
			(String)labels[0]) - (barWidth + barSpacing);
		    if (firstLabelDiff > 0)
		        west += firstLabelDiff/2;
                    // We've squeezed the margins, so re-calc the width
                    barWidth = ((componentSize.width - west - east)/columns) - 
		        barSpacing;

		    south += fontHeight; // For one row of labels

                    // PLI staggerLabels = barWidth < (maxLabelWidth - 2);
                    staggerLabels = false;

		    if (staggerLabels)
		        south += fontHeight; // For the second row of labels
		}
		if (namedBins != null && namedBins.size() > 0) {
		    north += fontHeight;

		    int minX = west; 
		    for (i = 0; i < namedBins.size(); i++) {
			NamedBin nb = (NamedBin)namedBins.elementAt(i);
			if (nb.isVisible()) {
		            int legWidth = 
				titleFontMetrics.stringWidth(nb.name);
			    int legX = Math.max(west + nb.column * 
			        (barWidth + barSpacing) + 
			        (barWidth - legWidth)/2, minX);
		            g.drawString(nb.name, legX,
		               fontHeight - titleFontMetrics.getDescent()); 
			    nb.setLegendPoint( new Point(
			        legX + legWidth/2, fontHeight));
			    nb.setColumnPoint( new Point(west +
				(int)(((double)nb.getColumn() + (double)(0.5)) *
		                (double)(barWidth + barSpacing)),
               			 getBaseLine() - (int)(values[nb.getColumn()] 
				* scale)));

			    minX = legX + legWidth;
			}
		    }
		}

		// if (showFrequencies)  // Always leave room for freqs
		north += fontHeight;

            	scale = 
		   (double)(componentSize.height - north - south)/(double)(max);

		baseLine = componentSize.height - south;

		if (showTicks) {
		    g.setColor(getForeground());
		    for (int t = 1; t <= numberOfTicks; t++) {
			int ty = (int)(baseLine - scale * (t * tickUnit)) + 1;
			String tickLabel = Integer.toString(t * tickUnit);
			g.drawString(tickLabel, defaultBorder + tickLabelWidth
			    - titleFontMetrics.stringWidth(tickLabel) - 2, 
			    ty + (fontHeight - titleFontMetrics.getDescent())/2 
			    - 2);
			g.drawLine(tickLabelWidth + 1, ty, 
			    tickLabelWidth + 1 + tickMarkWidth, ty);
		    }
		}

		firstBar = west;
		    
	    break;

            case HORIZONTAL:

		if (showLabels)
		    west += maxLabelWidth;

		south += fontHeight; // For the title

		if (showMeanMedianMode) {
/*
		    int mmmLegendSize = 0;

		    if (hasMedian) {
		        mmmLegendSize = titleFontMetrics.stringWidth("Median");
		        g.drawString("Median", 
			    (componentSize.width - mmmLegendSize),
			    2 * (componentSize.height/4));
			medianLegendPoint = new Point(
			    (componentSize.width - mmmLegendSize),
			    2 * (componentSize.height/4) - fontHeight/2);
		    }
		    if (hasMean) {
			if (mmmLegendSize == 0)
		            mmmLegendSize = 
			    	Math.max(titleFontMetrics.stringWidth("Mean"),
			     	titleFontMetrics.stringWidth("Mode"));
		        g.drawString("Mean", 
			    (componentSize.width - mmmLegendSize),
			    (componentSize.height/4));
			meanLegendPoint = new Point(
			    (componentSize.width - mmmLegendSize),
			    (componentSize.height/4) - fontHeight/2);
		    }
		    if (mmmLegendSize == 0)
		        mmmLegendSize = titleFontMetrics.stringWidth("Mode");
		    g.drawString("Mode", 
			(componentSize.width - mmmLegendSize),
			3 * (componentSize.height/4));
		    modeLegendPoint = new Point(
			(componentSize.width - mmmLegendSize),
			3 * (componentSize.height/4) - fontHeight/2);

		    east += mmmLegendSize;
*/
		}

		if (showFrequencies) 
		    east += titleFontMetrics.stringWidth("" + max);

		if (showTicks)
		    north += 0; // FIX THIS

            	scale = 
		   (double)(componentSize.width - east - west)/(double)(max);

		baseLine = west;

                barWidth = ((componentSize.height - north - south)/columns) - 
		    barSpacing;
		if ((showFrequencies || showLabels) && 
		    ((barWidth + barSpacing) < fontHeight))
		    barWidth = fontHeight - barSpacing;

		firstBar = north;

	    break;
	}

	// draw the title centered at the bottom of the bar graph
	g.setColor(labelColor);
	i = titleFontMetrics.stringWidth(title);
	g.setFont(titleFont);
	g.drawString(title, Math.max((componentSize.width - i)/2, 0),
		     componentSize.height - titleFontMetrics.getDescent()); 
	for (i=0; i < columns; i++) {

	    switch (orientation) {
	      case VERTICAL:
	      default:

		// set the next X coordinate 
		cx = i * (barWidth + barSpacing) + west;
		
		// set the next Y coordinate 
		cy = componentSize.height - south - (int)(values[i] * scale);

		// draw the label
		if (showLabels) {
	            g.setColor(labelColor);
		    String label = labels[i]; 
		    String abbrv = ".";
		    if (lastWidth == componentSize.width && abbrvs[i] != null)
			abbrv = abbrvs[i];
		    else if (titleFontMetrics.stringWidth(label) < barWidth)
			abbrv = label;
		    else {
			hasAbbreviations = true;
			if (label.length() > 4) {
			    int letters = 0;
			    int labelLen = label.length();
			    String lastAbbrev = null;
			    do {
				lastAbbrev = abbrv;
			    	abbrv = label.substring(0, letters) + ".." + 
				    label.substring(labelLen - letters);
			    	letters++;
			    }
		            while (titleFontMetrics.stringWidth(abbrv) < 
				barWidth - 2 && letters < 3); 
			    letters--;
			    abbrv = lastAbbrev;
			}
		    }
		    abbrvs[i] = abbrv;
		    g.drawString(abbrv, cx +
			(barWidth + barSpacing)/2 -
			titleFontMetrics.stringWidth(abbrv)/2,
			componentSize.height - fontHeight);
		    
 		}

		cx += barSpacing/2; // Center the bar

		// draw the shadow bar
	        if (drawShadow) {
		    if (colors[i] == Color.black) 
		        g.setColor(Color.gray);
		    g.fillRect(cx + 5, cy - 3, barWidth,  
			(int)(values[i] * scale));
		}

		// draw the bar with the specified color
		g.setColor((Color)(colors[i]));
		switch (styles[i]) {
		  case SOLID:
		  default:
		    if (outlineOnly) {
			g.setColor(getBackground());
		        g.fillRect(cx, cy, barWidth, (int)(values[i] * scale));
			g.setColor((Color)(colors[i]));
		        g.drawRect(cx, cy, barWidth, (int)(values[i] * scale));
		    }
		    else if ((int)(values[i] * scale) == 0 )
			g.drawRect(cx, cy-1, barWidth, 0);
		    else
		        g.fillRect(cx, cy, barWidth, (int)(values[i] * scale));
		    
		    break;
		  case STRIPED:
		    {
			int steps = (int)(values[i] * scale) / 2;
			int ys;

			for (j=0; j < steps; j++) {
			    ys = cy + (2 * j);
			    g.drawLine(cx, ys, cx + barWidth, ys);
			}
		    }
		    break;
		}
		if (showFrequencies && (values[i] != 0)) {
	            g.setColor(labelColor);
		    g.drawString("" + values[i], cx,
			     cy - titleFontMetrics.getDescent());
		}

		// Indicate mean, median, and mode if appropriate
		// We draw these after the bars to prevent bars from
		// being drawn over them.
                if (showMeanMedianMode) {
		    int ypos = cy; // + (baseLine - cy)/10;
		    if (hasMean && (i == meanColumn)) 
			meanPoint = new Point(cx + barWidth/2, ypos);
		    if (hasMedian && (i == medianColumn)) 
			medianPoint = new Point(cx + barWidth/2, ypos);
		    if (modeColumns.contains(new Integer(i))) 
			modePoints.addElement(new Point(cx + barWidth/2, ypos));
                }
		break;
	      case HORIZONTAL:
		// set the Y coordinate
		cy = ((barWidth + barSpacing) * i) + north;
			       
		if (showLabels) {
		    // set the X coordinate to be the size().width of the widest
		    // label 
		    cx = maxLabelWidth + 1;

		    cx += Math.max((size().width - (maxLabelWidth + 1 +
					 titleFontMetrics.stringWidth("" +
				         max) + (int)(max * scale))) / 2, 0);

		    // draw the labels and the shadow
	            g.setColor(labelColor);
		    g.drawString((String)labels[i], cx - maxLabelWidth - 1,
			         cy + titleFontMetrics.getAscent());
		}
		else
		    cx = 2;

		cy += barSpacing/2; // Center the bar
	        if (drawShadow) {
		    if (colors[i] == Color.black) {
		        g.setColor(Color.gray);
		    }
		    g.fillRect(cx + 3,
			   cy + 5, (int)(values[i] * scale),
			   barWidth);
		}

		// draw the bar in the current color
		g.setColor((Color)(colors[i]));
		switch (styles[i]) {
		  case SOLID:
		  default:
		    if (outlineOnly) {
			g.setColor(getBackground());
		        g.fillRect(cx, cy, (int)(values[i] * scale), barWidth);
			g.setColor((Color)(colors[i]));
		        g.drawRect(cx, cy, (int)(values[i] * scale), barWidth);
		    }
		    else if ((int)(values[i] * scale) == 0 )
			g.drawRect(cx+1, cy, 0, barWidth);
		    else
		        g.fillRect(cx, cy, (int)(values[i] * scale), barWidth);
		    break;
		  case STRIPED:
		    {
			int steps = (int)(values[i] * scale) / 2;
			int ys;

			for (j=0; j < steps; j++) {
			    ys = cx + (2 * j);
			    g.drawLine(ys, cy, ys, cy + barWidth);
			}
		    }
		    break;
		}

		// Indicate mean, median, and mode if appropriate
		// We draw these after the bars to prevent bars from
		// being drawn over them.
                if (showMeanMedianMode) {
		    int xpos = 5 * (baseLine + (int)(values[i] * scale))/6;
		    if (hasMean && (i == meanColumn)) 
			meanPoint = new Point(xpos, 
			  (i + 1) * (barWidth + barSpacing) - 
				(barWidth+ barSpacing)/2);
		    if (hasMedian && (i == medianColumn)) 
			medianPoint = new Point(xpos, 
			  (i + 1) * (barWidth + barSpacing) - 
				(barWidth+ barSpacing)/2);
		    if (modeColumns.contains(new Integer(i))) 
			modePoints.addElement(new Point(xpos, 
			  (i + 1) * (barWidth + barSpacing)  -
				(barWidth + barSpacing)/2));
                }
		if (showFrequencies) {
	            g.setColor(labelColor);
		    g.drawString("" + values[i],
			     cx + (int)(values[i] * scale) + 3,
			     cy + titleFontMetrics.getAscent());
		}     
		break;
	    }
	}

	// Expand the abbreviation of the column that the mouse is
	// over.
        if ((orientation == VERTICAL) && hasAbbreviations && 
	    (mouseColumn > -1) && (mouseColumn < columns) && showLabels && 
	    !(abbrvs[mouseColumn].equals(labels[mouseColumn]))) {
	    cx = west + mouseColumn * (barWidth + barSpacing) +
                (barWidth + barSpacing)/2 -
                titleFontMetrics.stringWidth((String)labels[mouseColumn])/2;
	    cy = componentSize.height - fontHeight;
	    g.clearRect(cx - 10, cy - fontHeight,
		titleFontMetrics.stringWidth((String)labels[mouseColumn]) + 20,
		fontHeight + 2);
	    g.setColor(Color.darkGray);
	    g.drawString((String)labels[mouseColumn], cx, cy);
	    g.drawLine(cx, cy - fontHeight + 3, 
		west + mouseColumn * (barWidth + barSpacing), baseLine);
	    g.drawLine(cx + titleFontMetrics.stringWidth(
		labels[mouseColumn]), cy - fontHeight + 3, 
		west + mouseColumn * (barWidth + barSpacing) + barWidth, 
		baseLine);
	    g.setColor(Color.black);
	}

	// Draw mean, median, and mode lines
	if (namedBins != null && namedBins.size() > 0) {
	    for (i = 0; i < namedBins.size(); i++) {
		NamedBin nb = (NamedBin)namedBins.elementAt(i);
		if ((nb instanceof MMMNamedBin) && 
		    (nb.getName().equals(MODE_LABEL)))
		    ((MMMNamedBin)nb).setPointVector(modePoints);
		nb.drawArrowLine(g, barWidth-2);
	    }
	}

	appG.drawImage(offScreen, 0, 0, null);
	lastWidth = componentSize.width;
    }

    public synchronized void paint(Graphics appG) {
	if (! okToDraw && (max == 0))
	    appG.clearRect(0, 0, size().width, size().height);
	//else if ((offScreen == null) || 
	//    (offScreen.getWidth(null) != getSize().width) ||
	//    (offScreen.getHeight(null) != getSize().height))
	    update(appG);
	//else
	  //  appG.drawImage(offScreen, 0, 0, null);
	super.paint(appG);
    }

    protected int getBaseLine() {
	return baseLine;
    }

    protected int getFirstBar() {
	return firstBar;
    }

/* What we want to do is only enforce a min size if we're showing labels
    public Dimension getMinimumSize() {
	if (!showLabels) 
	    return super.getMinimumSize();
	else {
	}
    }
*/

    public void print(Graphics g) {
        super.paint(g);
        paint(g);
    }

    /*                   MouseListener methods                 */

    protected int getColumn(MouseEvent e) {
	int col = -1;
	if (barWidth + barSpacing > 0) {
	    if (orientation == HORIZONTAL)
		col = (e.getY() - firstBar)/(barWidth + barSpacing);
	    else
		col = (e.getX() - firstBar)/(barWidth + barSpacing);
	}
	if (col >= columns)
	   col = -1;
	return col;
    }

    PopupMenu myPopup = null;
    public void mousePressed(MouseEvent e) {
	int col = getColumn(e);
	if ((col >= 0) && (col < columns)) {
	    myPopup = new PopupMenu();
	    if (((String)(labels[col])).length() > 0)
	        myPopup.add((String)labels[col]);
	    myPopup.add("Frequency: " + values[col]);
	    if (modeColumns.contains(new Integer(col))) {
	        if (modeColumns.size() == 1) 
	            myPopup.add("This value is the mode");
		else
	            myPopup.add("This value is a mode");
	    }
	    if ((col == medianColumn) && hasMedian)
	        myPopup.add("This value is the median");
	    if ((col == meanColumn) && hasMean)
	    //if ((col == meanColumn) )
	        myPopup.add("This value is the mean");
	    add(myPopup);
	    myPopup.show(this, e.getX() + 5, e.getY() + 5);
	}

    }

    public void mouseReleased(MouseEvent e) {
	if (myPopup != null) {
	    remove(myPopup);
	    myPopup = null;
	}
    }

    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) { }
    public void mouseExited(MouseEvent e) {
	// Remove label abbreviation 
	mouseColumn = -1;
	if (hasAbbreviations)
	    repaint();
    }

    /*                 MouseMotionListener Methods            */
    void expandAbbreviation(MouseEvent e) {
	int col = getColumn(e);
	if (hasAbbreviations && (col != mouseColumn)) {
	    mouseColumn = col;
	    repaint();
	}
    }

    public void mouseMoved(MouseEvent e) {
	expandAbbreviation(e);
    }
    public void mouseDragged(MouseEvent e) {
	expandAbbreviation(e);
    }


    /*                   TableView methods                 */

    /* Do we need to override table()? Only if we've edited the data? */

    public synchronized String[] getFieldNames() throws NoSourceException {
	TableView s = outputView(); // See if we have a source?
	String[] flds = {labelField, freqField};
	return flds;
    }

    public synchronized int getRecordCount() throws NoSourceException {
	TableView s = outputView(); // See if we have a source?
	return values.length;
    }
    
    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from
     * the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt,
        int numberOfRecords) throws InterchangeException {
	TableView s = outputView(); // See if we have a source?

        if ((startingAt < 0) || (startingAt >= values.length)) 
	    throw new NoSuchRecordException(startingAt);
	if ((numberOfRecords + startingAt) > values.length) 
	    throw new NoSuchRecordException(values.length);
        else {
	    Object[][] result = new Object[fields.length][(int)numberOfRecords];
	    for (int f = 0; f < fields.length; f++) {
		if (fields[f].equals(labelField))
		    for (int r = 0; r < numberOfRecords; r++)
			result[f][(int)(r + startingAt)] = 
			    labels[(int)(r + startingAt)];
		else if (fields[f].equals(freqField))
		    for (int r = 0; r < numberOfRecords; r++)
			result[f][(int)(r + startingAt)] = 
			    new Integer(values[(int)(r + startingAt)]);
		else
	    	    throw new sieve.sluice.NoSuchFieldException(fields[f]);
            }
	    return result;
        }
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt)
        throws InterchangeException {
        return get(fields, startingAt, getRecordCount());
    }

}

// Record-ish class for tracking mean, median, and mode bins.
class MMMNamedBin extends NamedBin {
    BarChart bc = null;
    Vector points = null;

    public MMMNamedBin(String name, int column, BarChart bc, Color c) {
	super(name, column, c);
	this.bc = bc;
    }
    public boolean isVisible() {
	return bc.getShowMeanMedianMode();
    }

    public void setPointVector(Vector v) {
	points = v;
    }

    public void drawArrowLine(Graphics g, int maxHeadWidth) {
	if (points == null)
	    super.drawArrowLine(g, maxHeadWidth);
	else if (lp != null && isVisible()) {
            g.setColor(color);
	    for (Enumeration e = points.elements(); e.hasMoreElements();) {
                Arrow.paintArrow(g, lp, (Point)(e.nextElement()), 20, 
		Math.min(7, maxHeadWidth));
	    }
        }
    }

}


