package sieve.nuggets;

import java.awt.event.*;
import java.awt.*;
import java.applet.Applet;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import sieve.util.SwingButtonGroupBox;
import sieve.sluice.*;
import java.util.Enumeration;
import sieve.sluice.codebook.CodebookFinder;
import java.beans.*;
											
/**
 * EditableRegressionPlot extends JComponent and contains an EditablePlot which
 * extends RegressionPlot. It also includes a "control panel" JPanel with
 * controls for selecting a field in the plot and switching between moving and 
 * deleting mode. EditableRegressionPlot does NOT propagate move/delete changes
 * to downstream view because the plot data is stored in integer arrays rather than
 * in the TableView itself (had to duplicate x-values for each y-variable). 
 * Will probably be combined with some additional functionality to allow user to 
 * save changes to create a new table. 
 * 
 * @see sieve.sluice.TableView
 * @see sieve.nuggets.TableViewJComponent
 * @see sieve.nuggets.RegressionPlot
 * @see sieve.nuggets.EditablePlot
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */

public class EditableRegressionPlot extends JComponent implements TableView, TableViewListener {

	private EditablePlot				plot = null;
	private SwingButtonGroupBox			modeRadio = null;
	private SwingButtonGroupBox			fieldLabelRadio = null;
	private Color[]						colors = null;
	private Color						bgCol = Color.white;
	private JPanel						radioPan = new JPanel();
	
	public EditableRegressionPlot() {
		setBackground(bgCol);
		setBorder(new LineBorder(Color.black,1));
		plot = new EditablePlot();
		plot.setBackground(bgCol);
		colors = plot.getColorList();

		String[] coms = {plot.moveStr, plot.delStr};
		modeRadio = new SwingButtonGroupBox(BoxLayout.Y_AXIS,2,coms,0);
		modeRadio.addActionListener(plot);
		modeRadio.setBackground(bgCol);

		radioPan.setLayout(new FlowLayout(FlowLayout.LEFT, 10,15));
		radioPan.setMinimumSize(new Dimension(300,300));
		radioPan.setPreferredSize(new Dimension(300,300));
		radioPan.setBackground(bgCol);
		radioPan.add(modeRadio);

		setLayout(new BorderLayout());
		makeFieldRadio();	
		radioPan.add(fieldLabelRadio);
		//does as with modeRadio, but made separate method to respond to table changes
		//Can't add anything to layout after the field radios, since they 
		//are removed and added back in tableViewChanged()	
		add("West",plot);
		add("East",radioPan);
	}

	private void makeFieldRadio() {
		String[] names = plot.getYLabels();
		fieldLabelRadio = new SwingButtonGroupBox(BoxLayout.Y_AXIS,names.length,names,0);
		fieldLabelRadio.addActionListener(plot);
		fieldLabelRadio.setBackground(bgCol);
		int c = 0;
		Enumeration e = fieldLabelRadio.getElements();
		if (e != null) {
			while(e.hasMoreElements()) {
				if(c < colors.length)
					((Component)(e.nextElement())).setForeground(colors[c]);
				else
					((Component)(e.nextElement())).setForeground(Color.black);
				c++;
			}
		}
	}

	//********* visible PROPERTY ************************************

	public int getMaxRecords() {
		return plot.getMaxRecords();
	}
	public void setMaxRecords(int toPlot) {
		plot.setMaxRecords(toPlot);	//this will also fire propertyChange since
		repaint();					//...listeners have been registered to plot
	}			



	public void addPropertyChangeListener(PropertyChangeListener p) {
		plot.addPropertyChangeListener(p);
	}
		
	public void removePropertyChangeListener(PropertyChangeListener p) {
		plot.removePropertyChangeListener(p);
	}


	public void tableViewChanged(TableViewEvent t) {
		plot.tableViewChanged(t);
		if(fieldLabelRadio != null) {
			fieldLabelRadio.removeActionListener(plot);
			radioPan.remove(fieldLabelRadio);
		}
		makeFieldRadio();
		radioPan.add(fieldLabelRadio);
		validate();
	}


	/**
	 * TableView interface methods, use the EditableRegressionPlot
	 */
	public void addSourceView(TableView tv) throws TooManySourcesException {
		try{plot.addSourceView(tv);}
		catch(TooManySourcesException e) {throw e;}
		if(fieldLabelRadio != null) {
			fieldLabelRadio.removeActionListener(plot);
			radioPan.remove(fieldLabelRadio);
		}
		makeFieldRadio();
		radioPan.add(fieldLabelRadio);
		validate();
	}

	public void addTableViewListener(TableViewListener tl) {
		plot.addTableViewListener(tl);
	}

	public Object[][] get(String[] f, int n) throws InterchangeException {
		Object[][] gotten = null;
		try { gotten = plot.get(f,n);}
		catch(InterchangeException e) {throw e;}
		return gotten;
	}

	public Object[][] get(String[] f, int n, int s) throws InterchangeException {
		Object[][] gotten = null;
		try { gotten = plot.get(f,n,s);}
		catch(InterchangeException e) {throw e;}
		return gotten;
	}

	public FieldDescriptor getFieldDescriptor(String s) throws InterchangeException {
		FieldDescriptor fd = null;
		try{ fd = plot.getFieldDescriptor(s);}
		catch(InterchangeException e) {throw e;}
		return fd;
	}

	public String[] getFieldNames() throws NoSourceException {
		String[] nms = null;
		try {nms = plot.getFieldNames();}
		catch(NoSourceException e) {throw e;}
		return nms;
	}

	public int getRecordCount() throws NoSourceException {
		int num = 0;
		try {num = plot.getRecordCount();}
		catch(NoSourceException e) {throw e;}
		return num;
	}

	public TableDescriptor getTableDescriptor() throws NoSourceException {
		TableDescriptor td = null;
		try {td = plot.getTableDescriptor();}
		catch(NoSourceException e) {throw e;}
		return td;
	}

	public void removeSourceView(TableView t) {
		plot.removeSourceView(t);
		if(fieldLabelRadio != null) {
			fieldLabelRadio.removeActionListener(plot);
			radioPan.remove(fieldLabelRadio);
		}
		//makeFieldRadio();
		//radioPan.add(fieldLabelRadio);
		validate();
	}

	public void removeTableViewListener(TableViewListener l) {
		plot.removeTableViewListener(l);
	}

	public TableView table() throws NoSourceException {
		TableView tv = null;
		try {tv = plot.table();}
		catch(NoSourceException e) {throw e;}
		return tv;
	}

	public Dimension getPreferredSize() {
		return new Dimension(600,300);
    }
    
    public Dimension getMinimumSize() {
		return new Dimension(600,300);
    }
	
	public Dimension preferredSize() {
		return getPreferredSize();
    }
    
    public Dimension minimumSize() {
		return getMinimumSize();
    }

}

