package sieve.nuggets;
 
import java.awt.*;
import java.applet.*;
import java.io.*;
import java.util.Vector;
import java.text.*;

import sieve.sluice.*;
import sieve.sluice.views.*;

public class Histogram extends BarChart {
    int            bins       = 0;
    boolean	   wrongLevel = false;

    public Histogram() {
	orientation = VERTICAL;
	barSpacing = 0;	
	drawShadow = false;
	showFrequencies = false;
	outlineOnly = true;
	setName("Histogram");
    }

    public FieldDescriptor getFieldDescriptor(String field) 
	throws InterchangeException {
	FieldDescriptor fd = new FieldDescriptor(field);
	fd.copy(super.getFieldDescriptor(field));

	if (okToDraw && !wrongLevel && field.equals(labelField) &&
	    (bins < source.getRecordCount())) {
	    fd.setType("java.lang.String");
	    fd.setLevel(FieldDescriptor.LEVEL_NOMINAL);
	    fd.setAttributes(null);
	}
	return fd;
    }

    public void setFields(String labelField, String freqField) {
		this.labelField = labelField;
        this.freqField = freqField;
        FieldDescriptor fd = null;
        maxLabelWidth = 0;
        max = 0;

        try { 
			wrongLevel = true;
            String[] flds = {labelField, freqField};
			if (source == null)
				System.out.println("null source in Histogram.setFields");
			else
			if (source.getRecordCount() < 1) {  /*here*/
				okToDraw = false;
				repaint();
				return;
			}
            Object[][] data = new NumberView(new LabelView(source)).get(flds, 0);

	    // PLI if ((bins <= 0) || (bins > data[0].length))
	    //    bins = data[0].length;
		    DescriptiveView sv = null;
		    try {
		        sv = new DescriptiveView(source, labelField);
		        bins = ((Number) sv.getMax()).intValue() + 1;
			}
			catch (InterchangeException e) { e.printStackTrace();}
 
            fd = source.getFieldDescriptor(freqField);
            barColor = fd.getColor();
 
            fd = source.getFieldDescriptor(labelField);
 
            valueObjects = data[0];
            freqObjects = data[1];
 
            title = fd.getDescription(); //source.fieldDescription(labelField);
            labelColor = fd.getColor();
 
            if (title == null)
                title = labelField;
 
            okToDraw = true;
		    wrongLevel = false;
        }
        catch (sieve.sluice.NoSuchFieldException e) {
			super.setFields(labelField, freqField);
			return;
        }
        catch (Exception ex) {
		    okToDraw = false;
            ex.printStackTrace();
		    return;
        }
 
        titleFont = getFont(); 
		if (titleFont == null)
			titleFont = new java.awt.Font("Courier", Font.BOLD, 12);
        titleFontMetrics = getFontMetrics(titleFont);
		setNumberOfBins(bins);
    }

    public int getNumberOfBins() {
	return bins;
    }
 
    public void setNumberOfBins(int bins) {
	if (wrongLevel)
	    return;

// Should really check for bins > max - min
//        if ((bins <= 0) ||
//	    ((valueObjects != null) && (bins > valueObjects.length)))
//	    return;
	if (bins <= 0) 
	    return;

	this.bins = bins;

	// If we aren't connected we'll still let them set the number of bins
	if (valueObjects == null) 
	    return;

        columns = bins;

	DescriptiveView sv = null;
	Number minVal = null;
	Number maxVal = null;
        Vector valuesPresent = null;
	try {
	    sv = new DescriptiveView(source, labelField);
	    minVal = (Number) sv.getMin();
	    maxVal = (Number) sv.getMax();
	    valuesPresent = sv.getValues();
	}
	catch (InterchangeException e) {
	    e.printStackTrace();
	}
	max = 0;

        boolean isInteger = (minVal instanceof java.lang.Integer) ||
            (minVal instanceof java.lang.Long);
	DecimalFormat format = (DecimalFormat) NumberFormat.getInstance();

	if (! isInteger) {
	    format.applyPattern("####0.#####");
	}

        double step;
	double min = minVal.doubleValue();
	if (min < 0) {
	   step = ( 2 * maxVal.doubleValue() )/bins;
	   min = - maxVal.doubleValue();
	}
	else { // Need to check abs(min), cases where both are negative...
	   step = (maxVal.doubleValue())/bins;
	   min = 0;
	}
 
        values = new int[columns];
        colors = new Color[columns];
        labels = new String[columns];
        abbrvs = new String[columns];
        styles = new int[columns];

	boolean foundValue = false;
	int	nextInVector = 0;
        double  bottomOfBin = min;
        for (int i=0; i < columns; i++) {

            values[i] = 0;

	    double topOfBin = (min + ((i+1) * step));
	    if (isInteger) 
		topOfBin = ((int)topOfBin);

	    while ((nextInVector < valuesPresent.size()) && 
		((Number)valuesPresent.elementAt(nextInVector)).
		    doubleValue() <= topOfBin) { //(min + ((i+1) * step))) {

		    foundValue = false;
		    for (int j = 0; (j < freqObjects.length) && !foundValue; 
			j++){

			if (valueObjects[j].equals(valuesPresent.elementAt(
			     nextInVector))) {
                            values[i] += ((Number)(freqObjects[j])).intValue();
			    foundValue = true;
			}
		    }
		    nextInVector++;
	    }

            if (values[i] > max)
                max = values[i];
 
	    if (topOfBin == bottomOfBin)
                labels[i] = 
		    format.format(bottomOfBin, new StringBuffer(),
		    new FieldPosition(0)).toString();
	    else
                labels[i] = 
		    format.format(bottomOfBin, new StringBuffer(),
		    new FieldPosition(0)).toString() 
		    + " - " +
                    format.format(topOfBin, new StringBuffer(),
		    new FieldPosition(0)).toString();

            if (isInteger)
		bottomOfBin = topOfBin + 1;
	    else
		bottomOfBin = ChoiceFormat.nextDouble(topOfBin);

            maxLabelWidth = Math.max(maxLabelWidth, 
	        titleFontMetrics.stringWidth((String)labels[i]));

            styles[i] = SOLID;
            colors[i] = barColor;
        }

/*
	if (min == 0)
	    labels[0] = "0";
	else
	    labels[0] = "- " + maxVal.toString();
	labels[columns-1] = maxVal.toString();
*/

	// If we're a histogram (and not acting like a bar chart) then
	// we have to have a mean and median.
	findCentralTendancy(true, true);
        repaint();
	eventHandler.fireTableViewEvent(new TableViewEvent(this));
    }
    
}

