package sieve.nuggets.LineGraph;


import java.awt.*;
import java.util.*;
import sieve.sluice.*;
import sieve.sluice.event.*;

public class SimulatedSensor extends Panel implements TableView {
  TableViewEventSupport	eventHandler	= null;


  private static int numFields = 1;
  TableDescriptor tableDescriptor = null;
  String fieldNames[] = new String[numFields];
  Hashtable fieldDescriptors = new Hashtable();

  Vector records = new Vector();

  Button start = new Button("Start");
  Button stop = new Button("Stop");

  public SimulatedSensor() {
    this.eventHandler = new TableViewEventSupport();

    //    this.sensorTable = new SensorTable("Run 1");
    // Setup the interface
    stop.disable();
    setLayout(new FlowLayout());
    add(start);
    add(stop);


    // Setup the table
    tableDescriptor = new TableDescriptor();
    tableDescriptor.setLabelField("Simulated Sensor");

    fieldNames[0] = "TableViews";

    fieldDescriptors.put("TableViews",  new FieldDescriptor("TableViews"));
  }

  public void appendRecord(Object[] record) {
    records.addElement(record);

    debug("Appended Record: "+record[0]);
    Object newRecords[][] = new Object[1][];
    newRecords[0] = record;
    eventHandler.fireTableViewEvent(new AppendEvent(this, newRecords));
  }

  int runCounter = 0;
  SensorTable sensorTable = null;

  public boolean action(Event evt, Object what) {
    if (evt.target == start) {
      stop.enable();
      start.disable();
      sensorTable = new SensorTable("Run "+runCounter++);
      sensorTable.start();
      Object record[] = new Object[1];
      record[0] = sensorTable;
      appendRecord(record);
      return true;
    }
    else if (evt.target == stop) {
      start.enable();
      stop.disable();
      sensorTable.stop();
      return true;
    }
    return false;
  }

  //------ TableView interface -----------------
  public TableView table() throws NoSourceException {
    return this;
  }

    /**
     * Returns the TableDescriptor for this view's representation of
     * the table.
     */
  public TableDescriptor getTableDescriptor() throws NoSourceException {
    return tableDescriptor;
  }

    /**
     * Returns the names of fields in this view. These names should be
     * used to reference fields in the <CODE>get</CODE> methods.
     */
  public String[] getFieldNames() throws NoSourceException {
    return fieldNames;
  }
    /**
     * Returns the FieldDescriptor for the specified field.
     */
    public FieldDescriptor getFieldDescriptor(String field) 
      throws InterchangeException {
	return (FieldDescriptor)fieldDescriptors.get(field);
    }

    /**
     * Returns number of records in the view.
     */
  public int getRecordCount() throws NoSourceException {
    return records.size();
  }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>. The resulting array
     * is ordered by <CODE>[field][record]</CODE>.
     */
    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>. The resulting array
     * is ordered by <CODE>[field][record]</CODE>.
     */
    public Object[][] get(String[] fields, 
			  int startingAt, 
			  int numberOfRecords) 
      throws InterchangeException {

	debug("get: startingAt="+startingAt+" numberOfRecords="+numberOfRecords+
	      " numFields="+fields.length);

	Object returnRecords[][] = new Object[(int)numberOfRecords][fields.length];

	int fieldIndeces[] = new int[fields.length];

	for (int i = 0; i<fields.length; i++) {
	  for (int j = 0; j<fieldNames.length; j++) {
	    debug("i = "+i+" j = "+j+" fieldNames[j] = "+fieldNames[j]);
	    if (fieldNames[j].equals(fields[i])) {
		fieldIndeces[i] = j;
		break;
	    }
	  }
	}

	// see if we have  the requested records
	if ((records.size()) < (startingAt+numberOfRecords))
	  throw new InterchangeException();

	for (int i = 0; i<numberOfRecords; i++) {
	  Object[] oneRecord = (Object[])records.elementAt((int)startingAt+i);
	  for (int j = 0; j<fieldIndeces.length; j++ ) {
	    returnRecords[i][j] = oneRecord[fieldIndeces[j]];
	  }
	}
	return returnRecords;
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>. (I still don't
     * like this because it doesn't add anything. _Every_ implementation
     * will almost certainly just have a call to the three-parm get...)
     */
  public Object[][] get(String[] fields, int startingAt) 
    throws InterchangeException {
      return get(fields, startingAt, records.size()-startingAt);
  }

   
    /* -------------  Event-related methods ------------- */
    public void addTableViewListener(TableViewListener listener) {
	eventHandler.addTableViewListener(listener);
    }

    public void removeTableViewListener(TableViewListener listener) {
	eventHandler.removeTableViewListener(listener);
    }
    public void tableViewChanged(TableViewEvent event) {
      debug("Got tableViewChanged in : " + this);
	eventHandler.fireTableViewEvent(event);
    }

    /* -------------  Methods for link support ------------- */
  public void addSourceView(TableView source) throws TooManySourcesException {
    // no sources allowed for this table
    throw new TooManySourcesException();
  }
  public void removeSourceView(TableView source) {
  }

  //------ TableView interface -----------------



  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}
