package sieve.nuggets;
 
import java.awt.*;
import java.awt.event.*;
import sieve.sluice.*;
import java.util.*;

public class MedianHistogram extends Histogram implements MouseListener {
    int xCoord = 0;
    int totalObservations = 0;
    int observationsBelowLine = 0;
    boolean onTarget = false;
    boolean mouseIsDown = false;
    Object targetValue = null;

    public MedianHistogram() {
	super();
    }

    protected String getTargetName() {
	return "Median";
    }

    public void setShowMeanMedianMode(boolean set) {
	showMeanMedianMode = false;
    }

    int sumColumns(int start, int finish) {
	int sum = 0;
	for (int i = start; i <= finish; i++)
	    sum += values[i];
	return sum;
    }

	public void superSetFields(String labelField, String freqField) {
	super.setFields(labelField, freqField);
	}


    public void setFields(String labelField, String freqField) {
	super.setFields(labelField, freqField);
	xCoord = 0;
	totalObservations = sumColumns(0, columns - 1);
	observationsBelowLine = 0;
	onTarget = false;
	repaint();
    }

    public void tableViewChanged(TableViewEvent event) {
	super.tableViewChanged(event);
	xCoord = 0;
    }

    /**
     * Overriding to force mousePressed to be ignored.
     */
    public void mousePressed(MouseEvent e) { 
	mouseIsDown = true;
	repaint();
    }

    public void mouseReleased(MouseEvent e) {
	mouseIsDown = false;
	repaint();
    }

    public void superMouseDragged(MouseEvent e) {
	/**
	 * needed to override this class's mouseDragged method, but still access super.mouseDragged(Event).
	 * This is a cheap way to essentially allow the subclass (PercentileHistogram) to do
	 * super.super.mouseDragged(event) and bypass this class's method.
	 */
		super.mouseDragged(e);
	}

    public void mouseDragged(MouseEvent e) {
	super.mouseDragged(e);
	if (getCursor().equals(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR))){
	    int col = getColumn(e);
	    if (col >= 0) { 
	        xCoord = e.getX();
		if (xCoord < getFirstBar())
		    xCoord = getFirstBar();
		observationsBelowLine = sumColumns(0, col - 1);
		double horScale = ((double)values[col])/
		    ((double)(barWidth + barSpacing));
		observationsBelowLine += (int)(horScale * 
		    ((xCoord - getFirstBar()) - (col  * 
		    (barWidth + barSpacing))));
		int mid = totalObservations/2;
		if (Math.abs(mid - observationsBelowLine) < horScale) {
		    observationsBelowLine = mid;
		    onTarget = true;
		    targetValue = labels[col];
		}
		else 
		    onTarget = false;
	        repaint();
	    }
	}
    }

    public void superMouseMoved(MouseEvent e) {
	/**
	 * needed to override this class's mouseMoved method, but still access super.mouseMoved(Event).
	 * This is a cheap way to essentially allow the subclass (PercentileHistogram) to do
	 * super.super.mouseMoved(event) and bypass this class's method.
	 */
		super.mouseMoved(e);
	}

    public void mouseMoved(MouseEvent e) {
	super.mouseMoved(e);
	if (Math.abs(e.getX() - xCoord) < 7) 
	    setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
	else
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    public void superUpdate(Graphics e) {
	/**
	 * needed to override this class's mouseDragged method, but still access super.mouseDragged(Event).
	 * This is a cheap way to essentially allow the subclass (PercentileHistogram) to do
	 * super.super.mouseDragged(event) and bypass this class's method.
	 */
		super.update(e);
	}

    public void update(Graphics g) {
	super.update(g);
	if (xCoord == 0)
	    xCoord = getFirstBar();
 	double halfObs = (double)(totalObservations)/2.0;
	int blueVal = Math.min(Math.max((int)(256 * 
	    Math.abs((double)(observationsBelowLine) - halfObs)/halfObs) - 1,0),
	    255);
	Color c = new Color(255 - blueVal, 0, blueVal);
	g.setColor(c);
	g.drawLine(xCoord - 1, 1, xCoord - 1, getBaseLine() - 1);
	g.drawLine(xCoord, 0, xCoord, getBaseLine());
	g.drawLine(xCoord + 1, 1, xCoord + 1, getBaseLine() - 1);
	FontMetrics fm = g.getFontMetrics();
	String lLabel = Integer.toString(observationsBelowLine);
	String rLabel = Integer.toString(totalObservations - 
		observationsBelowLine);
	g.drawString(lLabel, xCoord - 5 - fm.stringWidth(lLabel), 
	    fm.getHeight());
	g.drawString(rLabel, xCoord + 5 , fm.getHeight());
	if (!mouseIsDown && onTarget) {
	    g.setColor(Color.black);
	    g.drawString(getTargetName() + " is " + targetValue, 
		xCoord + 5 , 2 * fm.getHeight());
	}
    }

    int[] getValues() {
	return values;
    }
    double getMean() {
	return mean;
    }
    int getChartWidth() {
	return columns * (barWidth + barSpacing);
    }
    int getMax() {
	return max;
    }
    int getLevel() {
	int level = FieldDescriptor.LEVEL_NOMINAL;
	try {
	    level = getFieldDescriptor(labelField).getLevel();
	}
	catch (InterchangeException e) { }
	return level;
    } 

    public Component getMeanFinder() {
	return new MeanFinder(this);
    }






    class MeanFinder extends Component implements MouseMotionListener,
	TableViewListener {
	MedianHistogram hist = null;
	int xCoord = 0;
	double leftWeight = 0.0;
	double rightWeight = 0.0;
	double pivot = 0.0;
	Image offscreen = null;

	int level = FieldDescriptor.LEVEL_ORDINAL;

	public MeanFinder(MedianHistogram hist) {
	    this.hist = hist;
	    addMouseMotionListener(this);
	    xCoord = hist.getFirstBar() + 1;
	    resetWeights();
	    int[] values = hist.getValues();
	    level = hist.getLevel();
	    hist.addTableViewListener(this);
	    int sum = 0;
	    int count = 0;
	    for (int i = 0; i < values.length; i++) {
		sum += i * values[i];
		count += values[i];
	    }
	    double mean = (double)sum/(double)count;
	    System.out.println("Mean should be: " + mean);
	}

	private double getLeftWeight(int[] values, double pivot) {
	    double weight = 0;
	    for (int i = 0; (i < values.length) && (i < pivot); i++)
		weight += (pivot - i) * values[i];
	    return weight;
	}
	private double getRightWeight(int[] values, double pivot) {
	    double weight = 0;
	    for (int i = (int)pivot; (i < values.length); i++)
		weight += (i - pivot) * values[i];
	    return weight;
	}
	private void resetWeights() {
	    int[] values = hist.getValues();
	    pivot = (double)(xCoord - hist.getFirstBar())/
		(double)(hist.getChartWidth())
		* (double)values.length;
	    if (pivot < 0)
		pivot = 0.0;
	    leftWeight = getLeftWeight(values, pivot);
	    rightWeight = getRightWeight(values, pivot);
	}

	public void paint(Graphics g) {
		super.paint(g);
	    update(g);
	}

	public void update(Graphics g) {
	    if ((offscreen == null) || 
		(offscreen.getHeight(null)!=getSize().height)
                || (offscreen.getWidth(null) != getSize().width)) {
                offscreen = createImage(getSize().width, getSize().height);
            }
            Graphics og = offscreen.getGraphics();
            og.setClip(0, 0, getSize().width, getSize().height);
            og.clearRect(0, 0, getSize().width, getSize().height);
	    fullPaint(og);
            g.drawImage(offscreen, 0, 0, null);
            og.dispose();
	}

	public void fullPaint(Graphics g) {
	    Dimension size = getSize();
	    g.setColor(hist.getBackground());
	    g.fillRect(0, 0, size.width, size.height);

	    if ((level != FieldDescriptor.LEVEL_INTERVAL) &&
	        (level != FieldDescriptor.LEVEL_RATIO)) 
	 	return;
		
	    int startX = hist.getFirstBar();
	    int chartWidth = hist.getChartWidth();
	    int[] values = hist.getValues();
	    g.setColor(Color.black);
	    if (xCoord < startX) {
		xCoord = startX;
		resetWeights();
	    }
	    int leftY = (int) ((size.height/2) * leftWeight/(leftWeight + 
		rightWeight));
	    int rightY = (size.height/2) - leftY;

	    g.drawLine(startX, size.height - rightY, startX + chartWidth, 
		size.height - leftY);

	    double delta = (double)(leftY - rightY)/(double)values.length;
	    double spacing = (double)chartWidth/(double)values.length;
	    double start = (double)startX + spacing/2.0;
	    int max = hist.getMax();
	    double scale = (double)(3 * size.height/4)/(double)max;
	    for (int i = 0; i < values.length; i++) {
		int h = (int) (values[i] * scale);
		if ((h < 1) && (values[i] > 0))
		   h = 1;
		int x = (int)(start + i * spacing);
		int y = size.height - rightY - (int)(delta * i);
		g.drawLine(x, y, x, y - h);
	    }

	    int blueVal = (int) (256 * (Math.abs(leftWeight - rightWeight)/
		(leftWeight + rightWeight)));
	    if (blueVal < 0)
		blueVal = 0;
	    else if (blueVal > 255)
		blueVal = 255;

	    //if (blueVal == 0) {
	    //g.drawString("Mean is ...", xCoord + 5, 20);
	    //}

	    g.setColor(new Color(255 - blueVal, 0, blueVal));

	    g.drawLine(xCoord - 1, 1, xCoord - 1, size.height - 1);
	    g.drawLine(xCoord, 0, xCoord, size.height);
	    g.drawLine(xCoord + 1, 1, xCoord + 1, size.height - 1);

	}

	private void propagateMouseEvent(MouseEvent e) {
	    MouseEvent me = new MouseEvent(hist, e.getID(),
		e.getWhen(), e.getModifiers(), e.getX(), 
		hist.getBaseLine() - 10, e.getClickCount(), 
		e.isPopupTrigger());
	    hist.mouseMoved(me);
	}
	public void mouseMoved(MouseEvent e) {
	    if (Math.abs(e.getX() - xCoord) < 7) 
	        setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
	    else
	        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    propagateMouseEvent(e);
	}
	public void mouseDragged(MouseEvent e) {
	    if (getCursor().equals(
		Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR))){

	        xCoord = e.getX();
	        if (xCoord < hist.getFirstBar()) 
	            xCoord = hist.getFirstBar();
	        if (xCoord > (hist.getFirstBar() + hist.getChartWidth())) 
		    xCoord = hist.getFirstBar() + hist.getChartWidth();
		resetWeights();
	        repaint();
	    }
	    propagateMouseEvent(e);
	}

	Dimension mySize() {
	    return new Dimension(hist.getSize().width, 100);
	}

 	public Dimension getMinimumSize() {
	   return mySize();
	}
	public Dimension getPreferredSize() {
	   return mySize();
	}

	public void tableViewChanged(TableViewEvent e) {
	    level = hist.getLevel();
	    xCoord = hist.getFirstBar();
	    offscreen = null;
	    repaint();
	}
    }

}



