package sieve.nuggets;			

import java.awt.*;
import java.awt.event.*;
import java.beans.*;	//for PropertyChangeSupport

import com.sun.java.swing.*;

/**
 * extends MedianHistogram, on which the user tries to locate the median,
 * with addition of locating the 25th percentile, and the 75th percentile. 
 *
 * Can register java.beans.PropertyChangeListener(s) 
 *
 * @see sieve.nuggets.MedianHistogram
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */


public class QuartileHistogram extends MedianHistogram implements MouseListener {

    //int observationsBelowLine = 0; -- in parent, this is # observations below median (50th percentile)
	int observationsBelow25th = 0;
	int observationsBelow75th = 0;

	//int xCoord = 0; -- in parent, this is xCoord of the Median line
    int x25Coord = 10;	//don't start right on edge (xCoord changed in constructor) --> this didn't seem to work! 
    int x75Coord = 0;	//x75 changed to width-5 when onTarget for median (in mouseReleased())

    //boolean onTarget = false; -- in parent, true if user has moved line to the correct place  
    boolean on25Target = false;
    boolean on75Target = false;

    //Object targetValue = null; -- in parent, gets the value of the current column, for median 
    Object target25Value = null;
    Object target75Value = null;


	//boolean doneMedian = false;	//start with the median bar only. After user has moved it
	boolean moving25 = false;		//to the right place, we'll draw the 25 and 75 percentile bars
	boolean moving75 = false;

	double horScale = 0.0;		//used in parent but declared in mouseDragged only. Make available to other methods

	PropertyChangeSupport propSupport = new PropertyChangeSupport(this);

	//String constants for property change events
	public static final String BELOW_25 = "below25";
	public static final String BELOW_75 = "below75";
	public static final String BELOW_MEDIAN = "belowMedian";
	public static final String TARGET_25 = "target25";
	public static final String TARGET_75 = "target75";
	public static final String TARGET_MEDIAN = "targetMedian";
	public static final String RESET = "reset";


    public QuartileHistogram() {
		super();
		xCoord = 10;
	}

	/**
	 * overrides MedianHistogram.setFields to reset properties
	 */
    public void setFields(String labelField, String freqField) {
		super.superSetFields(labelField, freqField); 
		totalObservations = sumColumns(0, columns - 1);
		observationsBelowLine = 0;
		observationsBelow25th = 0;
		observationsBelow75th = 0;
		xCoord = 10;
		x25Coord = 10;
		x75Coord = getWidth() - 10;
		onTarget = false;
		on25Target = false;
		on75Target = false;
		propSupport.firePropertyChange(RESET,null, null);
		repaint();
    }
	    

	/**
	 * overrides MedianHistogram.mouseDragged. Setting of "onTarget", etc, functionality moved
	 * to mouseReleased for all three quartile lines.
	 */
	public void mouseDragged(MouseEvent e) {
		super.superMouseDragged(e);	//indirect access to the Histogram class == super.super.MouseDragged
		if (getCursor().equals(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR))) {
			//then we are moving something...
			int col = getColumn(e);
			int tempCoord = e.getX();
			if (col >= 0) { 
				//much of this lifted directly from MedianHistogram...
				int oldObservations;	//save old for property event
				horScale = ((double)values[col])/((double)(barWidth + barSpacing));
				if (tempCoord < getFirstBar())
					tempCoord = getFirstBar();
				int tempObserv = sumColumns(0, col - 1);
				if(! onTarget) {	//1st, 3rd quartile lines not drawn at all until median found
					xCoord = tempCoord;
					oldObservations = observationsBelowLine;
					observationsBelowLine = tempObserv + (int)(horScale *((xCoord-getFirstBar()) - (col*(barWidth + barSpacing))));
					propSupport.firePropertyChange(BELOW_MEDIAN,new Integer(oldObservations), new Integer(observationsBelowLine));
					repaint();
				}
				else {	//user has found median. Other quartile lines drawn and might be moved
					if(moving25) {
						x25Coord = tempCoord;
						oldObservations = observationsBelow25th;
						observationsBelow25th = tempObserv + (int)(horScale *((x25Coord-getFirstBar()) - (col*(barWidth + barSpacing))));
						propSupport.firePropertyChange(BELOW_25,new Integer(oldObservations), new Integer(observationsBelow25th));
						repaint();
					}
					else if(moving75) {
						x75Coord = tempCoord;
						oldObservations = observationsBelow75th;
						observationsBelow75th = tempObserv + (int)(horScale *((x75Coord-getFirstBar()) - (col*(barWidth + barSpacing))));
						propSupport.firePropertyChange(BELOW_75,new Integer(oldObservations), new Integer(observationsBelow75th));
						repaint();
					}
				}
			}
			else if(col == -1) { //we're off the right side of the histogram. Just move the line.
				if(! onTarget) {
					xCoord = tempCoord;
					repaint();
				}
				else if(moving25) {
					x25Coord = tempCoord;	
					repaint();
				}
				else if(moving75) {
					x75Coord = tempCoord;
					repaint();
				}
			}
		}
	}


	/**
	 * overrides MedianHistogram.mouseReleased. Setting of "onTarget", etc, functionality 
	 * done here for all three quartile lines.
	 */
    public void mouseReleased(MouseEvent e) {
		mouseIsDown = false;
		int num;
		int col = getColumn(e);
		if(! onTarget) {	//have to find median before other lines are drawn
			num = totalObservations/2;
			if (Math.round((float)(100*observationsBelowLine)/totalObservations) == 50) {
				//WAS THIS TEST: ((Math.abs(num - observationsBelowLine) < horScale) && )
				//NOW POTENTIAL PROBLEM AT SOME SCREEN WIDTHS: can't use an = (in old test) because the particular value might not be 
				//reached -- 1 pixel-width's count-value depends on the column height. At some widths, desired percentile might be jumped
				//POSSIBLE FIX: insert test using current mouseX+1, mouseX-1. If our desired value is between, then that's as close
				//as user can get.
				if(observationsBelowLine != num) {
					propSupport.firePropertyChange(BELOW_MEDIAN, new Integer(observationsBelowLine), new Integer(num)); 
					observationsBelowLine = num;
				}
				targetValue = labels[col];	//set this before notification!
				onTarget = true;
				propSupport.firePropertyChange(TARGET_MEDIAN, null, targetValue); //no old value
				x75Coord = getWidth() - 10;	//set this up to draw inside edge
				repaint();		//draw the quartile lines
			}
		}
		else {
			if(moving25) {	//relying on mouseEvent order--these set to false in mouseMoved!
				num = totalObservations/4;
				if (Math.round((float)(100*observationsBelow25th)/totalObservations) == 25) //old test:((Math.abs(num - observationsBelow25th) < horScale)  && )
				{
					if(observationsBelow25th != num) {
						propSupport.firePropertyChange(BELOW_25, new Integer(observationsBelow25th), new Integer(num)); 
						observationsBelow25th = num;
					}
					target25Value = labels[col];
				    on25Target = true;
					propSupport.firePropertyChange(TARGET_25,null, target25Value);
				}
				else 
					on25Target = false;
			}
			else if(moving75) {
				num = (3*totalObservations)/4;
				if (Math.round((float)(100*observationsBelow75th)/totalObservations) == 75) {//old test:((Math.abs(num - observationsBelow75th) < horScale)  && ){
					if(observationsBelow75th != num) {
						propSupport.firePropertyChange(BELOW_75, new Integer(observationsBelow75th), new Integer(num)); 
						observationsBelow75th = num;
					}
					target75Value = labels[col];
				    on75Target = true;
					propSupport.firePropertyChange(TARGET_75,null, target75Value);
				}
				else 
					on75Target = false;
			}
		}
    }



	/**
	 * overrides MedianHistogram.mouseMoved to see if mouse is over any of our lines
	 */
   public void mouseMoved(MouseEvent e) {
		super.superMouseMoved(e);
		int mouseX = e.getX();
		moving25 = false;
		moving75 = false;
		if(onTarget) {  //these lines only drawn after median found
			if ((! on25Target) && (Math.abs(mouseX - x25Coord) < 5)) {	//can't move once get it right
				moving25 = true;
			}
			else if ((! on75Target) && (Math.abs(mouseX - x75Coord) < 5)) {
				moving75 = true;
			}
		}  
		if(((! onTarget) && (Math.abs(mouseX - xCoord) < 5) ) || moving25 || moving75) //user is over something they can move
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		else
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	public int getTotalObservations() {	//needed by container applet
		return totalObservations;
	}


	/**
	 * overrides MedianHistogram.update. Eliminates labelling the quartile line
	 * labels because with 3 they draw over each other. (QuartileApplet uses label
	 * in a panel under the QuartileHistogram)
	 */
    public void update(Graphics g) {
		super.superUpdate(g);

		g.setColor(Color.red);
		//triple-width line
		g.drawLine(xCoord - 1, 1, xCoord - 1, getBaseLine() - 1);
		g.drawLine(xCoord, 0, xCoord, getBaseLine());
		g.drawLine(xCoord + 1, 1, xCoord + 1, getBaseLine() - 1);

		if(onTarget) {	 
			g.setColor(Color.blue);
			g.drawLine(x25Coord - 1, 1, x25Coord - 1, getBaseLine() - 1);
			g.drawLine(x25Coord, 0, x25Coord, getBaseLine());
			g.drawLine(x25Coord + 1, 1, x25Coord + 1, getBaseLine() - 1);
				//x75Coord set to right side of histogram when onTarget set to true
			g.setColor(Color.green);
			g.drawLine(x75Coord - 1, 1, x75Coord - 1, getBaseLine() - 1);
			g.drawLine(x75Coord, 0, x75Coord, getBaseLine());
			g.drawLine(x75Coord + 1, 1, x75Coord + 1, getBaseLine() - 1);
		}

	}


	public void addPropertyChangeListener(PropertyChangeListener p) {
		propSupport.addPropertyChangeListener(p);
	}

	public void removePropertyChangeListener(PropertyChangeListener p) {
		propSupport.removePropertyChangeListener(p);
	}

}

