package sieve.nuggets;

import sieve.sluice.TableView;
import sieve.sluice.views.*;
import java.awt.*;
import sieve.sluice.*;
import com.sun.java.swing.border.*;
import java.beans.*;
/**
 * RegressionPlot takes data from a TableView class and finds the numeric fields,
 * using LabelView and NumberView. It uses the first field as the X-variable and 
 * all subsequent fields as Y-variables. It uses a NonEmptySampler to eliminate 
 * records with missing data and Descriptive view to get the min and max from the
 * sample. The data is plotted in different colors for each y-variable; regression
 * lines are also drawn for each.
 *
 * @see sieve.sluice.TableView
 * @see sieve.nuggets.TableViewJComponent
 * @see sieve.sluice.codebook.CodebookFinder
 * @see sieve.sluice.views.LabelView
 * @see sieve.sluice.views.NumberView
 * @see sieve.sluice.views.NonEmptySampler
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */

public class RegressionPlot extends TableViewJComponent 
	implements TableView, TableViewListener
{
	TableView			table = null;
	SampleView			sample = null;
	int					maxRecs = 0;	//maxTemp and maxRecs both needed since some
	int					maxTemp = 0;	//tables will have fewer than maxRecs valid records
	int					baseX,baseY,maxX,maxY,dotSize;			//for painting
	Double				dataMinX,dataMinY,dataMaxX,dataMaxY;	//for scaling and labeling
	int					actualFields = 0;						//how many selected and valid
	static final int	DELETED	= -256;							//just some number unlikely to show up in statistical data (I hope)
	Object[][]			tblData = null;							//for table.get()
	int[][]				plotData = null;						//scaled for painting
	String[]			x_yLabels = null;						//DescriptiveView labels
	String[]			tableFields = null;						//TableView labels
	Point[]				leftEndpoints = null;					//for regression line(s)
	Point[]				rightEndpoints = null;
	protected Color[]	colors = null;
	PropertyChangeSupport supporter = null;



	/**
	 * source table, initial fields to use, maximum number of records
	 * Calls initPlot(src, selections, max)
	 */
	public RegressionPlot(TableView src, String[] selections, int max) {
		super();
		initPlot(src,selections,max);
	}

	/**
	 * source table, initial fields to use
	 * Calls initPlot(src, selections, 0)
	 */
	public RegressionPlot(TableView src, String[] selections) {
		super();
		initPlot(src,selections,0);
	}

	/**
	 * source table
	 * Calls initPlot(src, null, 0)
	 */
	public RegressionPlot(TableView src) {
		super();
		initPlot(src,null,0);
	}

	/**
	 * Calls initPlot(null, null, 0)
	 */
	public RegressionPlot() {
		super();
		initPlot(null,null,0);
	}

	/**
	 * source table, initial fields to use, maximum number of records
	 * If src is null, maxRecords is set to the default and fields used (selections)
	 * is left null. Otherwise, if selections is null, all fields in the table will be
	 * available. If max is 0, the max is set to the number of records in src.
	 */
	private void initPlot(TableView src, String[] selections, int max) {
		supporter = new PropertyChangeSupport(this);
		setValues();
		setBorder(new LineBorder(Color.black,1));
		maxRecs = 500; 
		if (src != null) {
			table = src;
			super.source = src;
			if(selections != null) {		//set to selected fields; otherwise getFieldNames
					tableFields = selections;
					actualFields = selections.length;
			}
			if (max != 0)
				maxRecs = maxTemp = max; 
			else {
				try {
					maxRecs = src.getRecordCount();
					maxTemp = maxRecs;
				}
				catch (NoSourceException ex) {
					ex.printStackTrace();
				}
			}			
			try {makeCacheView();}
			catch(NoSourceException e) { e.printStackTrace();}
			makePlot();
		}
	}

	/**
	 * set up graphing "constants" and labels -- default values and 
	 * initializations which will change if the table view is not null or
	 * changes
	 */
	private void setValues() {
		x_yLabels = new String[2];
		x_yLabels[0] = new String("independent variable");
		x_yLabels[1] = new String("dependent variable");
	    dotSize = 6;	//needs to be even number	
		baseX = 40;		//axes drawn for graph
		baseY = 225;
		maxX = 240;
		maxY = 25;
		dataMinX = new Double(0.0);
		dataMinY = new Double(0.0);
		dataMaxX = new Double(100.0);
		dataMaxY = new Double(100.0);
		colors = new Color[10];
		colors[0] = Color.blue;	//should probably be done this way there instead...
		colors[1] = Color.red;
		colors[2] = Color.green;
		colors[3] = Color.cyan;
		colors[4] = Color.gray;
		colors[5] = Color.magenta;
		colors[6] = Color.lightGray;
		colors[7] = Color.orange;
		colors[8] = Color.yellow;
		colors[9] = Color.pink;
	}

	/**
	 * updates the display when the table changes
	 */
	public void tableViewChanged(TableViewEvent t) {
		if (table != null) {
		try 
			{
				maxTemp = table.getRecordCount();
			}
			catch (NoSourceException ex) {
				maxTemp = 0;
				ex.printStackTrace();
			}
			if (maxTemp > maxRecs)
				maxTemp = maxRecs;

		try {makeCacheView();}
		catch(NoSourceException e) { e.printStackTrace();}
		tableFields = null;
		makePlot();	//tableViewEvent propagated in handleSourceData
		repaint();
		}
	}

	public void addSourceView(TableView src) throws TooManySourcesException {
		if (table == null) {
			table = src;
		    super.source = src;
			if (table != null) { //might add null table somewhere
				try 
				{
					maxTemp = table.getRecordCount();
				}
				catch (NoSourceException ex) {
					maxTemp = 0;
					ex.printStackTrace();
				}
				if (maxTemp > maxRecs)
					maxTemp = maxRecs;
				try {makeCacheView();}
				catch(NoSourceException e) { e.printStackTrace();}
				tableFields = null;
				makePlot();
				repaint();
			}
		}
		else
			throw (new TooManySourcesException("RegressionPlot.addSourceView"));
	}

	public void removeSourceView(TableView tbl) {
		if (super.source == tbl) {
		    super.source = null;
			table = null;
			tableFields = null;
			actualFields = 0;
		}
		maxTemp = 0;
		repaint();
	}

	//********* visible PROPERTIES ************************************

	public int getMaxRecords() {
		return maxRecs;
	}
	public void setMaxRecords(int toPlot) {
		int old = maxRecs;
		maxRecs = toPlot;
		makePlot();
		repaint();
		supporter.firePropertyChange("maxRecs", (Object)(new Integer(old)), (Object)(new Integer(maxRecs)));
		//eventHandler.fireTableViewEvent(new TableViewEvent(sample));
	}

	public void addPropertyChangeListener(PropertyChangeListener p) {
		supporter.addPropertyChangeListener(p);
	}
		
	public void removePropertyChangeListener(PropertyChangeListener p) {
		supporter.removePropertyChangeListener(p);
	}

	//Not a "visible" property (no "set" method)
	/**
	 * returns the number of records used in plot; may be different than the
	 * set value, since NonEmptySampler eliminates records with missing data
	 */
	public int getNumberOfRecords() {
		return maxTemp;
	}

	/**
	 * access method: set fields to use in plot. Will not propagate into downstream
	 * data flow since <CODE>sample</CODE> (NonEmptySampler) points to whole records in 
	 * <CODE>table</CODE> with non-missing values <IT>in the indicated fields</IT>.
	 */
	public void setFields(String[] fds) {
		if (fds != null) {
			tableFields = fds;
			actualFields = fds.length;
		}
		else {
			tableFields = null;
			actualFields = 0;
		}
		makePlot();
		repaint();
	}

	/**
	 * calls local methods to do the work
	 */
	protected void makePlot() {
		retrieveFieldNames(); //if user hasn't specified, we'll get all from table
		if (actualFields > 0) { 
			handleSourceData();
			setDataToPaint(); //this calls calcRegression
			//repaint();	-- don't do this here because then can't subclass appropriately
		}
		else
			System.out.println("RegressionPlot.makePlot(): No fields of correct data type");
	}

	/**
	 * gets numeric fields from table and sets up a cached view to work from
	 */
	protected void makeCacheView() throws NoSourceException {
		if (table != null) {
			try {	//convert to a format we can use
				String[] f = table.getFieldNames();
				if ((f != null) && (f.length > 0)) {
						table = new sieve.sluice.views.LabelView(super.source);
						table = new sieve.sluice.views.NumberView(table);
						//make a cached copy of whole table to work with
						table = new sieve.sluice.views.CacheView(table); 
				}
			}
			catch (InterchangeException e) { 
				e.printStackTrace();
			}
		}
		else
			throw new NoSourceException("table is null");
	}

	/**
	 * tableFields is null at startup, after an addSourceView, and after a 
	 * getFieldNames returns no valid fields. Here we get any fields in the 
	 * source table.
	 */
	private void retrieveFieldNames() {
		if (tableFields == null) { //then we'll use all, making first one X
			try {
				String[] tmp = table.getFieldNames();  
				if(tmp != null) {
					if (tmp.length == 1) { //copy field for x and y
						tableFields = new String[2];
						tableFields[0] = tmp[0];
						tableFields[1] = tmp[0];
						actualFields = 2; 
					}
					else {
						actualFields = tmp.length;
						tableFields = new String[actualFields];
						tableFields = tmp;
					}
				}
				else
					actualFields = 0; //possible to have no fields after doing LabelView & NumberView
			}
			catch (NoSourceException ex) {
				ex.printStackTrace();
			}
		}
	}

	/**
	 * use DescriptiveViews to get min and max for scaling and plotting. 
	 */
	private void getMinMax(TableView t) {
		//pass this the sampleView table, don't use the cacheView
		sieve.sluice.views.DescriptiveView[] dv = new sieve.sluice.views.DescriptiveView[actualFields];
		double minTmp,min2;
		double maxTmp,max2;
		min2 = max2 = 0;
		int c;
		
		try {
			//DescriptiveView gets the descriptive statistics like min, max, mean, etc
			//for ONE field at a time
			for(c=0;c<actualFields;c++) 
				dv[c] = new sieve.sluice.views.DescriptiveView(t, tableFields[c]);
		}
		catch(InterchangeException e) {e.printStackTrace();}
		
		// Get min and max for plot range and scaling 
		if(actualFields != 0) {
			try {
				dataMaxX = new Double(((Number)dv[0].getMax()).doubleValue());
				dataMinX = new Double(((Number)dv[0].getMin()).doubleValue());
				//initialize these here, in case only 1 field and we're duplicating it for Y
				min2 = dataMinX.doubleValue();
				max2 = dataMaxX.doubleValue();

				if(actualFields > 1) {
					//get min and max from first Y variable field
					max2 = ((Number)dv[1].getMax()).doubleValue();
				 	min2 = ((Number)dv[1].getMin()).doubleValue();
					//compare min and max from rest of Y variable fields
					for(c=2;c<actualFields;c++) {
						minTmp = ((Number)dv[c].getMin()).doubleValue();
						maxTmp = ((Number)dv[c].getMax()).doubleValue();
						if(minTmp<min2)
							min2 = minTmp;
						if(maxTmp > max2)
							max2 = maxTmp;
					}
				}
			}
			catch (WrongDataKindException w) {w.printStackTrace();} 
			
			dataMaxY = new Double(max2);
			dataMinY = new Double(min2);
		}
	}

	/**
	 * whoever gave us the source table isn't responsible for knowing
	 * what we need, so we'll try to plot whatever we're given or fail
	 * gracefully...
	 * Gets labels for X and Y fields and uses NonEmptySampler to eliminate invalid 
	 * records, 
	 */
	private void handleSourceData() {
		x_yLabels = new String[actualFields];

		//make the correct fields array to use in the table.get()
		//If set by user, these will be in correct x,y order
		if(actualFields > 0) {
			try {
				FieldDescriptor fd;
				for(int c=0;c<actualFields;c++) {
					fd = table.getFieldDescriptor(tableFields[c]);
					x_yLabels[c] = fd.getDescription();
				}
			}
			catch (InterchangeException e) {e.printStackTrace();}
		
			//use NonEmptySampler to eliminate records with missing values
			try {
				sieve.sluice.views.NonEmptySampler neSamp = new sieve.sluice.views.NonEmptySampler(table,tableFields,maxRecs);
				sample = new sieve.sluice.views.SampleView(table,neSamp);
		        eventHandler.fireTableViewEvent(new TableViewEvent(sample));	//sample is a TableView
				getMinMax(sample);
				//get the data	
				tblData = sample.get(tableFields,0);//get all, since sampler has right number or has all valid
				maxTemp = tblData[0].length;		//number of records actually using
			}										
			catch (InterchangeException e) {
				e.printStackTrace();
			}
		} //end if(actualFields > 0)
	}
		
	/**
	 * scales data for plotting. To save an iteration, this method also sums
	 * up the values which are used to find the regression lines and calls
	 * calcRegression with them
	 */
	private void setDataToPaint() {
		if (actualFields > 1) { //this shouldn't be called if this fails, but for safety...
			float scaleFactX,scaleFactY;	//scale data to our graph
			double sumX2,sumX;	//These for calculating the regression line
			double temp;
			double[] sumY = new double[actualFields-1];
			double[] sumXY = new double[actualFields-1];
			double mxX,mxY,mnX,mnY;
			int tx,ty;
			
			sumX2 = sumX = 0.0;
			for(int m=0;m<actualFields-1;m++) {
				sumY[m] = 0;
				sumXY[m] = 0;
			}
			mxX = dataMaxX.doubleValue();
			mnX = dataMinX.doubleValue();
			mxY = dataMaxY.doubleValue();
			mnY = dataMinY.doubleValue();
	 
			if (tblData != null) { 
			//scale real data to fit on our plot
		    //map min to 0 and max to (max - base == 200)
	
				if (mxX != mnX)
					scaleFactX = (float)(maxX-baseX)/(float)(mxX-mnX);
				else
					scaleFactX = 1.0f;
				if (mxY != mnY)		//turned around for Y since Y's 0 is top left corner
					scaleFactY = (float)(baseY-maxY)/(float)(mxY-mnY); 
				else
					scaleFactY = 1.0f;
	
				//make the data array
				int len = tblData[0].length;
				plotData = new int[tblData.length][len];
	
				for(int i=0; i<len; i++) {
					//tblData[n][m] where n = number of fields and m = number records
					//tblData[0] is only x-variable; any other fields are y-vars
					temp = ((Number)tblData[0][i]).doubleValue()-mnX;
					tx = (int)(temp * scaleFactX)+baseX;
					plotData[0][i] = tx;
					sumX += tx;
					sumX2 += tx*tx;
					for(int q=1;q<actualFields;q++) {
						temp = ((Number)tblData[q][i]).doubleValue()-mnY;
						ty = baseY - (int)(temp * scaleFactY);
						plotData[q][i] = ty;
						sumY[q-1] += ty;
						sumXY[q-1] += tx * ty;
					}
				}
			  calcRegression(sumX,sumX2,sumY,sumXY);
			} //end if data not null
		}//end if actualFields >1
	}

	/**
	 * calculates regression line and sets the line drawing endpoints
     */
	private void calcRegression(double sumX, double sumX2,double[] sumY,double[] sumXY)
	{
	    int intercept, xTemp, yTemp;
		double denom, slope;
		leftEndpoints = new Point[actualFields-1];
		rightEndpoints = new Point[actualFields-1];

		denom =(maxTemp * sumX2) - (sumX * sumX);  //maxTemp is the number of records

		for (int j=0;j<actualFields-1;j++) {
			leftEndpoints[j] = new Point();
			rightEndpoints[j] = new Point();
		    if (denom != 0) {	//all our data has been scaled, so these values scaled
				intercept =(int)(((sumX2*sumY[j]) - (sumX*sumXY[j])) / denom);
				slope = ((maxTemp*sumXY[j]) - (sumX*sumY[j])) / denom;
			
				//left endpoint
				xTemp = baseX;
				yTemp = intercept;
				if (intercept <= maxY) {
					if (slope <= 0) { 
						leftEndpoints[j].x = 0;
						leftEndpoints[j].y = 0;
						rightEndpoints[j].x = 0;
						rightEndpoints[j].y = 0;
						continue;
					}
					else {
						xTemp = (int)((maxY-intercept)/slope);
						yTemp = maxY;	/*if left endpoints not right, add test here like at "reference" comment below*/
					}
				}
				else if (intercept >= baseY) {
					if (slope >= 0) { //really, neg slope drawn so won't be on graph
						leftEndpoints[j].x = 0;
						leftEndpoints[j].y = 0;
						rightEndpoints[j].x = 0;
						rightEndpoints[j].y = 0;
						continue;
					}
					else {
						xTemp = (int)((baseY-intercept)/slope);
						yTemp = baseY;
						if(xTemp < baseX) { /*reference*/
							xTemp = baseX;
							yTemp = ((int)(slope * xTemp) + intercept);
						}
					}
				}
				if (xTemp >= maxX) {	//then the left endpoint starts off the graph
					leftEndpoints[j].x = 0;
					leftEndpoints[j].y = 0;
					rightEndpoints[j].x = 0;
					rightEndpoints[j].y = 0;
					continue;
				}
				leftEndpoints[j].x = xTemp;
				leftEndpoints[j].y = yTemp;
				
				//right endpoint
				xTemp = maxX;
				yTemp = (int)(slope*maxX) + intercept;
				if (yTemp <= maxY) {
					xTemp = (int)((maxY-intercept)/slope); 
					yTemp = maxY;
				}
				else if (yTemp >= baseY) {
					xTemp = (int)((baseY-intercept)/slope); 
					yTemp = baseY;
				}
				rightEndpoints[j].x = xTemp;
				rightEndpoints[j].y = yTemp;
			}
			else { //denom == 0
			    slope = 0;
				intercept =(int)(sumY[j] / (maxTemp));
				leftEndpoints[j].y = rightEndpoints[j].y = intercept;
				leftEndpoints[j].x = baseX;
				rightEndpoints[j].x = maxX;
			}
		} //end for
	}

	public void paintComponent(Graphics g) {
		int i,strWidth;

		//paint the background
		g.setColor(Color.white);
		g.fillRect(0,0,getWidth(),getHeight());

		//draw border and axes
		g.setColor(Color.black);
		g.drawRect(0,0,size().width-1,size().height-1);
		g.drawLine(baseX,baseY,baseX,maxY);
		g.drawLine(baseX,baseY,maxX,baseY);
    
		//label x-axis and min/max values
		if (actualFields == 2) //at top in black if only one y-field, else in color (below)
			g.drawString("y: " + x_yLabels[1],10,15);
		g.drawString("x: " + x_yLabels[0],baseX-5,263);
		g.drawString(dataMinX.toString(),baseX-8,245);
		g.drawString(dataMinY.toString(),10,baseY);
		g.drawString(dataMaxY.toString(),10,baseY-192);
		g.drawString(dataMaxX.toString(),maxX-10,245);
	
		//draw tic marks on axes
		for(i=1;i<5;i++) {
		  g.fillRect(baseX-2+(50*i),baseY-2,2,4);
		  g.fillRect(baseX-2,baseY-2-(50*i),4,2);
		}//end for

		//draw the data points
		if (maxTemp > 500)
			dotSize = 3;	//should be even #, but 2 is too small!
		else if (maxTemp > 150)
			dotSize = 4;
		else
			dotSize = 6;
		int n = colors.length;
		for(i=1;i<actualFields;i++) {
			if(i<=n)
				g.setColor(colors[i-1]);
			else
				g.setColor(Color.black);

			for (int k=0;k<maxTemp;k++) {
				if(plotData[i][k] != DELETED)	//for the subclass EditableRegressionPlot which
												//allows users to delete and move points
				  g.fillOval(plotData[0][k]-dotSize/2, plotData[i][k]-dotSize/2, dotSize, dotSize);
			}
			//draw the regression line
			g.setXORMode(Color.white);
			g.drawLine(leftEndpoints[i-1].x,leftEndpoints[i-1].y,rightEndpoints[i-1].x,rightEndpoints[i-1].y);
			g.drawLine(leftEndpoints[i-1].x,leftEndpoints[i-1].y-1,rightEndpoints[i-1].x,rightEndpoints[i-1].y-1);
			g.setPaintMode();
			//label it in same color, at top if only 1 y-field, else listing by color
			if (actualFields > 2)
				g.drawString(x_yLabels[i],10,263+(18*i)); //263 is where x label is...
		}
	}

	public Color[] getColorList() {
		Color[] colors = new Color[10];
		colors[0] = Color.blue;	
		colors[1] = Color.red;
		colors[2] = Color.green;
		colors[3] = Color.cyan;
		colors[4] = Color.gray;
		colors[5] = Color.magenta;
		colors[6] = Color.lightGray;
		colors[7] = Color.orange;
		colors[8] = Color.yellow;
		colors[9] = Color.pink;
		return colors;
	}
	

	public String getXLabel() {
		return x_yLabels[0];
	}

	public String[] getYLabels() {
		int len = x_yLabels.length -1;
		String[] names = new String[len];
		for(int k=0;k<len;k++)
			names[k] = x_yLabels[k+1];
		return names;
	}

	/**
	 * Other TableView interface methods. RegressionPlot modifies the source
	 * view by getting only numeric fields and rows with non-missing values.
	 * The resulting table, (rather than the saved "source" table), "sample",
	 * is used accessed by all the get methods
	 */
	//addTableViewListener
	//removeTableViewListener

	/**
	 * Returns all records starting from record number num from the fields specified in fds. 
	 */
	public Object[][] get(String[] fds, int num) throws InterchangeException {
		if(sample != null) {
			try {return sample.get(fds,num);}
			catch (InterchangeException e) {throw e;}
		}
		else
			return null;
	}

	/**
	 * Returns no more than numberOfRecords records starting from record number num from the fields specified in fds. 
	 */
	public Object[][] get(String[] fds, int num, int numberOfRecords) throws InterchangeException {
		if(sample != null) {
			try {return sample.get(fds,num,numberOfRecords);}
			catch (InterchangeException e) {throw e;}
		}
		else return null;
	}

	/**
	 * Returns the FieldDescriptor for the specified field. 
	 */
	public FieldDescriptor getFieldDescriptor(String field) throws InterchangeException {
		if(sample != null) {
			try { return sample.getFieldDescriptor(field);}
			catch (InterchangeException e) {throw e;}
		}
		else return null;
	}

	/**
	 * Returns the names of fields in this view. 
	 */
	public String[]	getFieldNames() throws NoSourceException {
		if (sample != null)
			return sample.getFieldNames();
		else
			throw (new NoSourceException());
	}

	/**
	 * Returns number of records in the view. 
	 */
	public int getRecordCount() throws NoSourceException {
		if (sample != null)
			return sample.getRecordCount();
		else
			throw (new NoSourceException());
	}

	/**
	 * Returns the TableDescriptor for this view's representation of the table. 
	 */
	public TableDescriptor getTableDescriptor() throws NoSourceException {
		if (sample != null)
			return sample.getTableDescriptor();
		else
			throw (new NoSourceException());
	}

	public TableView table() throws NoSourceException {
		if (sample != null)
			return sample;
		else
			throw (new NoSourceException());
	}

	public Dimension getPreferredSize() {
		return new Dimension(300,300 + actualFields*25);
    }
    
    public Dimension getMinimumSize() {
		return new Dimension(300,300 + actualFields*25);
    }
	
	public Dimension preferredSize() {
		return getPreferredSize();
    }
    
    public Dimension minimumSize() {
		return getMinimumSize();
    }
      
} //end class



