package sieve.nuggets;

import java.awt.*;
import java.awt.event.*;
import sieve.sluice.*;

public class StretchableHistogram extends Histogram implements MouseListener,
    MouseMotionListener {

    protected boolean editable    = true;
    protected boolean isEditing	  = false;
    protected boolean resizeCursor = false;
    protected int     editColumn  = -1;
    protected Color   origColor   = null;
    protected boolean constrained = false;
    protected long    total	  = 0;

    public StretchableHistogram() {
	super();
    }

    public boolean isEditable() {
	return editable;
    }
    public boolean getEditable() {
	return editable;
    }
    public void setEditable(boolean editable) {
	this.editable = editable;
    }

    public boolean isConstrained() {
	return constrained;
    }
    public boolean getConstrained() {
	return constrained;
    }
    public void setConstrained(boolean constrained) {
	this.constrained = constrained;
	findTotal();
    }

    public void mousePressed(MouseEvent e) {
	if (!editable)
	    super.mousePressed(e);
	else if (e.getClickCount() == 3)
	    setFields(labelField, freqField);
	else
	    stretchColumn(e);
    }
    public void mouseReleased(MouseEvent e) {
	isEditing = false;
	clearResizeCursor();
	if (origColor != null) {
	    colors[editColumn] = origColor;
	    styles[editColumn] = BarChart.SOLID;
	}
	origColor = null;
	repaint();
    }

    /**
     * Finds the total number of observations when either the bin or
     * the field changes
     */
    private void findTotal() {
	total = 0;
	for (int i = 0; i < values.length; i++)
	    total += values[i];
//System.out.println("In findTotal: " + total);
    }

    public void setNumberOfBins(int bins) {
	super.setNumberOfBins(bins);
	findTotal();
    }

    public void setFields(String labelField, String freqField) {
	super.setFields(labelField, freqField);
	findTotal();
    }

    void showResizeCursor() {
	if (!resizeCursor) {
	    if (orientation == HORIZONTAL)
	        setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
	    else
	        setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
	    resizeCursor = true;
	}
    }
    void clearResizeCursor() {
	if (resizeCursor) {
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    resizeCursor = false;
	}
    }

    public void mouseMoved(MouseEvent e) {
	super.mouseMoved(e); 
	if (editable) {
	    if (! isEditing && !resizeCursor && onHandle(e)) 
	        showResizeCursor();
	    else if (resizeCursor && !onHandle(e) )
	        clearResizeCursor();
	}
    }
    public void mouseDragged(MouseEvent e) {
	stretchColumn(e);
	boolean inColumn = isEditing && (getColumn(e) == editColumn);
	if (inColumn && !resizeCursor)
	    showResizeCursor();
	else if (!inColumn && resizeCursor)
	   clearResizeCursor();
    }


    protected boolean onHandle(MouseEvent e) {
	int col = getColumn(e);
	if ((col >= 0) && (col < columns)) {
	    if (orientation == HORIZONTAL) {
	        return Math.abs((e.getX() - getBaseLine()) -
		    values[col] * scale) < 10;
	    }
	    else
	        return Math.abs(e.getY() - 
		    (getBaseLine() - values[col] * scale)) < 10;
	}
	return false;
    }

    protected int getColumn(MouseEvent e) {
	int col = -1;
	if (orientation == HORIZONTAL) 
            col = (e.getY() - getFirstBar())/(barWidth + barSpacing);
	else 
            col = (e.getX() - getFirstBar())/(barWidth + barSpacing);
	return col;
    }

    protected void stretchColumn(MouseEvent e) {
        int col = getColumn(e);
        boolean inColumn;

        if (! editable)
	    return;

        int minPixel = getBaseLine();
        int maxPixel = (int)(scale * max) + minPixel;
        int newValue = 0;

        switch (orientation) {
            default:
            case VERTICAL:
                newValue = (int)((minPixel - e.getY())/scale);
                inColumn = (! isEditing && onHandle(e))
		      || (isEditing && (col == editColumn) && 
		      (e.getY() <= minPixel));
                break;
            case HORIZONTAL:
                newValue = (int)((e.getX() - minPixel)/scale);
                inColumn = (! isEditing && onHandle(e))
		      || (isEditing && (col == editColumn) && 
		      (e.getX() >= minPixel));
                break;
        }
	if (total == 0)
	    findTotal();

        // We have to be in a valid column and if constrained, we cannot
        // exceed our original total
        if ((col >= 0) && (col < columns) && (newValue >= 0) 
	    && (total - values[col] + newValue > 0) && inColumn && 
	    (!constrained || (newValue <= total))) {
	    isEditing = true;
	    editColumn = col;
	    if (origColor == null) {
	        origColor = (Color)(colors[col]);
	        colors[col] = Color.red;
		styles[col] = BarChart.STRIPED;
		showResizeCursor();
	    }

	    int origValue = values[col];
total = total - origValue + newValue;
	    values[col] = newValue;
            if (constrained) {
		int change = origValue - newValue;
int allChanges = 0;
long newTotal = 0;
		for (int i = 0; i < values.length; i++) {
		    if (i != col) {
/*
			int iChange = (int) Math.round(change *
			    ((double)(values[i]))/((double)(total - newValue)));
*/
			int iChange = change/(values.length - 1);
			values[i] += iChange;
			if (values[i] < 0)
			    values[i] = 0;
allChanges += iChange;
		    }
newTotal += values[i];
/*
System.out.println("   Adding " + Math.round(change *
			    ((double)(values[i]))/((double)(total - newValue)))
+ " to col " + i + " now: " + values[i]);
*/
		}
System.out.println("change = " + change + " total = " + total + " allChanges = " + allChanges + " drift = " + new Integer(allChanges - change));
System.out.println("      new Total = " + newTotal + " total drift = " + new Integer((int)(newTotal - total)));

   	    }
            //System.out.println("Setting column " + col + " to " + newValue);
	    findCentralTendancy(hasMean, hasMedian);
	    repaint();
	    eventHandler.fireTableViewEvent(new TableViewEvent(this));
        }
    }

}

