package sieve.nuggets;

import java.applet.*;
import sieve.sluice.*;
import java.util.*;
import java.awt.*;

/**
 * 
 */
public abstract class TableViewComponent extends Component {
    protected TableView	source 		= null;

    protected TableViewEventSupport	eventHandler	= null;

    public TableViewComponent() {
	eventHandler = new TableViewEventSupport();
    }

    protected TableView outputView() throws NoSourceException {
	if (source == null)
	    throw new NoSourceException();
	return source;
    }

    public synchronized TableView table() throws NoSourceException {
	return outputView().table();
    }

    public synchronized TableDescriptor getTableDescriptor() throws
        InterchangeException {
        return outputView().getTableDescriptor();
    }

    public synchronized String[] getFieldNames() throws NoSourceException {
	return outputView().getFieldNames();
    }

    public synchronized FieldDescriptor getFieldDescriptor(String field) throws
        InterchangeException {
        return outputView().getFieldDescriptor(field);
    }

    /**
     * Returns number of records in the view.
     */
    public synchronized int getRecordCount() throws NoSourceException {
	return outputView().getRecordCount();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt, 
	int numberOfRecords) throws InterchangeException {
	return outputView().get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException { 
	return outputView().get(fields, startingAt); 
    } 

    /* -------------  Event-related methods ------------- */
    public synchronized void addTableViewListener(
	TableViewListener listener) {
	eventHandler.addTableViewListener(listener);
    }

    public synchronized void removeTableViewListener(
	TableViewListener listener) {
	eventHandler.removeTableViewListener(listener);
    }

    /**
     * Propagates event to all listeners with no filtering.
     */
    public void tableViewChanged(TableViewEvent event) {
	eventHandler.fireTableViewEvent(event);
    }

    public Dimension getPreferredSize() {
        Dimension size = getSize();
        if (size.width == 0) {
            size = new Dimension(200, 200);
        }
        return size;
    }
 
    public Dimension preferredSize() {
        return getPreferredSize();
    }
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    public Dimension minimumSize() {
        return getPreferredSize();
    }

	public void addNotify() {
		super.addNotify();
	}
}


