package sieve.nuggets.kit;


/**
 * DataDistributors are responsible for distributing data points along
 * an axis. Different implementations of this interface will handle
 * categorical, linear int continuous, log int continuous, linear real
 * continuous, and log real continuous. A "stretchable" implementation
 * can be created for things like histrograms whose bin widths can be
 * adjusted.
 * Axes will typically defer calls to  their <CODE>getCentralPixel</CODE> 
 * methods to one of these objects.
 */
public interface DataDistributor {
    /**
     * Return the pixel in the center of the space available for representation
     * of the given datapoint.
     */
    public int getCentralPixel(Object dataPoint);

    /**
     * Returns the size (in pixels) available for representation of the given
     * datapoint.
     */
    public int getMaxSize(Object dataPoint);

    /**
     * Returns the object which would be mapped to the specified pixel.
     */
    public Object getDataPoint(int pixel);

    /**
     * Sets the number of pixels over which data points may be distributed.
     */
    public void setSize(int pixels);
}

