package sieve.nuggets.kit;

import java.awt.*;

public class LabeledVerticalAxis extends SimpleVerticalAxis implements 
	Axis {
    private Object[] labelObjects = null;
    private int maxLabelWidth = 0;
    private boolean labelsOnLeft = true;

    public LabeledVerticalAxis(Component parent, 
	DataDistributor distributor) {
	super(parent, distributor);
    }

    public Object[] getLabelObjects() {
	return labelObjects;
    }
    public void setLabelObjects(Object[] labelObjects) {
	this.labelObjects = labelObjects;
    }

    public void setLabelsOnLeft(boolean labelsOnLeft) {
	this.labelsOnLeft = labelsOnLeft;
    }
    public boolean getLabelsOnLeft() {
	return labelsOnLeft;
    }

    public Dimension getPreferredSize() {
	int fontSize = 0;
	Component p = getParent();
	if (labelObjects != null) {
  	    if (p.getFont() != null) {
	        FontMetrics fm = p.getFontMetrics(p.getFont());
		maxLabelWidth = 0;
	      	for (int i = 0; i < labelObjects.length; i++) 
		    maxLabelWidth = Math.max(maxLabelWidth,
			fm.stringWidth(labelObjects[i].toString()));
	    }
	}
	return new Dimension(maxLabelWidth + 5, p.getSize().height);
    }

    public void paint(Graphics g) {
	super.paint(g);
	Rectangle bounds = getBounds();
	if (bounds != null) {
	    FontMetrics fm = g.getFontMetrics();
	    int height = fm.getHeight();
	    int first = getFirstPixel();
	    DataDistributor dist = getDataDistributor();
	    g.setColor(Color.black);
	    if ((labelObjects != null) && (bounds.width >= 2 + maxLabelWidth)) {
	        for (int i = 0; i < labelObjects.length; i++) {
		    int pos = dist.getCentralPixel(labelObjects[i]);
		    int width = fm.stringWidth(labelObjects[i].toString());
		    int x = 0;
		    if (labelsOnLeft)
			x = bounds.x - 3 - width;
		    else
			x = bounds.x + 3;
		    int y = bounds.y + bounds.height - pos + height/2;
		    if ((x >= 0) && (y >= 0))
		     	g.drawString(labelObjects[i].toString(), x, y);
		}
	    }
	}
    }

}



