package sieve.nuggets.kit;

/**
 * Linearly distributes a set of integer data on an axis. This 
 * is really simple: each int gets the same amount of space.
 */
public class LinearDataDistributor implements DataDistributor {
    private double min = 0;
    private double max = 0;
    private int size = 0;
    private double pixelsPerInt = 0;
    private boolean constrainedToInt = false;

    public LinearDataDistributor() { }

    /**
     * Returns true if each integer in the data must get an integer number
     * of pixels.
     */
    public boolean isConstrainedToInt() {
	return constrainedToInt;
    }

    /**
     * Set to true if each integer in the data must get an integer number
     * of pixels.
     */
    public void setConstrainedToInt(boolean constrainedToInt) {
	this.constrainedToInt = constrainedToInt;
    }

    /**
     * Sets the minimum data value.
     */
    public void setMin(double min) {
	this.min = min;
	resetMinMax();
    }

    /**
     * Sets the maximum data value.
     */
    public void setMax(double max) {
	this.max = max;
	resetMinMax();
    }

    private void resetMinMax() {
	pixelsPerInt = (((double)size)/(max - min + 1));
    }

    /**
     * Returns the size (in pixels) available for representation of the given
     * datapoint.
     */
    public int getMaxSize(Object dataPoint) {
	if (dataPoint instanceof Number) {
	    double d = ((Number)dataPoint).doubleValue();
	    if ((d >= min) && (d <= max))
	        return (int) Math.max((int)pixelsPerInt, 1);
	}
	return -Integer.MAX_VALUE;
    }

    /**
     * Return the pixel in the center of the space available for representation
     * of the given datapoint.
     */
    public int getCentralPixel(Object dataPoint) {
	if (dataPoint instanceof Number) {
	    double d = ((Number)dataPoint).doubleValue();
	    if ((d >= min) && (d <= max)) {
		if (constrainedToInt)
	            return (int) ((int)pixelsPerInt * (0.5 + d - min));
		else
	            return (int) (pixelsPerInt * (0.5 + d - min));
	    }
	}
	else
	    System.err.println("Warning: got non-numeric value " + dataPoint 
		+ " in LinearDataDistributor");
	return -Integer.MAX_VALUE;
    }

    public Object getDataPoint(int pixel) {
	if ((pixel >= 0) && (pixel < size)) {
	    if (constrainedToInt)
	        return 
	 	    new Double((double)(pixel)/((int)pixelsPerInt));
	    else
	        return 
	 	    new Double(((double)pixel)/((double)pixelsPerInt));
	 	// new Double(min + (max - min) * ((double)pixel)/((double)size));
	}
	return null;
    }
 
    /**
     * Sets the number of pixels over which data points may be distributed.
     */
    public void setSize(int pixels) {
	this.size = pixels;
	resetMinMax();
    }

}

