package sieve.phys.circuit;

import java.awt.*;
import java.awt.event.*;
import sieve.workbench.*;

/**
 * CircuitComponentTool is the tool used to create CircuitComponents.
 * It implement drag-and-drop-ish behavior in that the visual representation
 * of the component is added to the workspace while the user is dragging,
 * but the component is not technically added to the circuit until the
 * user released the mouse.
 */
public class CircuitComponentTool extends SimpleTool implements Tool {
    Workbench wb = null;
    Class     cl = null;
    CircuitComponent currentComponent = null;

    public CircuitComponentTool(Workbench wb, Class cl) {
	init(wb, cl);
    }

    public CircuitComponentTool() { }

    public void init(Workbench wb, Class cl) {
	this.wb = wb;
	this.cl = cl;
	CircuitWorkbenchListener l = 
	    CircuitWorkbenchListener.getCircuitWorkbenchListener(wb);
    }

    /**
     * Handle the positioning of the component. All the relevant 
     * MouseListener routines can just call this.
     */
    private void moveCurrentComponent(MouseEvent e) {
	Point p = e.getPoint();
	Component source = (Component)e.getSource();
	while (! (source instanceof Workbench) && (source != null)) {
	     Point loc = source.getLocation();
	     p.translate(loc.x, loc.y);
	     source = source.getParent();
	}

	// If we don't currently have a component, make one now.
	if (currentComponent == null) {
	    try {
	        currentComponent = (CircuitComponent)cl.newInstance();
	        currentComponent.setWorkbench(wb);
	        wb.add(currentComponent.getView(), 0);
	    }
	    catch (Exception ex) {
	        ex.printStackTrace();
		return;
	    }
	}
	// Move the component and force the workbench to validate
	Dimension b = currentComponent.getView().getPreferredSize();

	// We might want to change this to position the component
	// differently relative to the pointer
	currentComponent.getView().setBounds(p.x, p.y, b.width, b.height);
	wb.validate();
    }

    /**
     * Actually place the component when the user released the mouse.
     */
    public void mouseReleased(MouseEvent e) { 
	// Make sure we move the component to its "final" position
	moveCurrentComponent(e);
	Point p = e.getPoint();
	Component source = (Component)e.getSource();
       
       Component parent = source;
	while (parent != null  ) { // move up till parent=workbench or palette
	    if (parent instanceof Palette )
		{
                
               // check if it contains this point
                if (parent.contains(p.x,p.y))
                  // remove this instance
	           {
                 //   wb.remove(currentComponent.getComponent());
                     wb.remove(currentComponent.getView());
	            currentComponent = null;
	            wb.validate();
                   }
                else 
                   {
                    // Tell the workbench that we're done
              	    wb.componentAdded(currentComponent, this);

           	    // Just in case this instance is re-used, set this to null
          	    currentComponent = null; 
                   }
                parent=null;	
           	}
	    else if (parent instanceof Workbench)
               {
                 // we have gotten here so add component
	
                // Tell the workbench that we're done
              	wb.componentAdded(currentComponent, this);

           	// Just in case this instance is re-used, set this to null
          	currentComponent = null;
                parent=null; 
                }
            else parent=parent.getParent();  
             // not a workench or a Palette so move up one
	    }
    }

    public void mouseMoved(MouseEvent e) { 
        // only move component if not froma Palette
 	Point p = e.getPoint();
	Component source = (Component)e.getSource();
       
       Component parent = source;
	while (parent != null  ) { // move up till parent=workbench or palette
	    if (parent instanceof Palette )
		{ 
                parent=null;	
           	}
	    else if (parent instanceof Workbench)
               { // if we are from the workbench and not a Palette then 
                 // move component on move
	        moveCurrentComponent(e);
                parent=null; 
                }
            else parent=parent.getParent();  
             // not a workench or a Palette so move up one
	    }
    }
    public void mouseDragged(MouseEvent e) { 
           // allways move on a drag	
       moveCurrentComponent(e);
    }

    /**
     * If the user leaves the workspace, remove the component. We might
     * also want to do this if the user right-clicks, or if we detect
     * a tool change.
     */
    public void mouseExited(MouseEvent e) { 
	if (currentComponent != null) {
	    wb.remove(currentComponent.getView());
	    currentComponent = null;
	    wb.validate();
	}
    }

    public Cursor getPreferredCursor() {
	return null;
    }

}



