package sieve.phys.circuit;

import java.awt.*;
import java.io.IOException;
import java.util.*;
import java.lang.Math;


public class Equation{
   Vector ReturnLoopEquations;
   Vector ReturnJunctionEquations;
   Solver EquationSolver;
   private Math power;

   Equation(){
      ReturnLoopEquations = new Vector();
      ReturnJunctionEquations = new Vector();
   }


   public void SolveEquations(AGraph Matrix, Vector LoopPaths) throws BadCircuitException {
      Integer TempInt;
      Double TempDouble;
      Vector JunctionEquation;        
      Vector LoopEquation;
      Vector AllEquations = new Vector();
      Vector NextEquation;
      int numberofLoops;
      int numberofJunctions;
      

      //Find the number of equations we need to add into the equation solver
      numberofLoops = ReturnLoopEquations.size();
      numberofJunctions = 0;

      if(ReturnJunctionEquations.size() != 0){
          numberofJunctions = ReturnJunctionEquations.size();
          JunctionEquation = (Vector)ReturnJunctionEquations.elementAt(0);
      }
      LoopEquation = (Vector)ReturnLoopEquations.elementAt(0);

      //variables are the number of variables in the junction equation 
      int Variables = (Matrix.NumberOfCurrents()-1);
      EquationSolver = new Solver(Variables, (numberofLoops + numberofJunctions));

      for (int i = 1; i <= numberofJunctions; i++){ 
         AllEquations.addElement((Vector)ReturnJunctionEquations.elementAt(i-1));
      }

      for (int i = 1; i <= numberofLoops; i++){
         AllEquations.addElement((Vector)ReturnLoopEquations.elementAt(i-1));
      }

      //load Equations into the Solver

      for (int k = 0; (k < AllEquations.size()); k++){ 
         NextEquation = (Vector)AllEquations.elementAt(k);
         if (NextEquation.elementAt(0) instanceof Double){
            TempDouble = (Double)NextEquation.elementAt(0);
            EquationSolver.setupMatrix(k+1, (Variables+1), TempDouble.doubleValue());
  
            //insert the rest of the equation into the matrix solver
            for (int i = 1; i <= Variables; i++){
               TempDouble = (Double)NextEquation.elementAt(i);
               EquationSolver.setupMatrix(k+1, i, TempDouble.doubleValue());
            } //end for

         }
         else{
            TempInt = (Integer)NextEquation.elementAt(0);
            //insert the answer to the equation into the last position of the matrix
            //solver
            EquationSolver.setupMatrix(k+1, (Variables+1), (double)TempInt.intValue());

            //insert the rest of the equation into the matrix solver
            for (int i = 1; i <= Variables; i++){
               TempInt = (Integer)NextEquation.elementAt(i);
               EquationSolver.setupMatrix(k+1, i, (double)TempInt.intValue());
            } //end for
         } //end else
      } //end for
      EquationSolver.gaussianSolve();
      AssignComponentValues(Matrix, LoopPaths);
      EquationSolver.displayResults();
   }


 private void PrintPath(Vector AlphaPath){
      Integer localInt;
      Integer LastInt;
      LastInt = (Integer)AlphaPath.lastElement();
 
      int j = AlphaPath.indexOf(LastInt, 0);
 
      for(int i = j; i < AlphaPath.size(); i++){
         localInt = (Integer)AlphaPath.elementAt(i);
         System.out.println(" ");
         System.out.print(" " + localInt.intValue());
         System.out.print(" ");
      }
 
     System.out.println("");
  }


private void AssignComponentValues(AGraph Matrix, Vector LoopPaths){
     Integer localInt1;
     Integer localInt2;
     Vector LoopEquation;
     LoopEquation = (Vector)LoopPaths.elementAt(0);
     Integer LastInt;
     Edge ComponentEdge;
     CircuitComponent Comp = null;
     LastInt = (Integer)LoopEquation.lastElement();
     Vector LastLoopInt = (Vector)LoopPaths.lastElement();
     int j = LoopEquation.indexOf(LastInt, 0);
     int lastLoop = LoopPaths.indexOf(LastLoopInt, 0);

     for (int k = 0; k < LoopPaths.size(); k++){
        LoopEquation = (Vector)LoopPaths.elementAt(k);
        LastInt = (Integer)LoopEquation.lastElement();
        localInt2 = new Integer(-1);
        localInt1 = new Integer(-1);
   //  for (int i = 1; (localInt2.intValue() != LastInt.intValue()); i++){
   // The very first object was not getting it's amps changed. 
     for (int i = 0; (localInt2.intValue() != LastInt.intValue()); i++){
         localInt1 = (Integer)LoopEquation.elementAt(i);
         localInt2 = (Integer)LoopEquation.elementAt(i + 1);
         ComponentEdge = Matrix.getEdge(localInt1.intValue(), localInt2.intValue());
         Comp = ComponentEdge.GetComponent();
         if(Comp != null){
	    Comp.setAmps(EquationSolver.GetResults(ComponentEdge.CurrentName));
            ComponentEdge.AddComponent(Comp);
         }
         Matrix.returnEdge(ComponentEdge, localInt1.intValue(), localInt2.intValue());
         Comp = null;
     }
     localInt2 = new Integer(-1);
     }
}
        
public void PrintVectorofVectors(Vector PathVector){
      Vector TempVector;
      Integer TempInt;
      System.out.println("");
      for (int i = 0; i<PathVector.size(); i++){
        TempVector =(Vector)PathVector.elementAt(i);
        for (int j = 0; j<TempVector.size(); j++){
           TempInt = (Integer)TempVector.elementAt(j);
           System.out.println(TempInt.intValue());
        }
        System.out.println(" ");
      }
   }

   public void PrintLoopEquations(){
      Vector TempVector;
      Double TempInt;
      System.out.println("This is an equation:");
      for (int i = 0; i<ReturnLoopEquations.size(); i++){
        TempVector =(Vector)ReturnLoopEquations.elementAt(i);
        for (int j = 0; j<TempVector.size(); j++){
           TempInt = (Double)TempVector.elementAt(j);
           System.out.println(TempInt.doubleValue());
        }
        System.out.println(" ");
      }
   }

   public void PrintJunctionEquations(){
      Vector TempVector;
      Integer TempInt;
      System.out.println("This is a junction equation:");
      
      for (int i = 0; i < ReturnJunctionEquations.size(); i++){
        TempVector = (Vector)ReturnJunctionEquations.elementAt(i);
        System.out.println("Equation number(node): " + i);
        for (int j = 0; j<TempVector.size(); j++){
           TempInt = (Integer)TempVector.elementAt(j);
           System.out.println(TempInt.intValue());
        }
        System.out.println(" ");
      }
   }


/*This function finds the junction equations and puts them in the 
  return junction equations vector. The format of the euqation is 
  the following.
  The first element in the equation vector is the constant. The 
  next element is the current value (I1) the next value of the 
  vector is the coefficient to the current value (I2) and so on.
*/

   public boolean FindJunctionEquations(AGraph Matrix){
      Vector NewEquation;
      Edge NextEdge;

      
      int NumCurrents = Matrix.NumberOfCurrents();
      ReturnJunctionEquations = new Vector(NumCurrents);

      for (int i=1; i < Matrix.Capacity(); i++){
         if (Matrix.nodeEdges(i) > 2){
            NewEquation = GetJunctEquation(i, Matrix);
            ReturnJunctionEquations.addElement(NewEquation);
         }
      }
      if ((ReturnJunctionEquations.size() + ReturnLoopEquations.size()) < (NumCurrents-1)){
/*         System.out.println("False returned: " + (NumCurrents-1) + " :Number of Currents " +
                   ReturnJunctionEquations.size() + " :Number of JunctEq " +  ReturnLoopEquations.size() +
                   " Number of loop equations  ");
*/
         return false;
      }
      else{
/*         System.out.println("True returned: " + (NumCurrents-1) + " :Number of Currents " +
                   ReturnJunctionEquations.size() + " :Number of JunctEq " +  ReturnLoopEquations.size() +
                   " Number of loop equations  ");
*/
         return true;
      }
   }//end function

 /*This function find the junction equation from the junction 
   VertexNumber and returns a Vector with the Junciton equation
   from that junction
 */
   public Vector GetJunctEquation(int VertexNumber, AGraph Matrix){
      Integer CurrentDirection;
      int vectSize = Matrix.NumberOfCurrents();
      int Capacity = Matrix.NumberOfCurrents();
      Vector NewEquation = new Vector(Capacity); 
      
      Edge NextEdge = Matrix.First(VertexNumber);
      for (int i=0; i < Capacity; i++){
          Integer initalize = new Integer(0);
          NewEquation.addElement(initalize);
      }

      //if edge is incoming give this current positive value
      if(NextEdge.Direction == VertexNumber){
         CurrentDirection = new Integer(1); 
         int Iname = NextEdge.CurrentName;
         NewEquation.removeElementAt(Iname);
         NewEquation.insertElementAt(CurrentDirection, Iname);
      }
      else{
         CurrentDirection = new Integer(-1);
         int Iname = NextEdge.CurrentName;
         NewEquation.removeElementAt(Iname);
         NewEquation.insertElementAt(CurrentDirection, Iname);
      } 

      NextEdge = Matrix.Next(NextEdge.positionx, NextEdge.positiony);    
      while(NextEdge.On != 0){
         if(NextEdge.Direction == VertexNumber){
            CurrentDirection = new Integer(1);
            NewEquation.removeElementAt(NextEdge.CurrentName);
            NewEquation.insertElementAt(CurrentDirection, NextEdge.CurrentName);
         }
         else{
            CurrentDirection = new Integer(-1);
            NewEquation.removeElementAt(NextEdge.CurrentName);
            NewEquation.insertElementAt(CurrentDirection, NextEdge.CurrentName);
         } //end else
         NextEdge = Matrix.Next(NextEdge.positionx, NextEdge.positiony);
      }//end while
      return NewEquation;
   } //end function 


   public boolean FindLoopEquations(AGraph Matrix, Vector LoopPaths){

      int edge;
      int BatterySum;
      double Sum;
      int sign;

      Vector Loop;
      Vector NewEquation;

      Edge SearchEdge;
      Integer FirstElement;
      Integer SecondElement;
      int secondElement;
      int firstElement;

      if (LoopPaths.size() == 0){
         return false;
      }
      int NumCurrents = Matrix.NumberOfCurrents(); 

      ReturnLoopEquations = new Vector(NumCurrents); 

      ReturnLoopEquations = new Vector(NumCurrents);
      for (int i=0; i<LoopPaths.size(); i++){
         NewEquation = new Vector(NumCurrents);
         for (int k = 0; k < NumCurrents; k++){
            Double initalize = new Double(0.0);
            NewEquation.addElement(initalize);
         }
         edge = 0;
         Loop = (Vector)LoopPaths.elementAt(i);
         for (int j=0; j< (Loop.size()-1); j++){
            //get the node numbers of the first two nodes in the path
            //this specifies an edge we can get from the graph
            FirstElement = (Integer)Loop.elementAt(j);
            SecondElement = (Integer)Loop.elementAt(j+1);
            firstElement = FirstElement.intValue();
            secondElement = SecondElement.intValue();

            //get the edge from the graph
            SearchEdge = Matrix.getEdge(firstElement, secondElement);

            //Sum up the batteries on that edge
            //and sum up the resistors on that edge
            Sum = AddResistors(SearchEdge);
            BatterySum = AddBatteries(SearchEdge, secondElement);
            sign = getDirection(SearchEdge, secondElement);

            /* Since there can only be one battery per edge, we will add a resistance
               to the wire that represents the resistance of one battery;
            */
            if (BatterySum != 0){
               Sum = Sum + power.pow(10, -14);
            }
            Sum = Sum * sign;

            //Save these results into the new equation we are making
            //This fnction passes SearchEdge.CurrentName to Store Values, giving
            //the name of currents in the circuit.
            //NewEquation 
//            System.out.println("CurrentName : " + SearchEdge.CurrentName);
//            System.out.println("Number of Currents: " + NumCurrents);
            StoreValues(Sum, BatterySum, SearchEdge.CurrentName, NewEquation);
//            System.out.println("Store Values: Sum " + Sum + " BatterySum " + BatterySum);

         }
         ReturnLoopEquations.addElement(NewEquation);
      }
      return true;
   }

   public int getDirection(Edge SearchEdge, int SecondElement){
   /* This procedure is passed an edge and the direction we are traversing 
      the edge. If the direction is the same as the Current direction this 
      function returns 1, otherwise it returns -1 */

      int sameDirection = 1;
      int otherDirection = -1;
   
      if (SearchEdge.Direction == SecondElement){
         return sameDirection;
      }
      else
         return otherDirection;
   }

   public double AddResistors(Edge SearchEdge){
      /* This function adds up the value of all the resistors on the edge SearchEdge and
         returns that value
      */
      CircuitComponent Item;
      Vector ComponentVector;
      double SumResist = 0;

      ComponentVector = SearchEdge.Components;
      for (int i = 0; i < ComponentVector.size(); i++){
         Item = (CircuitComponent)ComponentVector.elementAt(i);
         if(Item instanceof Resistor || Item instanceof LightBulb
            || Item instanceof Voltmeter){
            SumResist = SumResist + Item.GetValue();
         }
      } 
    return SumResist;  
   }

   public int AddBatteries(Edge SearchEdge, int NodeTo){
      /* This function adds up the value of all the resistors on the edge SearchEdge and
         returns that value
      */

      CircuitComponent Item;
      Vector ComponentVector;
      int SumBatteries = 0;

      ComponentVector = SearchEdge.Components;
       for (int i = 0; i < ComponentVector.size(); i++){
         Item = (CircuitComponent)ComponentVector.elementAt(i);
         if(Item instanceof Battery){
            //if the battery is in the positive direction add 
            //the battery to a sum of batteries.
            if(Item.batteryDirection() == NodeTo){
               SumBatteries += Item.GetValue();
//               System.out.print("   BatteryValue: " + Item.GetValue() + " '' ");
            }
            else
               //the battery is in the negative direction subtract the value of the battery
               SumBatteries -= Item.GetValue();
//                System.out.print("   BatteryValue: " + Item.GetValue() + " '' ");
         }
       }

      return SumBatteries;
   }

   /*Procedure: StoreValues
     CalledBy: FindLoopEquation
   */

   public void StoreValues(double Sum, int BatterySum, int CurrentName, Vector LocalEquation){
    /*This function stores the value of sum in the CurrentName position in 
      the ReturnLoopEquations Vector. The BatterySum represents a coefficient
      for the CurrentName current. Battery Sum is added into the 0th position
      in the array.
    */


    /* In the Vectors there must be Integer types not ints */
    double totalBattery;
    double totalCurrent;
    Double TotalBattery;
    Double TotalCurrent;

    TotalBattery = (Double)LocalEquation.elementAt(0);
    TotalCurrent = (Double)LocalEquation.elementAt(CurrentName);
    
    totalBattery = TotalBattery.doubleValue();
    totalCurrent = TotalCurrent.doubleValue();

    totalBattery += BatterySum;
    totalCurrent += Sum;

    TotalBattery = new Double(totalBattery);
    TotalCurrent = new Double(totalCurrent);

    LocalEquation.removeElementAt(0);
    LocalEquation.insertElementAt(TotalBattery, 0);

    LocalEquation.removeElementAt(CurrentName);
    LocalEquation.insertElementAt(TotalCurrent, CurrentName);

   }
      
      

}
