package sieve.phys.circuit;

public class Graph{
        private int capacity = 100;
  	private Edge[][] GraphMatrix = new Edge[capacity][capacity];
        private Node[] NodeVector = new Node[capacity];
	private int NumVertex = 0;
	private int NumEdge = 0;
        private int localEID = 0;
        private int Variable = 1;
	
	Graph(){
	   for(int i = 0; i < capacity; i++)
	   {
              //allocate room for the nodeVector
              NodeVector[i] = new Node(i);
	      //allocate room for each row
                    GraphMatrix[i] = new Edge[capacity];
	      for (int j = 0; j < capacity; j++)
	      {
		//now create an int for each cell
		GraphMatrix[i][j] = new Edge();
                GraphMatrix[i][j].positionx = i;
                GraphMatrix[i][j].positiony = j;
	      }
	   } 
	}

      public int Capacity(){
         return capacity;
      }

      public int NumberOfCurrents(){
         return Variable;
      }

      /* This procedure created new current values for all of the 
         edges and then assigns them new current values.
         Called by the Overlord class
      */

      public void AssignCurrentValues(){
         int StartAt = 0;
         NewCurrents();
         AssignCurrents(StartAt);
      }

     public void PrintCurrents(){
         System.out.println("CurrentValues: ");
         for(int i = 0; i < 10; i++){
            for(int j = 0; j < 10; j++){
               System.out.print(" " + GraphMatrix[i][j].CurrentName + " ");
            }
            System.out.println(" ");
         }
         System.out.println(" ");
      }
 
      public void PrintDirections(){
         CircuitComponent Temp;
         System.out.println("Directions: ");
         for(int i = 0; i < 10; i++){
            for(int j = 0; j < 10; j++){
               Temp = GraphMatrix[i][j].GetComponent();
               if (Temp != null){
                  Temp.direction = GraphMatrix[i][j].Direction;
                  GraphMatrix[i][j].AddComponent(Temp);
               }
               System.out.print(" " + GraphMatrix[i][j].Direction + " ");
            }
            System.out.println(" ");
         }
      }

      private void NewCurrents(){
         Variable = 1;
         for(int i = 0; i < capacity; i++){
            for(int j = 0; j < capacity; j++){
               GraphMatrix[i][j].CurrentName = 0;
               GraphMatrix[i][j].Direction = 0;
            }
         }
      }

      public int nodeEdges(int i){
         return NodeVector[i].returnEdges();
      }

/*This procedure assigns current values and directions to all of the 
  edges in the graph. 

  Called by: AssignCurrentValues in this class

*/
      private void AssignCurrents(int StartingPoint){
         /* This function gets all nodes with two or more edges and assigns 
            current variable names on those edges.
         */

         Variable = 1;
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() == 2){
               WireCurrentValues(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() == 2){
               WireDirections(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() > 2){
               CurrentValues(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() > 2){
               NodeDirections(i);
            }
         }
     }


    private void AssignNodeDirections(int StartingPoint){
       int i = StartingPoint;
 
       if (NodeVector[i].returnEdges() == 2){
            WireDirections(i);
       }
       if (NodeVector[i].returnEdges() > 2){
            NodeDirections(i);
       }
    }


    private void AssignNodeCurrent(int StartingPoint){
       /* This function gets all nodes with two or more edges and assigns
            current variable names on those edges.
        */

       int i = StartingPoint;

       if (NodeVector[i].returnEdges() == 2){
            WireCurrentValues(i);
       }
       if (NodeVector[i].returnEdges() > 2){
            CurrentValues(i);
       }
    }



     private void WireDirections(int i){
        int Direction;
        boolean FoundIncoming = false;

        for(int j = 0; (j < capacity) && (FoundIncoming == false); j++){
           if (GraphMatrix[i][j].On == 1){
              if (GraphMatrix[i][j].Direction == 0){
                 /* There is no direction for the first edge, find other
                    edge and see if it has a direction
                 */
                 int l = j;
                 for(int k = ++l; (k < capacity); k++){
                    if (GraphMatrix[i][k].On == 1){
                       if (GraphMatrix[i][k].Direction == 0){
                          //other edge does not have a direction
                          GraphMatrix[i][j].Direction = i;
                          GraphMatrix[j][i].Direction = i;
                          //Then no other edge has direction
                          GraphMatrix[k][i].Direction = k;
                          GraphMatrix[i][k].Direction = k;
                          FoundIncoming = true;
                          AssignNodeDirections(k);
                       }
                       else
                          if (GraphMatrix[i][k].Direction == i){
                             //then other edge has incoming directio
                             // manke this edge outgoing 
                             GraphMatrix[i][j].Direction = j;
                             GraphMatrix[j][i].Direction = j;
                             FoundIncoming = true;
                          }
                          else
                             if (GraphMatrix[i][k].Direction == k){
                                //then other edge has outgoing direction
                                // make this edge incoming
                                GraphMatrix[i][j].Direction = i;
                                GraphMatrix[j][i].Direction = i;
                                FoundIncoming = true;
                             }
                    }//end if
                 } //end for
              } // end if 0
           else
             if (GraphMatrix[i][j].Direction == i){
                //this is an incoming edge, make other one outgoing
                for(int k = ++j; (k < capacity); k++){
                   if (GraphMatrix[i][k].On == 1){
                      if (GraphMatrix[i][k].Direction == 0){
                         GraphMatrix[i][k].Direction = k;
                         GraphMatrix[k][i].Direction = k; 
                         FoundIncoming = true;
                         AssignNodeDirections(k);
                      }
                   }
                }
             }  //end if i
           else
             if (GraphMatrix[i][j].Direction == j){
                //this is an outgoing edge, make other edge incoming
                for(int k = ++j; (k < capacity); k++){
                   if (GraphMatrix[i][k].On == 1){
                      if (GraphMatrix[i][k].Direction == 0){
                         GraphMatrix[i][k].Direction = i;
                         GraphMatrix[k][i].Direction = i;
                         FoundIncoming = true;
                         AssignNodeDirections(k);
                      }
                   }
                }
             } //end if j
           } //end if on
         } //end for
   } //end procedure


/*This procedure gives direction to node with three or more edges incoming*/

     private void NodeDirections(int i){
        boolean FoundIncoming = false;
        boolean FoundOutgoing = false;
        boolean FirstFound = false;

      if(nodeEdges(i) > 2){
        for(int j = 0; (j < capacity) && (FoundIncoming == false) && (FoundOutgoing == false); j++){
           if (GraphMatrix[i][j].On == 1){
              //if first is zero check next to find possible incoming edges

              if (GraphMatrix[i][j].Direction == 0){
                 for(int k = ++j; (k < capacity) && (FoundIncoming == false) && (FoundOutgoing == false); k++){
                    if (GraphMatrix[i][k].On == 1){
                       if (GraphMatrix[i][k].Direction == i){
                          FoundIncoming = true;
                       } //end if
                       if (GraphMatrix[i][k].Direction == k){
                          FoundOutgoing = true;
                       } //end if
                    } //end if
                 } //end for
              } //end if 0
              else
                 {  //the first edge found is incoming
                 if (GraphMatrix[i][j].Direction == i){
                    /* Change all non-directional edges to outgoing */
                    for(int k = ++j; k < capacity; k++){
                       if (GraphMatrix[i][k].On == 1){
                          if (GraphMatrix[i][k].Direction == 0){
                             GraphMatrix[i][k].Direction = k;
                             GraphMatrix[k][i].Direction = k;
                             AssignNodeDirections(k);
                          }
                       }
                    } //end for
                 }// end if 1
                 else
                    {
                    //the first edge found is outgoing
                    if (GraphMatrix[i][j].Direction == j){
                       for(int k = ++j; k < capacity; k++){
                          if (GraphMatrix[i][k].On == 1){
                             if (GraphMatrix[i][k].Direction == 0){
                                GraphMatrix[i][k].Direction = i;
                                GraphMatrix[k][i].Direction = i;
                                AssignNodeDirections(k);
                             }
                          }
                       }
                    }
                 } //end else
              } //end else
          } //end if
        } //end for
            if (FoundOutgoing == true){
               /* Change all non-directional edges to incoming */
                for(int j = 0; (j < capacity); j++){
                   if (GraphMatrix[i][j].On == 1){
                      if (GraphMatrix[i][j].Direction == 0){
                         GraphMatrix[i][j].Direction = i;
                         GraphMatrix[j][i].Direction = i;
                         AssignNodeDirections(j);
                      }
                   }
                } //end for
            }
            else{

            if (FoundIncoming == true){
               /* Change all non-directional edges to outgoing */
                for(int j = 0; (j < capacity); j++){
                   if (GraphMatrix[i][j].On == 1){
                      if (GraphMatrix[i][j].Direction == 0){
                         GraphMatrix[i][j].Direction = j;
                         GraphMatrix[j][i].Direction = j;
                         AssignNodeDirections(j);
                      }
                   }
                } //end for
            } //end if

            else {
               //change first edge to incoming and all other edges to outgoing
               for(int j = 0; (j < capacity) && (FirstFound == false); j++){
                  if (GraphMatrix[i][j].On == 1){
                     if (GraphMatrix[i][j].Direction == 0){
                        GraphMatrix[i][j].Direction = i; 
                        GraphMatrix[j][i].Direction = i;
                        FirstFound = true;
                        AssignNodeDirections(j);
                     } //end if
                     for(int k = ++j; k < capacity; k++){
                        if (GraphMatrix[i][k].On == 1){
                          if (GraphMatrix[i][k].Direction == 0){
                             GraphMatrix[i][k].Direction = k;
                             GraphMatrix[k][i].Direction = k;
                             AssignNodeDirections(k);
                          }
                        }
                     } //end for
                  } //end if
                } //end for
            } //end else
            } //end else
      } //end if
      else
         WireDirections(i);
         
} //end
                
/*Procedure: CurrentValues
  CalledBy: AssignCurrents
  Purpose: To assign current names to the edges in nodes with more than two 
           edges connected to it.
*/

     private void CurrentValues(int i){
         for(int j = 0; j < capacity; j++){     
           if (GraphMatrix[i][j].On == 1){
               if (GraphMatrix[j][i].CurrentName == 0){
                  GraphMatrix[j][i].CurrentName = Variable;
                  GraphMatrix[i][j].CurrentName = Variable++;
                  //AssignNodeCurrent(j);
               }
               if (GraphMatrix[i][j].CurrentName == 0){
                  //GraphMatrix[i][j].CurrentName = Variable++;
                  //AssignNodeCurrent(j);
               }
           }
         } 
     }

/* This procedure is called when there are only two edges 
coming into a node. it gives all of the edges current values */

     private void WireCurrentValues(int i){
        for(int j = 0; j < capacity; j++){
           if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].CurrentName != 0){
           /* if this edge is on and already has a current value
            * then find the other edge from this node and give it that Current
            * Value
            */
              int l = j;
              for(int k = ++l; k < capacity; k++){
                 if (GraphMatrix[i][k].On == 1){ 
                    if ((GraphMatrix[i][k].CurrentName == 0)){
                       GraphMatrix[k][i].CurrentName = GraphMatrix[i][j].CurrentName;
                       GraphMatrix[i][k].CurrentName = GraphMatrix[i][j].CurrentName;
                       if(NodeVector[k].returnEdges() == 2){
                          AssignNodeCurrent(k);
                       }
                    }
                 }
              }
           }
           else
              if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].CurrentName == 0){
              /* if this edge is on and does not have a current value yet then find the other
               * edge from this node and see if it has a current value
               */
                 int l = j;
                 for(int k = ++l; k < capacity; k++){
                     if (GraphMatrix[i][k].On == 1){
                        /* the second edge has no value*/
                        if ((GraphMatrix[i][k].CurrentName == 0)){
                           GraphMatrix[k][i].CurrentName = Variable;
                           GraphMatrix[i][k].CurrentName = Variable;
                           GraphMatrix[i][j].CurrentName = Variable;
                           GraphMatrix[j][i].CurrentName = Variable++;
                           if(NodeVector[k].returnEdges() == 2){
                              AssignNodeCurrent(k);
                           }
                        }
                        else{
                           if ((GraphMatrix[i][j].CurrentName == 0)){
                              /*the second edge has a value and we give it to the first*/
                              GraphMatrix[i][j].CurrentName = GraphMatrix[k][i].CurrentName;
                              GraphMatrix[j][i].CurrentName = GraphMatrix[k][i].CurrentName;
                              if(NodeVector[j].returnEdges() == 2){
                                 AssignNodeCurrent(j);
                              }
                           }
                        }
                     }
                 }
               }
      }
}

   public boolean EdgesInLoops(Search SearchGraph){
      for (int i = 1; i < capacity; i++){
          for(int k = i; k < capacity; k++){
            if (GraphMatrix[i][k].On == 1)
               if (!(SearchGraph.edgeIsInLoop(GraphMatrix[i][k].EID))){
                  return false;
               }
          }
      }
      return true;
   }



      public void PrintGraph(){
         System.out.println();
         System.out.println();
         for(int i = 0; i < 15; i++){
             for(int j = 0; j < 15; j++){
                System.out.print(GraphMatrix[i][j].On + "  ");
             }
          System.out.println(" ");
         }
       }
  
      public void PrintNodes(){
         System.out.println("Node List ");
         for(int i = 0; i < capacity; i++){
             System.out.print(i+ ": "  + NodeVector[i].returnEdges() + " ");
         }
         System.out.println(" ");
      }

	public int Vertices(){
	   return NumVertex;
	}

	public int nEdges(){
 	   return NumEdge;
	}

        public void AddComponent(int Vertex1, int Vertex2, 
	   CircuitComponent newComp){
           GraphMatrix[Vertex1][Vertex2].AddComponent(newComp);
           GraphMatrix[Vertex2][Vertex1].AddComponent(newComp);
        }

        public void ConnectNodes(int Vertex1, int Vertex2){
           NumEdge++;
           localEID++;
           //System.out.println("LocalEID: " + localEID);
           GraphMatrix[Vertex1][Vertex2].EID = localEID;
           GraphMatrix[Vertex2][Vertex1].EID = localEID;
           GraphMatrix[Vertex1][Vertex2].MakeConnection();
           GraphMatrix[Vertex2][Vertex1].MakeConnection();
           NodeVector[Vertex1].addEdge();
           NodeVector[Vertex2].addEdge();
        }

        public Edge getEdge(int Vertex1, int Vertex2){
           return GraphMatrix[Vertex1][Vertex2];
        }

        /*This function returns an edge that has been modified after
          a call to getEdge
        */ 

        public void returnEdge(Edge retEdge, int Vertex1, int Vertex2){
           GraphMatrix[Vertex1][Vertex2] = retEdge;
        }


        /*
         Function: RemoveNodes
         Purpose: This procedure removes an edge from the adjacency 
                  matrix graph given the number of the nodes the edge 
                  connects. It also removes the edgeID from the list of 
                  edge ID's kept in SearchGraph.
         SideEffects: this procedure decreased the number of edge counter 
        */

        public void RemoveNodes(int Vertex1, int Vertex2, Search SearchGraph){
           NumEdge--;
           SearchGraph.removeEdge(GraphMatrix[Vertex1][Vertex2].EID);
           GraphMatrix[Vertex1][Vertex2].RemoveConnection();
           GraphMatrix[Vertex2][Vertex1].RemoveConnection();
           NodeVector[Vertex1].removeEdge();
           NodeVector[Vertex2].removeEdge();
        }
	
	/* This function returns the first edge of a Vertex*/
	public Edge First(int VertexNum){
                  Edge TempEdge = new Edge();
	   for (int j = 0; j < capacity; j++){
		TempEdge = GraphMatrix[VertexNum][j];
		 if (TempEdge.On != 0)
			return TempEdge;
	   }
         return TempEdge; //returns null if there is no Edge  
  	}


	/* This function returns true if the Vertex (VertexNum) has 
	     an edge that goes to "Conntects To", false otherwise. */

	public boolean IsEdge(int VertexNum, int ConnectsTo){
                 Edge TempEdge = new Edge();
	   TempEdge = GraphMatrix[VertexNum][ConnectsTo];
	   if(TempEdge.On != 0)
		return true;
	    else
		return false;
	}

	/* This function returns the Edge from the Vertex "VertexNum"
	     That comes after "ConnectsTo" If this operation is called on the 
      last vertex the first edge will be returned */

	public Edge Next(int VertexNum, int ConnectsTo){
	   int Start = ConnectsTo;
	   Edge TempEdge = new Edge();
	   for (int j = ++ConnectsTo; j < capacity; j++){
		TempEdge = GraphMatrix[VertexNum][j];	
		if (TempEdge.On != 0)
			return TempEdge;
	   }
         /*
	   for (int i = 0; i < ConnectsTo; i++){
		TempEdge = GraphMatrix[VertexNum][i];
		if(TempEdge.On != 0)
			return TempEdge;
	   }
         */
	   return TempEdge;   //return null if Next is not found
	}

	/* This function returns the weight of the edge at position VertexNum
	   and connectsTo */

	public int EID(int VertexNum, int ConnectsTo){
	   return GraphMatrix[VertexNum][ConnectsTo].EID;
}


}
