package sieve.phys.circuit;

import java.awt.*;
import java.io.IOException;
import java.lang.Math;

/**
 * Solver is a singleton object. Each circuit component must register
 * itself with the solver using a call such as: 
 * <PRE>
 *      Solver.getSolver().registerComponent(aComponent);
 * </PRE>
 * This should typically be done on construction. Currently we just
 * support components as "nodes" in the circuit. 
 * <P>
 * The Solver's <CODE>componentConnectionAdded</CODE> and
 * <CODE>componentConnectionRemoved</CODE> methods will be invoked when
 * connections between the nodes are made or broken.
 */
public class Solver
{
    // Define the matrix to solve
    double answerMatrix[];
    double f_matrix[][];
    double newmatrix[][];
    int Size;             
    int NumberofEquations;
    double smallestInt;
    Math power;

    public Solver(int NumberofVariables, int EquationNum)
    {
       if(NumberofVariables > EquationNum){
          System.out.println("Not going to work");
       }
       smallestInt = power.pow(10, -18);
       Size = NumberofVariables;
       NumberofEquations = EquationNum;
       newmatrix = new double[NumberofEquations+1][Size+2];
       answerMatrix = new double[Size+1];
       f_matrix = new double[NumberofEquations+1][Size+2];
       for (int i = 0; i < (NumberofEquations+1); i++){
          for (int j = 0; j < (Size+2); j++){
              f_matrix[i][j] = 0;
          }
       }
    }

   public void printMatrix(){
       for (int i = 1; i < (NumberofEquations+1); i++){
          for (int j = 1; j < (Size+2); j++){
             System.out.print(f_matrix[i][j] + "  ");
          }
       System.out.println("");
       }
    }

    public void printnewMatrix(){
       for (int i = 1; i < (NumberofEquations+1); i++){
          for (int j = 1; j < (Size+2); j++){
             System.out.print(newmatrix[i][j] + "\t\t");
          }
       System.out.println("");
       }
    }


    public void setupMatrix(int row, int column, double value){
       f_matrix[row][column] = value;
    }

    private boolean findZeroColumn(){
//       System.out.println("in findZeroRow");
       /*This function returns true if if finda a row consisting of Zero's so the 
         caller can search for further rows of zero */

       int columnNumber = 0;

       for (int j=1; j< (Size+2); j++){
          for(int i=1; i<(NumberofEquations+1) && newmatrix[i][j] == 0;){
                columnNumber = i++;
          }
//          System.out.println(columnNumber);
//          System.out.println(Size);
          if (columnNumber == (NumberofEquations)){
             //we found a column of zeros', now we must remove it
             System.out.println("Found col of zero");
             eliminateColumn(j);
             return true;
          }
          else
             columnNumber = 0;
       }
       return false;
    }

    private void eliminateColumn(int ColumnNumber){
       /*This function removes the row identified by RowNumber because it was found to have
         a row of zeros. it also changes the total number of rows to one less */

        for(int i=ColumnNumber; i< (Size+2); i++){
            for (int j=1; j< (NumberofEquations+1); j++){
               newmatrix[j][i] = newmatrix[j][i+1];
            }
        }
        Size--;
    }
                
    public boolean gaussianSolve(){
        
        int roworder[] = new int[Size+1];

        //Read in and find the biggest order magnitude in each diagonal

        int location;
        double temp;

        // Copy the matrix to an analysis one
        for (int i=1; i< (NumberofEquations+1); i++)
            for (int j=1; j< (Size+2); j++)
                newmatrix[i][j] = f_matrix[i][j];

        location = 1;
        for (int j=1; j < (Size+1); j++)
        {
            temp = Math.abs(newmatrix[j][j]);
            roworder[j] = j;

            for (int i=1; i<(Size+1); i++)
            {
                if ((Math.abs(newmatrix[i][j]) > temp))
                {
                    boolean inRowOrder = false;
                    for(int k=1; k<(Size+1); k++){
                       if(roworder[k] == i){
                          inRowOrder = true;
                       }
                    }
                    if(!inRowOrder){
                       temp = newmatrix[i][j];
                       roworder[j] = i;
                    }
                }
            }

            for (int k=1; k<(Size+2); k++){
               newmatrix[roworder[j]][k] = 0;
            }
        }


        // Rearrage the matricies in this order

        for (int j=1; j<Size+2; j++)
        {
            for(int k=1; k<Size+1; k++){
               newmatrix[k][j] = f_matrix[roworder[k]][j];
            }
        }
        System.out.println("This is the matrix.");
        printnewMatrix();
        while (findZeroColumn()){}
//        System.out.println("This is the matrix.");
//        printnewMatrix();
        // Create zeros below main diagonal

        double multiplier;
        for (int i=1; i<(Size+1); i++){
           for(int j=i+1; j<(NumberofEquations+1); j++){

              /*If there is a zero on the diagonal, switch this 
                row with one that has no zero */
               
              if (newmatrix[i][i] == 0){   
                 boolean allZero = changeRows(i);
                 if(allZero==false){
                    return allZero;
                 }
              }
              if (Math.abs(newmatrix[i][i]) != 0){
                 multiplier = -(newmatrix[j][i])/newmatrix[i][i];
              }
              else{
                 System.out.println("newmatrix[i][i] " + newmatrix[i][i] + " i:" + i);
                 System.out.println("Division by zero narrowly avioded!!");
                 multiplier = 1;
              }
              double oldNum; 
              for (int k=i; k<(Size+2); k++){ 
                 oldNum = newmatrix[j][k];
                 if (Math.abs(((multiplier * newmatrix[i][k]) + oldNum)) < smallestInt){
                    newmatrix[j][k] = 0;
                 }
                 else
                    newmatrix[j][k] = ((multiplier * newmatrix[i][k]) + oldNum);
              while (findZeroColumn()){
//                 System.out.println("Removed Column");
//                 printnewMatrix();
              }
              }   
           }        
        }        
 
        System.out.println("ZerosBelow:");
        printnewMatrix();
        //Create zeros above main diagonal
        for (int i=Size; i>0; i--){
           for(int j=i-1; j>0; j--){
              if (newmatrix[i][i] != 0){
                 multiplier = -(newmatrix[j][i]/newmatrix[i][i]);
              }
              else 
                 multiplier = 1;
              for(int k=i; k<(Size+2); k++){
                 if (Math.abs(((multiplier * newmatrix[i][k]) + newmatrix[j][k])) < smallestInt){
                    newmatrix[j][k] = 0;
                 }
                 else
                    newmatrix[j][k] = ((multiplier * newmatrix[i][k]) + newmatrix[j][k]);
              while (findZeroColumn()){
//                 System.out.println("Zero Column");
//                 printnewMatrix();
              }
              }
           }
        }
        boolean below;
        below = zeroColumn();
        if (below == false){
           return below;
        }

//        System.out.println("ZerosAbove:");
//        printnewMatrix();
        // Backsolve for x,y,z - then add to answer matrix


        for (int i=1; i<Size+1; i++){
           answerMatrix[i] = newmatrix[i][Size+1]/newmatrix[i][i];
        }

        below = zeroColumn();
        return below;

    }

//Given a row number that has a zero in the pivot this function searches all 
//Equations below that row for a row that does not have a zero in the pivot

public boolean changeRows(int i){
    int l;
    l = i;
    while((newmatrix[l][i]==0) && (l < NumberofEquations)){
//    while((newmatrix[l][i]==0) && (l < NumberofEquations+1)){
      l++;
    }
    if (l==(NumberofEquations+1)){        //column of all zeros
       return false;
    }
    double changeRow[];
    double origChangeRow[];
    changeRow = new double[Size+2];
    origChangeRow = new double[Size+2];

    /*Copy row with zero in diagonal into vector*/
    for(int m=1; m<(Size+2); m++){
       changeRow[m] = newmatrix[i][m]; 
       origChangeRow[m] = f_matrix[i][m];
    }  
    /*copy column with no zero on diagonal into this column*/
    for(int n=1; n<(Size+2); n++){
       newmatrix[i][n] = newmatrix[l][n];
       f_matrix[i][n] = f_matrix[l][n];
    }
    /*copy column formerly on diagonal into column l*/
    for(int p=1; p<(Size+2); p++){
       newmatrix[l][p] = changeRow[p];
       f_matrix[l][p] = origChangeRow[p];
    }

    return true;
}

public boolean zeroColumn(){
   int j = 1;
   int numberOfZero = 0;
   for (int i=1; i<(NumberofEquations+1); i++){
     while(j<(Size+1) && newmatrix[i][j]==0){
        j++;
     }
     if (j==(Size+1)){
        numberOfZero++;
     }
     else
        j = 1;
   }
   if ((NumberofEquations - numberOfZero) < Size)
      return false;
   else
      return true;
}

public double GetResults(int var){
   return answerMatrix[var];
}

   
public void displayResults () throws BadCircuitException{
   System.out.println("Linear System Solutions:");
   System.out.println("------------------------\n");
   for(int i=1; i<Size+1; i++)
      System.out.println(i + " variable   " + answerMatrix[i]);
   if (checkResults()){
      throw new BadCircuitException();
   }
}

private boolean checkResults(){
   for(int i=1;i<(Size+1);i++){
      if (power.abs(answerMatrix[i]) > power.pow(10, 12))
         return true;
   }
   return false;
}

}
