/*
 *
 * Written By:		James "Bo" Begole
 * Date:			18 June 1996
 * Description:	Block and Plane Simulation Applet
 *
 * Usage Notes:
 *
 * 
 */ 
 
/*
 *
 * Copyright (c) 1996 James "Bo" Begole, All Rights Reserved.
 *
 * 
 */
package sieve.phys.lwblockandplane; 

import java.applet.Applet;
import java.awt.*;
import java.util.*;
import java.beans.*;

import sieve.sluice.*;
import sieve.sluice.event.*;
 
public class BlockAndPlane extends Panel implements TableView {

    public static final int STOPPED = 0;
    public static final int RUNNING = 1;
    public static final int PAUSED = 2;

    private int executionState = STOPPED;

    PropertyChangeSupport       changes 
	= new PropertyChangeSupport(this);

    public SimCanvas simCanvas = null;
    Button Button_run, Button_pause, Button_reset;
    protected Float t_incr = null;

    public  int   runCount = 0;
    SimulationThread thread = null;
    boolean Started = false; // Has "Run" been pushed?
    public  Plane plane1 = null;
    public  Plane plane2 = null;
    public  Plane floor = null;
    public  MovingBody MB = null;
    TextField runCounter = null; 

  private static final String[] columnNames = 
      {"Distance", "Velocity", "Acceleration", "Time"};
  private static final String[] columnTypes = 
      {"java.lang.Long","java.lang.Long","java.lang.Long","java.lang.Long"};
  private static final String[] columnDescs = 
      {"meters","meters/millisec","meters/millisec/millisec","millisec"};

  public BlockAndPlane() {
    // Check if we are working collaboratively and initiate, if so
    //createSession(getParameter("SessionURL"));

    // init canvas objects
    plane1 = new Plane(0.5f, 1.0f, 0.0f, 0.00f, 0.02f, 0, true, "Plane1");
    plane2 = new Plane(1.0f, 2.0f, 0.0f, 0.00f, 0.02f, 500, false, "Plane2");
    floor  = new Plane(0f, 2.0f, 0.0f, 0.0f, 0f, 100, true, "Floor");
    MB = (MovingBody)(new Block(0.15f,0.15f, this, "Block"));
    t_incr = new Float(0.10f);

    simCanvas = new SimCanvas(plane1, plane2, floor, MB);

    // Set Applet layout
    setLayout(new BorderLayout());
		
    // Create Simulation Canvas
    //PLI simCanvas.setBackground(Color.black);
    //simCanvas.setBackground(getBackground());
		
    add("Center",simCanvas);
		
    //PLI Panel BottomPanel = new Panel();
    Container BottomPanel = new Panel();
    BottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    runCounter = new TextField(new Integer(runCount).toString(),3);
    runCounter.setEditable(false);
    BottomPanel.add(runCounter);
    Button_run = new Button("Run");
    BottomPanel.add(Button_run);
    Button_pause = new Button("Pause");
    BottomPanel.add(Button_pause);
    Button_reset = new Button("Stop");
    BottomPanel.add(Button_reset);
		
    add("South",BottomPanel);
		
    refresh();

    setupTable(); // part of quick hack 05/23/97 -Bo

  }

    private Frame getParentFrame() {
    	Component comp = this;
    	while (!(comp instanceof Frame)) {
      	    comp = comp.getParent();
    	}
    	return (Frame)comp;
    }

    public  void  refresh() {
     	// Set the block redraw flag
    	MB.reset();
    	// Recompute the relative plane positions
    	debug("refresh() setfloors and planes");
    	floor.setLeftEnd(plane1.getLength()+plane1.getLeftEnd());
    	plane2.setLeftEnd(floor.getLength()+floor.getLeftEnd());
    	simCanvas.repaint(50);
    }

    public  Plane getPlane1() {
    	return (Plane)plane1;
    }
    public  Plane getPlane2() {
    	return (Plane)plane2;
    }
    public  Plane getFloor() {
    	return (Plane)floor;
    }

    	public  float getTimeStep() {
      	    return t_incr.floatValue();
    	}
    	public  void setTimeStep(float t) {
            t_incr = new Float(t);
        }
    	public float getPlane1Height() {
    	    return plane1.getHeight();
        }
        public void setPlane1Height(float height) {
	    float oldHeight = plane1.getHeight();
	    if (height != oldHeight) {
		plane1.setHeight(height);
		refresh();
		changes.firePropertyChange("plane1Height", new Float(oldHeight),
		    new Float(height));
	    }
	}
        public float getPlane1Length() {
	    return plane1.getLength();
        }
        public void setPlane1Length(float length) {
	    float oldLength = plane1.getLength();
	    if (length != oldLength) {
		plane1.setLength(length);
		refresh();
		changes.firePropertyChange("plane1Length", new Float(oldLength),
		    new Float(length));
	    }
	}
        public float getPlane1Angle() {
	    return plane1.getAngle();
        }
        public void setPlane1Angle(float angle) {
	    float oldAngle = plane1.getAngle();
	    if (angle != oldAngle) {
		plane1.setAngle(angle);
		refresh();
		changes.firePropertyChange("plane1Angle", new Float(oldAngle),
		    new Float(angle));
	    }
	}
        public float getPlane1MuS() {
	    return plane1.getMuS();
        }
        public void setPlane1MuS(float MuS) {
	    float oldMuS = plane1.getMuS();
	    if (MuS != oldMuS) {
		plane1.setMuS(MuS);
		refresh();
		changes.firePropertyChange("plane1MuS", new Float(oldMuS),
		    new Float(MuS));
	    }
	}
        public float getPlane1MuK() {
	    return plane1.getMuK();
        }
        public void setPlane1MuK(float MuK) {
	    float oldMuK = plane1.getMuK();
	    if (MuK != oldMuK) {
		plane1.setMuK(MuK);
		refresh();
		changes.firePropertyChange("plane1MuK", new Float(oldMuK),
		    new Float(MuK));
	    }
	}

    	public float getPlane2Height() {
    	    return plane2.getHeight();
        }
        public void setPlane2Height(float height) {
	    float oldHeight = plane2.getHeight();
	    if (height != oldHeight) {
		plane2.setHeight(height);
		refresh();
		changes.firePropertyChange("plane2Height", new Float(oldHeight),
		    new Float(height));
	    }
	}
        public float getPlane2Length() {
	    return plane2.getLength();
        }
        public void setPlane2Length(float length) {
	    float oldLength = plane2.getLength();
	    if (length != oldLength) {
		plane2.setLength(length);
		refresh();
		changes.firePropertyChange("plane2Length", new Float(oldLength),
		    new Float(length));
	    }
	}
        public float getPlane2Angle() {
	    return plane2.getAngle();
        }
        public void setPlane2Angle(float angle) {
	    float oldAngle = plane2.getAngle();
	    if (angle != oldAngle) {
		plane2.setAngle(angle);
		refresh();
		changes.firePropertyChange("plane2Angle", new Float(oldAngle),
		    new Float(angle));
	    }
	}
        public float getPlane2MuS() {
	    return plane2.getMuS();
        }
        public void setPlane2MuS(float MuS) {
	    float oldMuS = plane2.getMuS();
	    if (MuS != oldMuS) {
		plane2.setMuS(MuS);
		refresh();
		changes.firePropertyChange("plane2MuS", new Float(oldMuS),
		    new Float(MuS));
	    }
	}
        public float getPlane2MuK() {
	    return plane2.getMuK();
        }
        public void setPlane2MuK(float MuK) {
	    float oldMuK = plane2.getMuK();
	    if (MuK != oldMuK) {
		plane2.setMuK(MuK);
		refresh();
		changes.firePropertyChange("plane2MuK", new Float(oldMuK),
		    new Float(MuK));
	    }
	}

        public float getFloorLength() {
	    return floor.getLength();
        }
        public void setFloorLength(float length) {
	    float oldLength = floor.getLength();
	    if (length != oldLength) {
		floor.setLength(length);
		refresh();
		changes.firePropertyChange("floorLength", new Float(oldLength),
		    new Float(length));
	    }
	}
        public float getFloorMuS() {
	    return floor.getMuS();
        }
        public void setFloorMuS(float MuS) {
	    float oldMuS = floor.getMuS();
	    if (MuS != oldMuS) {
		floor.setMuS(MuS);
		refresh();
		changes.firePropertyChange("floor2MuS", new Float(oldMuS),
		    new Float(MuS));
	    }
	}
        public float getFloorMuK() {
	    return floor.getMuK();
        }
        public void setFloorMuK(float MuK) {
	    float oldMuK = floor.getMuK();
	    if (MuK != oldMuK) {
		floor.setMuK(MuK);
		refresh();
		changes.firePropertyChange("floor2MuK", new Float(oldMuK),
		    new Float(MuK));
	    }
	}

        public float getMovingBodyHeight() {
	    return ((Block)MB).getHeight();
        }
        public void setMovingBodyHeight(float height) {
	    float oldHeight = ((Block)MB).getHeight();
	    if (height != oldHeight) {
		((Block)MB).setHeight(height);
		((Block)MB).reset();
		refresh();
		changes.firePropertyChange("movingBodyHeight", 
		    new Float(oldHeight),
		    new Float(height));
	    }
	}
        public float getMovingBodyLength() {
	    return ((Block)MB).getLength();
        }
        public void setMovingBodyLength(float length) {
	    float oldLength = ((Block)MB).getLength();
	    if (length != oldLength) {
		((Block)MB).setLength(length);
		((Block)MB).reset();
		refresh();
		changes.firePropertyChange("movingBodyLength", 
		    new Float(oldLength),
		    new Float(length));
	    }
	}

        public float getGravity() {
	    return ((Block)MB).getGravity();
        }
        public void setGravity(float gravity) {
	    float oldGravity = ((Block)MB).getGravity();
	    if (gravity != oldGravity) {
		((Block)MB).setGravity(gravity);
		((Block)MB).reset();
		refresh();
		changes.firePropertyChange("gravity", 
		    new Float(oldGravity),
		    new Float(gravity));
	    }
	}

    public boolean action(Event e, Object arg) {
    	Object target = e.target;
    	if (target == Button_run) {
      	    //runSimulation(true);
	    setExecutionState(RUNNING);
      	    return true;
    	}
        if (target == Button_pause) {
      	    //pauseSimulation();
	    setExecutionState(PAUSED);
      	    return(true);
    	}
    	if (target == Button_reset) {
      	    //resetSimulation();
	    setExecutionState(STOPPED);
      	    return(true);
    	}
    	return false;
    }

    public int getExecutionState() {
	return executionState;
    }

    public void setExecutionState(int state) {
	if (state != executionState) {
	    int oldState = executionState;
	    executionState = state;
	    if (state == RUNNING)
      	    	runSimulation(true);
	    else if (state == PAUSED)
		pauseSimulation();
	    else
		resetSimulation();
		
	    changes.firePropertyChange("executionState", new Integer(oldState),
		new Integer(state));
	}
    }

  public void runSimulation(boolean iPressedRun) {
      Started = true;
      Button_run.disable();
      resetSimVars();
      if (thread == null) {
	  runCount+=1;
          startNewTable(runCount);   // part of quick hack 5/23/97 - Bo

	runCounter.setText(new Integer(runCount).toString());

	RecordUserSettings();
	thread = new SimulationThread(simCanvas, MB, this, getTimeStep());
	thread.start();
      }
      else {
	thread.resume();			   
      }
  }

  public void pauseSimulation() {
    debug("pauseSimulation begin");
      if (thread != null) {
	thread.suspend();
	Button_run.setLabel("Resume");
	Button_run.enable();
	Button_run.validate();
	//	Button_run.resize(Button_run.preferredSize());
	validate();
      }
    debug("pauseSimulation end");
  }

  public void resetSimulation() {
    debug("resetSimulation begin");
      if (thread != null) {
	Button_run.enable();
	Button_run.setLabel("Run");
	Button_run.resize(Button_run.preferredSize());
	validate();
	Started = false;
	thread = null;

	MB.reset();
	simCanvas.repaint(50);
      }
    debug("resetSimulation end");
  }

   //bo
  public void update(Graphics g) { 
    //    System.out.println("bnp update"); 
    paint(g); 
  }
  /*
  public void paint(Graphics g) {
    System.out.println("bnp paint");
    simCanvas.paint(g);
  }
  */

  public void RecordUserSettings() {
    addHeader("Plane1 Length",new Float(plane1.getLength()).toString());
    addHeader("Plane1 Height",new Float(plane1.getHeight()).toString());
    addHeader("Plane1 Angle",new Float(plane1.getAngle()).toString());
    addHeader("Plane1 MuS",new Float(plane1.getMuS()).toString());
    addHeader("Plane1 MuK",new Float(plane1.getMuK()).toString());
    addHeader("Plane2 Length",new Float(plane2.getLength()).toString());
    addHeader("Plane2 Height",new Float(plane2.getHeight()).toString());
    addHeader("Plane2 Angle",new Float(plane2.getAngle()).toString());
    addHeader("Plane2 MuS",new Float(plane2.getMuS()).toString());
    addHeader("Plane2 MuK",new Float(plane2.getMuK()).toString());
    addHeader("Floor Length",new Float(floor.getLength()).toString());
    addHeader("Floor MuS  ",new Float(floor.getMuS()).toString());
    addHeader("Floor MuK",new Float(floor.getMuK()).toString());
    addHeader("Block Length",new Float(((Block)MB).getLength()).toString());
    addHeader("Block Height",new Float(((Block)MB).getHeight()).toString());
    addHeader("Time Step    ",new Float(getTimeStep()).toString());
    addHeader("Gravity      ",new Float(MB.getGravity()).toString());
  }         
  public void addHeader(String ParamName, String Value) {
	if (currentTable != null) {
	    currentTable.setAttribute(ParamName, Value);
	}
  }

  float oldX = 0.0f;   //previous position
  float oldY = 0.0f;   // previous position
  float oldTime = 0.0f;
  float deltaTime = 0.0f;
  double deltaDistance = 0.0f;
  double distance = 0.0f;
  double oldDistance = 0.0f;
  double velocity = 0.0f;
  double deltaVelocity = 0.0f;
  double oldVelocity = 0.0f;
  double acceleration = 0.0f;
  int timeCounter = 0;

  private void resetSimVars() {
    oldX = 0.0f;   //previous position
    oldY = 0.0f;   // previous position
    oldTime = 0.0f;
    deltaTime = 0.0f;
    deltaDistance = 0.0f;
    distance = 0.0f;
    velocity = 0.0f;
    deltaVelocity = 0.0f;
    oldVelocity = 0.0f;
    acceleration = 0.0f;
    timeCounter = 0;
  }

  public void addRecord(float x,float y,float time) {
    debug("addRecord");
    timeCounter++;
    
    deltaDistance = Math.sqrt((x-oldX)*(x-oldX) + (y-oldY)*(y-oldY));
    //deltaDistance = distance-oldDistance;
    //oldDistance = distance;
    oldX = x;
    oldY = y;

    distance += Math.abs(deltaDistance);

    deltaTime = getTimeStep();
    if (timeCounter > 1) 
      velocity = deltaDistance/deltaTime;
    
    deltaVelocity = velocity-oldVelocity;
    oldVelocity = velocity;
    if (timeCounter > 2)
      acceleration = (deltaVelocity)/deltaTime;
    
	if (currentTable != null) {
	    try {
	        currentTable.addObservation(time, distance, velocity,
		    acceleration);
	    }
	    catch(InterchangeException e) {
		e.printStackTrace();
	    }
	}
  }

  public Dimension minimumSize() {
    return new Dimension(450,200);
  }

/*
  public Dimension preferredSize() {
    return minimumSize();
  }
*/

 

    //--------------------------------------

    Table tableOfTables = null; 
    SimTable currentTable  = null;

    void setupTable() {
      	// Setup the table
	tableOfTables = new sieve.sluice.local.TableImpl();
	FieldDescriptor fd = new FieldDescriptor("Simulation Results");
	fd.setType("sieve.sluice.TableView");
	try {
	    tableOfTables.addField(fd);
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
    }

    void startNewTable(int runNumber) {
	try {
	    currentTable = new SimTable(runNumber);
	    Object[][] record= {{currentTable}};
	    tableOfTables.append(record);
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
    }

    //------ TableView interface -----------------
    public TableView table() throws NoSourceException {
	return tableOfTables.table();
    }

    /**
     * Returns the TableDescriptor for this view's representation of
     * the table.
     */
    public TableDescriptor getTableDescriptor() throws NoSourceException {
	return tableOfTables.getTableDescriptor();
    }

    /**
     * Returns the names of fields in this view. These names should be
     * used to reference fields in the <CODE>get</CODE> methods.
     */
    public String[] getFieldNames() throws NoSourceException {
	return tableOfTables.getFieldNames();
    }

    /**
     * Returns the FieldDescriptor for the specified field.
     */
    public FieldDescriptor getFieldDescriptor(String field) 
        throws InterchangeException {
	return tableOfTables.getFieldDescriptor(field);
    }

    /**
     * Returns number of records in the view.
     */
    public int getRecordCount() throws NoSourceException {
	return tableOfTables.getRecordCount();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>. The resulting array
     * is ordered by <CODE>[field][record]</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt, int numberOfRecords) 
        throws InterchangeException {
	return tableOfTables.get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>. (I still don't
     * like this because it doesn't add anything. _Every_ implementation
     * will almost certainly just have a call to the three-parm get...)
     */
    public Object[][] get(String[] fields, int startingAt) 
        throws InterchangeException {
	return tableOfTables.get(fields, startingAt);
    }
   
    /* -------------  Event-related methods ------------- */
    public void addTableViewListener(TableViewListener listener) {
	tableOfTables.addTableViewListener(listener);
    }

    public void removeTableViewListener(TableViewListener listener) {
	tableOfTables.removeTableViewListener(listener);
    }
    public void tableViewChanged(TableViewEvent event) {
	// We should never actually listen to anyone
	System.err.println("Block and Plane is (and shouldn't be) listening " +
	    " to " + event.getSource());
    }

    /* -------------  Methods for link support ------------- */
    public void addSourceView(TableView source) throws TooManySourcesException {
        throw new TooManySourcesException();
    }
    public void removeSourceView(TableView source) {
    }

  //------ End TableView interface -----------------


  //--------------------------------------
  // End quick hack
  //--------------------------------------

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }

    //----------------------------------------------------------------------
    // Methods for registering listeners:
    /**
     * The specified PropertyChangeListeners <b>propertyChange</b> method will
     * be called each time the value of any bound property is changed.
     *
     * @see #removePropertyChangeListener
     * @param l the PropertyChangeListener
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        changes.addPropertyChangeListener(l);
    }
 
    /**
     * Remove this PropertyChangeListener from the JellyBeans internal list.
     * If the PropertyChangeListener isn't on the list, silently do nothing.
     *
     * @see #addPropertyChangeListener
     * @param l the PropertyChangeListener
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        changes.removePropertyChangeListener(l);
    }
}



/**
  SimultaionThread handles running the simulation when Run is pressed.
 */
class SimulationThread extends Thread {
  private SimCanvas simCanvas = null;
  private  MovingBody MB = null;
  private BlockAndPlane bnp = null;
  private float timeStep = 0.0f;

  SimulationThread(SimCanvas simCanvas, MovingBody MB, 
		   BlockAndPlane bnp, float timeStep) {
    this.simCanvas = simCanvas;
    this.MB = MB;
    this.bnp = bnp;
    this.timeStep = timeStep;
  }

  public void run() {
    float t = 0;
    while ((SimCanvas.ScaleFactor*MB.getX_CM() < simCanvas.size().width) && 
	   bnp.Started ) {
      MB.move(t);				     
      bnp.addRecord(MB.getX_CM(), MB.getY_CM(), t);
      //      System.out.println("pre simCanvas.repaint()");
      simCanvas.repaint(50);
      try {
	sleep((long)Math.round(timeStep*1000));
      } catch(InterruptedException e) {
	System.out.println(e);
      }
      t += timeStep;
    }
  }


}
