/*
 *
 * Written By: 	Mike Parks
 * Date:			6/29/96
 * Description:	MovingBody Class
 *
 * Usage Notes:
 *
 * 
 */
 /*
 *
 * Copyright (c) 1996 Michael Parks. All Rights Reserved.
 *
 * 
 */ 
package sieve.phys.lwblockandplane;
 
import java.awt.*;  

  
public abstract class MovingBody {

  protected float Gravity = 9.8f;

  // Reset the body?
  protected  boolean reset = false;

  // The x and y coordinates of the body's CM
  protected  float x,y;
  protected  float a=0; // Acceleration
  protected  float v=0; // Velocity

  // The plane we're currently on and its info
  protected  Plane plane;
  
  // The last plane we were on
  protected  Plane oldplane;
  protected  float MuS,MuK;
  
  // Last known size of graphics object
  protected  Dimension D = null;
  protected BlockAndPlane bnp = null;
	
  public MovingBody(BlockAndPlane bnp) {
    this(bnp, "NoName");
  }

  public MovingBody(BlockAndPlane bnp, String name) {
    this.bnp = bnp;
    // figure out which plane we're on 
    if (bnp.getPlane1() != null) {
      plane = bnp.getPlane1();
      MuS = plane.getMuS();
      MuK = plane.getMuK();
    }
    else {
      plane = null;
      MuS = 0.40f;
      MuK = 0.35f;
    }
  }   	

  public float getGravity() {
    return Gravity;
  }

  public void setGravity(float NewGravity) {
    Gravity = NewGravity;
  }

  public void reset() {
    reset = true;
  }

  public  float getX_CM() {
    return(x);
  }
  
  public  float getY_CM() {
    // Return position above floor, not from (0,0))
    float ypos = (D.height - SimCanvas.ScaleFactor*y);
    ypos = ypos/SimCanvas.ScaleFactor;
    return ypos;
  }

  public abstract void move(float t);
  public abstract void paint(Graphics g, Dimension d);
}
