package sieve.quarry;

/**
 * The BeanBox is a reference container for JavaBeans.  It has two purposes.
 * First, it allows bean developers to check that their bean correctly
 * implements the various bean APIs.  Second, it provides a reference example
 * container that can be copied and modified freely.
 * <p>
 * The BeanBox allows beans to be manipulated visually.  You can use the
 * associated property sheet to edit the exposed properties, you can use the
 * edit/events menu to connect events from am event source to an event sink,
 * or you can use the edit/customize menu to test a customizer.
 * <p>
 * The BeanBox is itself a bean, and you can recursively embed a BeanBox
 * inside a BeanBox.  At the top-level, a BeanBox is embedded inside a
 * BeanBoxFrame, which handles things like the MenuBar.
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*; 
import java.io.*;
import java.util.*;
import java.beans.*;
import java.net.*;
import java.lang.reflect.Method;

import sieve.sluice.*;
import sieve.crew.*; // Just need to get rid of NuggetInfo ref and this can go
import sieve.quarry.event.*;

public class BeanBox extends Panel 
implements Serializable,  Runnable,  vt.cs.widgets.MURadarPaneParent, 
  vt.cs.widgets.Scalable, MouseListener, MouseMotionListener {
    QuarryLinkListenerSupport linkListeners = null;
    QuarryNuggetListenerSupport nuggetListeners = null;

    Hashtable atts = null;

    Link currLink = null;

    /** 
     * Initialize a new BeanBox.
     */
    public BeanBox(Hashtable atts) {
	linkListeners = new QuarryLinkListenerSupport(this);
	nuggetListeners = new QuarryNuggetListenerSupport(this);

    	this.atts = atts;
    	addMouseListener(this);
    	addMouseMotionListener(this);

    	setLayout(null);

        setName("Quarry Workspace");
    }


    //------------- MURadarPaneParent interface --------
    public void updateBounds(Rectangle bounds){ 
	// MU stuff is implemented in the QuarryBox subclass
    }
    //------------- end MURadarPaneParent interface --------

    public void update(Graphics g) {
        debug("update");
    	paint(g);
    }

    CheckboxMenuItem fastRadarView = new CheckboxMenuItem("Fast Radar View", true);

    /**
     * Update the MenuBar for the current beanBox and focus bean.
     */
    public synchronized void updateMenuBar(MenuBar bar) {
	BeanBoxFrame frame = getFrame();
    
	if (bar.getMenuCount() == 0) {
	    // Create the three basic menus.  
	    // These are bean/beanbox independent.

	    Menu m = new Menu("File");
	    addMenuItem(frame, m, new MenuItem("Save..."));
            addMenuItem(frame, m, new MenuItem("SerializeComponent..."));
            addMenuItem(frame, m, new MenuItem("Load..."));
            addMenuItem(frame, m, new MenuItem("LoadJar..."));
            //addMenuItem(frame, m, new MenuItem("Clear"));
            addMenuItem(frame, m, new MenuItem("Exit"));

	    bar.add(m);

	    m = new Menu("Edit");
	    addMenuItem(frame, m, new MenuItem("Cut"));
            //addMenuItem(frame, m, new MenuItem("Copy"));
            //addMenuItem(frame, m, new MenuItem("Paste"));
            addMenuItem(frame, m, new MenuItem("Report..."));
	    m.addSeparator();
	    addMenuItem(frame, m, new MenuItem("Bring Forward"));
	    addMenuItem(frame, m, new MenuItem("Bring to Front"));
	    addMenuItem(frame, m, new MenuItem("Send Backwards"));
	    addMenuItem(frame, m, new MenuItem("Send to Back"));
	    m.addSeparator();
	    bar.add(m);

	    m = new Menu("View");
	    bar.add(m);

	    m = new Menu("Options");
	    addMenuItem(frame, m, fastRadarView);
	    bar.add(m);

	}

	Menu editMenu = bar.getMenu(1);

	// Remove any bean-specific menu items from the edit menu.
	for (int i = 0; i < editMenu.getItemCount(); i++) {
	    String label = editMenu.getItem(i).getLabel();
	    if (label.equals("Customize...") || label.equals("Events") ||
			label.equals("Bind property...")) {
		editMenu.remove(i);
		i--;
	    }
	}
	customizerClass = null;
	getFrame().setCustomizer(null);
	eventMap = null;
	methodMap = null;

	resetViewMenu();

	// Take a look at the current focus bean, and create appropriate
	// menu items for its events, customizer, etc.

	try {
	    Object bean = getFrame().getCurrentBean();
	    BeanInfo bi = Introspector.getBeanInfo(bean.getClass());

	    try {
		BeanDescriptor bd = bi.getBeanDescriptor();
	        customizerClass = bd.getCustomizerClass();
		if (customizerClass != null) {
		    addMenuItem(frame, editMenu, new MenuItem("Customize..."));
		}
	    } catch (Exception ex) {
		error("Couldn't initialize customizer", ex);
	    }

	    java.beans.EventSetDescriptor esd[] = bi.getEventSetDescriptors();
	    boolean bindable = false;
	    if (esd.length > 0) {
	        Menu eventMenu = new Menu("Events");
		eventMap = new Hashtable();
		methodMap = new Hashtable();
		// We create a separate menu for each event listener interface,
		// with a menu item for each event handler method.
	        for (int i = 0; i < esd.length; i++) {
		    String dname = esd[i].getDisplayName();
		    Menu methodMenu = new Menu(dname);
		    eventMap.put(methodMenu, esd[i]);
		    Method methods[] = esd[i].getListenerMethods();
		    for (int j = 0; j < methods.length; j++) {
			String mname = methods[j].getName();
			MenuItem mi = new MenuItem(mname);
			addMenuItem(frame, methodMenu, mi);
			methodMap.put(mi, methods[j]);
	    	    }
		    eventMenu.add(methodMenu);
		    if (esd[i].getName().equals("propertyChange")) {
			bindable = true;
		    }
	        }
	        editMenu.add(eventMenu);
	    }

	    if (bindable) {
		addMenuItem(frame, editMenu, new MenuItem("Bind property..."));
	    }

	} catch (IntrospectionException ex) {
	    error("Caught unexpected exception while building event menu", ex);
	}
    }

    protected void addMenuItem(BeanBoxFrame frame, Menu m, MenuItem mi) {
        // Register the BeanBoxFrame as the event handler.
        mi.addActionListener(frame);
        // Add the menuitem to the menu
        m.add(mi);
    }

    /**
     * This implements the "close" menu item.  If we are the top-level BeanBox
     * then we also shut down our parent BeanBoxFrame.
     */
    public void close() {
	BeanBoxFrame frame = getFrame();
	if (frame == getParent()) {
	    // We're the top-level BeanBox in a BeanBoxFrame.  Blow away
	    // the BeanBoxFrame.
	    frame.dispose();
	} else {
	    Container wrapper = getParent();
	    Container parent = wrapper.getParent();
	    parent.remove(wrapper);	 
	    parent.layout();
	}
    }

    /**
     * Provide an identifier for serializing this object
     */
    private synchronized String serFileName(Object o) {
	if (o == null) {
	    // distinguished root-component
	    return getFrame().getTmpDir()+File.separator+"___comp_ROOT.ser";
	} else {
	    String s = (String) serNames.get(o);
	    if (s == null) {
		String name = "___comp_"+HookupManager.getId()+".ser";
		s = getFrame().getTmpDir()+File.separator+name;
		serNames.put(o, s);
	    }
	    return s;
	}
    }

    private boolean generateManifestTemplate(File file) {
	File dir = new File(file.getParent());
	if (dir != null) {
	    dir.mkdirs();
	}
	OutputStream os;
	try {
	    os = new FileOutputStream(file);
	    PrintWriter pos = new PrintWriter(os);

	    int count = countComponents();
	    for (int i = 0; i < count; i++) {
		Wrapper w = Wrapper.findWrapper(getComponent(i));
		pos.println("Name: "+serFileName(w).replace(File.separatorChar, '/'));
		pos.println("Java-Bean: True");
		pos.println();
	    }
	    pos.flush();
	    pos.close();

	} catch (Exception ex) {
	    error("Problems creating manifest template file", ex);
	    return false;
	}
	return true;
    }

    /**
     * This implements the "save" menu item.  This stores away the
     * current state of the BeanBox to a named file.
     *
     * DISCLAIMER:  We do not claim that this is the "proper way" to
     * save a collection of inter-dependent beans.  Future BDKs will
     * expand in this area.
     */
    public void save() {
	// write a JAR file that contains all the hookups and a
	// single serialized stream.  That is easy
	// then, on load, read the components and mock with the AppletStub.

	boolean saveAdaptors = true;

	FileDialog fd = new FileDialog(getFrame(), "Save File", FileDialog.SAVE);
	// the setDirectory() is not needed, except for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	// Solaris currently ignores the setFile()
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);	    

	try {
	    // create the single ObjectOutputStream
	    File serFile = new File(serFileName(null));
	    File dir = new File(serFile.getParent());
	    if (dir != null) {
		dir.mkdirs();
	    }
	    FileOutputStream f = new FileOutputStream(serFile);
	    ObjectOutputStream oos = new ObjectOutputStream(f);
	    writeContents(oos);
	    oos.close();

	    // Generate the manifest file?

	    Vector hfiles = new Vector(0);	// hookups
	    if (saveAdaptors) {
		hfiles = HookupManager.getHookupFiles();
	    }
	    String args[] = new String[3+hfiles.size()];

	    args[0] = "cf";
	    args[1] = file.getPath();	// name for the JAR file
	    // no manifest file - so far
	    args[2] = serFileName(null);
	    for (int i=0; i<hfiles.size(); i++) {
		args[3+i] = (String) hfiles.elementAt(i);
	    }

	    sun.tools.jar.Main jartool =
		new sun.tools.jar.Main(System.out, System.err, "jar");

	    boolean ok = jartool.run(args);
	    if (!ok) {
		error("Jar tool invocation failed");
	    }

	} catch (Exception ex) {
	    error("Save failed", ex);
   	}
    }


    /**
     * This implements the "load" menu item.  This loads the BeanBox state
     * from a named file.
     */
    private void load() {
	removeAll();
	FileDialog fd = new FileDialog(getFrame(), "Load File", FileDialog.LOAD);
	// needed for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	// ignored under Solaris due to a bug...
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);

	JarLoader jl = null;
	try {
	    jl = new JarLoader(file.getPath());
	    if (jl == null) {
		System.err.println("file name passed is not a valid JarFile");
		return;
	    }
	} catch (Exception ex) {
	    System.err.println("caught an exception while trying to load "+file.getPath());
	    ex.printStackTrace();
	    return;
	}

	JarInfo ji = jl.loadJar();
	// OK, loaded all the classes -- now, instantiate them...

	try {
	    // get the one object as a bean...
	    ClassLoader cl = jl.getLoader();
	    InputStream is = cl.getResourceAsStream(serFileName(null).replace(File.separatorChar, '/'));
	    ObjectInputStream ois = new ObjectInputStreamWithLoader(is, cl);

	    // Read all the contained beans into this BeanBox.
	    readContents(ois);

	} catch (Exception ex) {
	    ex.printStackTrace();
	    return;
	}
    }


    // Support for serialization

    private void writeContents(java.io.ObjectOutputStream s)
        		throws java.io.IOException {
	int count = countComponents();
	s.writeInt(count);
	for (int i = 0; i < count; i++) {
	    s.writeObject(getComponent(i));
	}
    }


    private void readContents(java.io.ObjectInputStream ois)
        		throws java.lang.ClassNotFoundException,
			       java.io.IOException {
	ClassLoader cl = SimpleClassLoader.ourLoader;

        int count = ois.readInt();

        for (int i=0; i<count; i++) {
	    Component c = (Component) ois.readObject();
	    Wrapper w = (Wrapper) c;

	    Object bean = w.getBean();
	    if (bean instanceof Applet) {
	      System.out.println("bean instance of Applet 1");
		// Assign an AppletStub
		assignAppletStub((Applet) bean, cl, bean.getClass());
	    }

	    // Add the new component to our AWT container.
	    add(c);
	    c.invalidate();

	    if (bean instanceof Applet) {
	      System.out.println("bean instance of Applet 2");
		// start the Applet.
		((Applet)bean).start();
	    }
	}
    }


    // Support for serialization

    private void writeObject(java.io.ObjectOutputStream oos)
        		throws java.io.IOException {
	writeContents(oos);
    }

    private void readObject(java.io.ObjectInputStream ois)
        		throws java.lang.ClassNotFoundException,
			       java.io.IOException {
	mouseDown = new Object();
	readContents(ois);
    }


    // Guess a codebase for a class in a classloader

    private void assignAppletStub(Applet applet, ClassLoader cl, Class k) {
	String className = k.getName();
	String resourceName = className.replace('.','/').concat(".class");

	// This code is identical to portions of Beans.intantiate()
	URL objectUrl = null;
	URL codeBase = null;
	URL docBase = null;

	// Now get the URL correponding to the resource name.
	if (cl == null) {
	    objectUrl = ClassLoader.getSystemResource(resourceName);
	} else {
	    objectUrl = cl.getResource(resourceName);
	}

	// If we found a URL, we try to locate the docbase by taking
	// of the final path name component, and the code base by taking
	// of the complete resourceName.
	// So if we had a resourceName of "a/b/c.class" and we got an
	// objectURL of "file://bert/classes/a/b/c.class" then we would
	// want to set the codebase to "file://bert/classes/" and the
	// docbase to "file://bert/classes/a/b/"
	
	if (objectUrl != null) {
	    String s = objectUrl.toExternalForm();
	    if (s.endsWith(resourceName)) {
		try{ 
		    int ix = s.length() - resourceName.length();
		    codeBase = new URL(s.substring(0,ix));
		    docBase = codeBase;
		    ix = s.lastIndexOf('/');
		    if (ix >= 0) {
			docBase = new URL(s.substring(0,ix+1));
		    }
		} catch (Exception ex) {
		    docBase = codeBase = null;
		}
	    }
	}
	    	    
	// Setup a default context and stub.
	BeanBoxAppletContext context = new BeanBoxAppletContext(applet, getFrame());
	BeanBoxAppletStub stub = new BeanBoxAppletStub(applet, context, codeBase, docBase);
	applet.setStub(stub);
	
	// object is always deserialized
	stub.active = true;
    }
    
    /**
     * This implements the "loadJar" menu item.
     * Load the contents of a JAR file
     */
    private void loadJar() {
	FileDialog fd = new FileDialog(getFrame(), "Load from JAR File", FileDialog.LOAD);
	// the setDirectory() is not needed, except for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	// Solaris currently ignores the setFile()
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);

	getFrame().getToolBox().addBeansInJar(file.getPath());
    }

    /**
     * Serialize the current focus bean to our clipboard file, and
     * then remove it from the BeanBox.
     */
// PLI -- These need to send NuggetInfo messages... Should also require
// unlinking the component before removing it...
    private void cut() {
	Object bean = getFrame().getCurrentBean();
	Component child = getFrame().getCurrentComponent();
	if (bean != getFrame().getTopBox()) {
	    copy();
	    Container parent = child.getParent();
	    if (parent instanceof Wrapper) {
		child = parent;
		parent = parent.getParent();
	    }
	    getFrame().setCurrentComponent(null);
	    if (parent != null) {
		parent.remove(child);
		parent.layout();
	    }
	}
        repaint();
    }

    /**
     * Print a report on the current focus bean.
     */
    private void report() {
	Object bean = getFrame().getCurrentBean();
	if (bean == null) {
	    System.out.println("No current focus.");
	    return;
	}
        if (bean instanceof TableView) 
	    TableViewReport.report(bean);
        else
	    Report.report(bean.getClass());
System.out.println("Bounds: " + getFrame().getCurrentComponent().getParent().getBounds());
      
    }

    /**
     * Serialize the current focus bean to our clipboard file.
     */
    private void copy() {
	Object bean = getFrame().getCurrentBean();
	if ((bean == null) 
	    || ! (bean instanceof Serializable)) { // PLI 
	    return;
	}
	try {
	    File f = new File(getFrame().getClipFileName());
	    File dir = new File(f.getParent());
	    dir.mkdirs();
            FileOutputStream fos = new FileOutputStream(f);
            ObjectOutput s = new ObjectOutputStream(fos);
   	    s.writeObject(bean);
	    fos.close();
	} catch (Exception ex) {
	    error("Copy failed", ex);
   	}
    }

    /**
     * Read in a bean from our clipboard file and insert it into
     * the BeanBox.
     */
    private void paste() {
      debug("paste");
	try {
	    SimpleClassLoader loader = SimpleClassLoader.ourLoader;
	    MyProducer p = new MyProducer(getFrame().getClipFileName());
	    String clipName = getFrame().getClipResource();

	    loader.setLocalResourceSource(clipName+".ser", p);
	    Object bean = Beans.instantiate(loader, clipName);
	    doInsert(bean);
	} catch (Exception ex) {
	    error("Paste failed", ex);
   	}
	invalidate();
	layout();
        getFrame().invalidate();
	getFrame().layout();
	repaint();
    }

    /* -------- Methods for registering listners ------- */
    public void addQuarryLinkListener(QuarryLinkListener listener) {
	linkListeners.addListener(listener);
    }
    public void removeQuarryLinkListener(QuarryLinkListener listener) {
	linkListeners.addListener(listener);
    }

    public void addQuarryNuggetListener(QuarryNuggetListener listener) {
	nuggetListeners.addListener(listener);
    }
    public void removeQuarryNuggetListener(QuarryNuggetListener listener) {
	nuggetListeners.addListener(listener);
    }

    /* ------ END Methods for registering listners ----- */

    /**
     * Implementation method to make a connection between two beans.
     * We draw a rubber banded line from a source bean until the mouse
     * is clicked on a target bean.
     * @return  the selected target bean.
     */

    private Wrapper getConnection() {
	connectionSource = getFrame().getCurrentComponent();

	synchronized (mouseDown) {
	    try {
	        mouseDown.wait();
	        connectionSource = null;
		Component target = mouseDownEvent.getComponent();
		return Wrapper.findWrapper(target);
	    } catch (Exception ex) {
	        connectionSource = null;
		return null;
	    }
	}
    }

    /**
     * Create a link between two components.
     */
    public Link doLinkRemote(Wrapper source, Object dest) {
    	Link link = new Link(source, null);
        Dimension size = getSize();
        link.setBounds(0, 0, size.width, size.height);
        add(link, -1);
	finishLink(link, dest);
	return link;
    }

    public void doLink() {
	Component wr = getFrame().getCurrentComponent().getParent();
	if (wr instanceof Wrapper) {
            Wrapper source = (Wrapper)  wr;
	    if (source.getBean() != this)
	        doLink(new Link(source, null), source);
	}
    }
    
    public void doLink(Link link, Object source) {
	currLink = link;
        Dimension size = getSize();
        link.setBounds(0, 0, size.width, size.height);
        add(link, -1);

        Wrapper target = getConnection();

	// This is temporary. When we eventually support different types of 
	// links we will need to re-think what constitutes a duplicate
	// link. We will also need to consider that some link types 
	// (e.g., those in the circuit simulator) will be undirected.
	if ((Link.findLink(((Wrapper)source).getBean(), target.getBean())
	    != null)  || (((Wrapper)source).getBean() == target.getBean()) ) {
	    System.out.println("Ignoring duplicate or recursive link");
	    link.removeSource();
	    currLink = null;
	    repaint();
	    getFrame().scroller.repaint();
	    return;
	}

	finishLink(link, target);

	// It might be a good idea to have these events fired by the
	// link class somehow. This was handle cases where links 
	// "break themselves", e.g., when a bean is cut from the workspace.
	// As cutting isn't properly handled now anyway, this isn't
	// yet an issue.

	// If this was a link removal, then the source of the link object
	// would have been cleared in finishLink.
	if (link.getSource() != null) { 
	    // Don't fire an event if the link is being re-made to itself
	    if ((link.getLastDest() == null) ||
	        (link.getLastDest().getBean() != target.getBean())) {

		// Handle the case where a link was switched between two
		// beans
		if ((link.getLastDest() != null) && 
		    (link.getLastDest().getBean() != null))
	    	    linkListeners.linkRemoved(((Wrapper)source).getBean(), 
			link.getLastDest().getBean());

	        linkListeners.linkAdded(((Wrapper)source).getBean(), 
		    target.getBean());
	    }
	}
	else if (link.getLastDest() != null)
	    linkListeners.linkRemoved(((Wrapper)source).getBean(), 
		link.getLastDest().getBean());
    }

    private void finishLink(Link link, Object targ) {
	Wrapper target = (Wrapper) targ;
        Object source = link.getSourceBean();

	// Eventually we'll possibly want to have the link throw an exception
	// if there is no way for the two beans to be linked.
	if (target.getBean() != this)
	    link.setDest(target);
	else
	    link.removeSource();

	currLink = null;
	repaint();
	getFrame().scroller.repaint();
    }

    /**
     * Bind a source property to a target property.
     */
    void doBind() {

	Object bean = getFrame().getCurrentBean();
	PropertyNameDialog dialog = new PropertyNameDialog(getFrame(),
				bean, "Select source property", null, true);
	PropertyDescriptor sourceProperty = dialog.getResult();
	if (sourceProperty == null) {
	    return;
	}

 	Object target = getConnection().getBean();
	if (target == null) {
	    return;
	}	

	dialog = new PropertyNameDialog(getFrame(),
		 target, "Select target property",
		 sourceProperty.getPropertyType(), false);

	PropertyDescriptor targetProperty = dialog.getResult();

	if (targetProperty == null) {
	    return;
	}

        PropertyHookup.attach(bean,
			sourceProperty.getName(),
			sourceProperty.getReadMethod(), target,
			targetProperty.getWriteMethod());
    }

    /**
     * Create an event connection from source to target
     */

    void doEventHookup(ActionEvent evt) {	
	Object source = getFrame().getCurrentBean();

	MenuItem mi = (MenuItem)evt.getSource();
	Menu parent = (Menu)mi.getParent();

	EventSetDescriptor esd = (EventSetDescriptor) eventMap.get(parent);
        Method meth = (Method) methodMap.get(mi);

	if (esd == null || meth == null) {
	    return;
	}

 	Object target = getConnection().getBean();
	if (target == null) {
	    return;
	}	

        new EventTargetDialog(getFrame(), source, target, esd, meth);
    }

    /**
     * Get the address of a target component relative to the 
     * current BeanBox.
     */

    private Rectangle getLocalCoordinates(Component c) {
	int width = c.size().width;
	int height = c.size().height;
	int x = 0;
	int y = 0;
	for (;;) {
	    if (c == null) {
		return new Rectangle(0,0,0,0);
	    }
	    if (c == this) {
		return new Rectangle(x, y, width, height);
	    }
	    x += c.location().x;
	    y += c.location().y;
	    c = c.getParent();
	}
    }

    /**
     * Paint a rubber-banded line from our current connection
     * source "connectionSource" to the sepcified newX, newY.
     */

    private void paintDraggedConnection(int newX, int newY) {
	unpaintDraggedConnection();

	Graphics g = getGraphics();
	if (g == null) {
	    return;
	}
	g.setColor(Color.red);

	Rectangle b = getLocalCoordinates(connectionSource);
	int x2 = b.x + (b.width/2);
	int y2 = b.y + (b.height/2);
	g.drawLine(newX, newY, x2, y2);
	g.drawLine(newX+1, newY+1, x2+1, y2+1);

	oldX1 = newX;
	oldX2 = x2;
	oldY1 = newY;
	oldY2 = y2;
    }

    /**
     * Erase any outstanding connection rubber-band line.
     */

    private void unpaintDraggedConnection() {
	Graphics g = getGraphics();
	if (g == null || oldX1 < 0) {
	    return;
	}
	g.setColor(getBackground());
	g.drawLine(oldX1, oldY1, oldX2, oldY2);
	g.drawLine(oldX1+1, oldY1+1, oldX2+1, oldY2+1);
	oldX1 = -1;
    }

  Image offscreen = null;
  Object offScreenLock = new Object();

  /**
   * null out the offscreen buffer as part of invalidation
   */
  public void invalidate() {
    debug("invalidate");
    synchronized(offScreenLock) {
      debug("invalidate have lock");
      super.invalidate();
      //      offscreen = null;  // interferes with paint(g) below!!! -Bo
    }
    debug("end invalidate");
  }


  private Rectangle scaleRectangle(Rectangle toScale,
				   Dimension relativeSize,
				   Dimension fullSize) {
    debug("scaleRectangle "+toScale+fullSize+relativeSize);
    if ((toScale == null) || (relativeSize == null) || (fullSize == null))
      return null;

    float horizScale = (float)relativeSize.width/(float)fullSize.width;
    float vertScale = (float)relativeSize.height/(float)fullSize.height;
    int scaledX = Math.round(toScale.x * horizScale);
    int scaledWidth = Math.round(toScale.width * horizScale);
    if ((scaledWidth+scaledX)>(relativeSize.width-1)) 
      scaledWidth = relativeSize.width-1-scaledX;
    int scaledY = Math.round(toScale.y * vertScale);
    int scaledHeight =  Math.round(toScale.height * vertScale);
    if ((scaledHeight+scaledY)>(relativeSize.height-1))
      scaledHeight = relativeSize.height-1-scaledY;

    Rectangle scaled =  new Rectangle(scaledX, scaledY, 
				      scaledWidth, scaledHeight);
    debug("scaleRectangle "+scaled);
    return scaled;
  }

  private  Point scalePoint(Point toScale,
			    Dimension relativeSize,
			    Dimension fullSize) {
    debug("scalePoint "+toScale+fullSize+relativeSize);
    if ((toScale == null) || (relativeSize == null) || (fullSize == null))
      return null;

    float horizScale = (float)relativeSize.width/(float)fullSize.width;
    float vertScale = (float)relativeSize.height/(float)fullSize.height;
    int scaledX = Math.round(toScale.x * horizScale);
    int scaledY = Math.round(toScale.y * vertScale);
    return new Point(scaledX, scaledY);
 }


  // ------------ Scalable methods ---------------
  public void paintScaled(Graphics g, Dimension size) {
    debug("scaling....");
    Rectangle scaleBounds = new Rectangle(0, 0, size.width, size.height);
    Rectangle fullBounds = getBounds();

    // A Windows AWT bug prevent clearRect from working correctly. 
    // Instead use the following to blank the background.
    g.setColor(getBackground());
    g.fillRect(0, 0, scaleBounds.width, scaleBounds.height);

    Component[] comps = getComponents();

    for (int i = 0; i< comps.length; i++) {
      if (comps[i] instanceof Link){
	Rectangle sourceBounds = null;
	Rectangle destBounds = null;
	Component tComp = (Component)((Link)comps[i]).getSource();
	if (tComp != null)
	  sourceBounds = tComp.getParent().getBounds();
	tComp = (Component)((Link)comps[i]).getDest();
        if (tComp != null)
	  destBounds = tComp.getParent().getBounds();
	if ((sourceBounds != null) && (destBounds != null)) {
	  Point sourcePoint = scalePoint(
                   new Point(sourceBounds.x+(sourceBounds.width/2),
			     sourceBounds.y+(sourceBounds.height/2)),
		   new Dimension(scaleBounds.width, scaleBounds.height),
		   new Dimension(fullBounds.width, fullBounds.height));
	  Point destPoint = scalePoint(
		   new Point(destBounds.x+destBounds.width/2, 
			     destBounds.y+destBounds.height/2), 
		   new Dimension(scaleBounds.width, scaleBounds.height),
		   new Dimension(fullBounds.width, fullBounds.height));
	  g.setColor(Color.black);
	  debug("sourcePoint "+sourcePoint+" destPoint "+destPoint);
	  g.drawLine(sourcePoint.x, sourcePoint.y, destPoint.x, destPoint.y);
	}
      }
    }
    for (int i = 0; i< comps.length; i++) {
      if (!(comps[i] instanceof Link)) {
	Rectangle compBounds = scaleRectangle(comps[i].getBounds(),
					 new Dimension(scaleBounds.width, scaleBounds.height),
					 new Dimension(fullBounds.width, fullBounds.height));
	if ((comps[i] instanceof Wrapper) && (((Wrapper)comps[i]).getBean() 
	   instanceof Component)) {
	  Component bean = (Component)((Wrapper)comps[i]).getBean();
	  g.setColor(bean.getBackground().darker());
	  debug("bean bg = "+ bean.getBackground());
	} else {
	    g.setColor(comps[i].getBackground());
	}
	g.fillRect(compBounds.x, compBounds.y, compBounds.width, compBounds.height);
	g.setColor(Color.black);
	g.drawRect(compBounds.x, compBounds.y, compBounds.width, compBounds.height);
      }
    }
  }
  public boolean scaleSelf() {
    return fastRadarView.getState();
  }

  // ------------ end Scalable methods ---------------

  /**
   * Repaint the current beanbox. Actaully all we need to do
   * is take note that any rubber-band lines or boxes have 
   * been removed.
   */
  public void paint(Graphics g) {
    debug("paint");
    oldX1 = -1;
    oldRubberBox = null;
    synchronized(offScreenLock) {
      debug("paint have lock");
      if(offscreen == null) {
	offscreen = createImage(getSize().width, getSize().height);
      }
    debug("paint pre og");
      Graphics og = offscreen.getGraphics();
      og.setClip(0,0,getSize().width, getSize().height);
      og.clearRect(0,0,getSize().width, getSize().height);
    debug("paint pre super.paint");
      super.paint(og);

      debug("pre g.drawImage");
      g.drawImage(offscreen, 0, 0, null);
      og.dispose();
    }
    debug("end paint");
  }


    public synchronized void mouseMoved(MouseEvent evt) {
	int x = evt.getX();
        int y = evt.getY();

	// There should be a better way to do this. To make links and
	// telepointers work properly, we have to translate from the
        // coordinate space of an arbitrarily nested component to 
        // the BeanBox's coordinate space. Yuck.
        if (connectionSource != null) {
	    if (evt.getComponent() != this) {
	        Component last = evt.getComponent();
	        Component curr = last.getParent();
	        while ((last != this) && (curr != null)) {
                    x += last.getLocation().x;
                    y += last.getLocation().y;
		    last = curr;
	            curr = last.getParent();
	    	}
	    }
	    currLink.setCurrXY(x, y);
	    repaint();
        }

    }

    public synchronized void mouseDragged(MouseEvent evt) {

	int x = evt.getX();
        int y = evt.getY();
	Dimension size = getSize();

        // If we're in the middle of a resize, do a rubber-banded
	// resize box.
	if (resizeChild != null) {
            // Make sure the event coordinates are relative to the child.
            if (evt.getComponent() != this) {
                x += resizeChild.getLocation().x;
                y += resizeChild.getLocation().y;
            }
	    if (x > size.width)
		x = size.width;
	    else if (x < 0)
		x = 0;
	    if (y > size.height)
		y = size.height;
	    else if (y < 0)
		y = 0;
	    drawRubberBox(getResizeBox(x, y, true));
        }
	// If we're in the middle of a move, draw a rubber banded box.
	if (moveChild != null) {
            // Make sure the event coordinates are relative to the child.
            if (evt.getComponent() != this) {
                x += moveChild.getLocation().x;
                y += moveChild.getLocation().y;
            }
	    drawRubberBox(getMoveBox(x, y));
	}
    }

    public void mouseClicked(MouseEvent evt) {
        // To workaround #4039858 we do mouseClicked processing in mouseRelease
        // and do nothing here.
    }

    public void mousePressed(MouseEvent evt) {
	synchronized(mouseDown) {
	    mouseDownEvent = evt;
	    mouseDown.notify();
	}
	if (connectionSource != null) {
	    repaint();
	}
    }

    public void mouseReleased(MouseEvent evt) {
	int x = evt.getX();
        int y = evt.getY();

	if (resizeChild != null) {
            if (evt.getComponent() != this) {
                x += resizeChild.getLocation().x;
                y += resizeChild.getLocation().y;
            }
	    deleteRubberBox();
	    finishResize(x, y);
        }
	else if (moveChild != null) {
            if (evt.getComponent() != this) {
                x += moveChild.getLocation().x;
                y += moveChild.getLocation().y;
            }
	    deleteRubberBox();
	    finishMove(x, y);
	}
	else {
            // OK, treat this as a genuine mouse click.
            // Wakeup anyway who is waiting for a mouse down.
            synchronized (this) {
/* 
                mouseClickEvent = new MouseEvent(evt.getComponent(), 
		    evt.getID(), evt.getWhen(), evt.getModifiers(), 
		    evt.getX(), evt.getY(), evt.getClickCount(), 
		    evt.isPopupTrigger());
                notifyAll();
*/
 
                // set the new focus, unless we're in the middle of an insert.
                if (getFrame().getCursor() != crosshairCursor) {
                    Component target = evt.getComponent();
                    getFrame().setCurrentComponent(target);
                }
	    }
	}
    }

    public void mouseEntered(MouseEvent evt) {
    }
 
    public void mouseExited(MouseEvent evt) {
    }

    void shutdown(int exitValue) {
    	System.exit(exitValue);
    }

    /**
     * This method is called from a menuWorkerThread to execute
     * the work associated with a given menu event.  Note that a
     * new MenuWorkerThread is created for each menu event, so they
     * can afford to block while they work their way through the
     * processing associated with the menu action.
     */

    void doMenuItem(ActionEvent evt) {
	MenuItem mi = (MenuItem)evt.getSource();
	Menu parent = (Menu)mi.getParent();
	String label = mi.getLabel();
	debug("parent = "+parent+"   menu item = "+mi);

        if (parent == null) {
            System.err.println("domenuItem: disconnected MenuItem " + mi);
            return;
        }

	if (parent.getLabel().equals("File")) {
	    if (label.equals("Exit")) {
	      shutdown(0);
	    } else if (label.equals("Close")) {
	        close();
	    } else if (label.equals("Clear")) {
	        removeAll();
	        getFrame().setCurrentComponent(null);
	    } else if (label.equals("Save...")) {
	        save();
	    } else if (label.equals("Load...")) {
	        load();
	    } else if (label.equals("LoadJar...")) {
	        loadJar();
	    }
	} 
  	else if (parent.getLabel().equals("Edit")) {
	    if (label.equals("Cut")) {
	        cut();
	    } else if (label.equals("Copy")) {
	        copy();
	    } else if (label.equals("Customize...")) {
		if (customizerClass != null) {
		    try {
		        Customizer customizer = (Customizer) customizerClass.newInstance();
		        BeanBoxFrame frame = getFrame();
			Object bean = frame.getCurrentBean();
		        customizer.setObject(bean);
		        frame.setCustomizer(customizer);
		        new CustomizerDialog(frame, customizer, bean);
		    } catch (Exception ex) {
			System.err.println("Couldn't instantiate customizer: " + ex);
		    }
		}
	    } else if (label.equals("Bind property...")) {
		// Bind a source property to a target property.
		doBind();
	    } else if (label.equals("Paste")) {
	        paste();
	    } else if (label.equals("Report...")) {
		report();
	    } else { 
		Object bean = getFrame().getCurrentBean();
		if (bean != this) {
		    Component selected = getFrame().getCurrentComponent();
		    Component wrapper = selected.getParent();
		    Rectangle bounds = wrapper.getBounds();

		    if (label.equals("Bring to Front")) {
		        remove(wrapper);
		        add(wrapper, 0);
		    }
	     	    else if (label.equals("Send to Back")) {
		        remove(wrapper);
		        add(wrapper);
		    }
		    else {
			int idx = -1;
			for (int i = 0;i < getComponentCount() && idx < 0;i++) {
			    if (getComponent(i) == wrapper)
				idx = i;
			}
			remove(idx);
		    	if (label.equals("Bring Forward") && (idx > 0)) {
			    add(wrapper, idx - 1);
		    	}
		    	else if (label.equals("Send Backwards") &&
			    (idx < getComponentCount() - 1)) {
			    add(wrapper, idx + 1);
			}
			else
			    add(wrapper, idx);
		    }
		    wrapper.setBounds(bounds);
		    selected.repaint();
		}
            }

	} 
/* COLLAB -- This still isn't working quite right in QuarryBox. Remove
  	     this section once that has been fixed.

	else if (parent.getLabel().equals("Share")) {
	  debug("Share menu selection");
	  if (label.equals("Edit Participant Info...")) {
	    editUser();
	  } 
	} else if (parent == telepointerMenu ) {
	  if (mi == showNames) {
	    debug("showNames selected. showNames.getState() = "+showNames.getState());
	    for (Enumeration e = telepointers.elements(); e.hasMoreElements(); ) {
	      Telepointer tp = (Telepointer)e.nextElement();
	      tp.setShowName(showNames.getState());
	    }
	  } else if (mi == showRemotePointers) {
	    debug("showRemotePointers selected. showRemotePointers.getState() = "+showNames.getState());
	    for (Enumeration e = telepointers.elements(); e.hasMoreElements(); ) {
	      Telepointer tp = (Telepointer)e.nextElement();
	      tp.setVisible(showRemotePointers.getState());
	    }
	  } else if (mi == showLocalPointer) {
	    debug("showLocalPointer selected. showLocalPointer.getState() = "+showNames.getState());
	    if (showLocalPointer.getState()) {
	      // Create telepointer that uses an "echo" of the localParticipant
	      Participant localEcho = null;
	      if (participantControl != null) {
		localEcho = participantControl.getLocalEcho();
	      }
	      if ((localEcho == null) && (localParticipant != null)) {
		localEcho = new Participant(localParticipant);
		localParticipant.addObserver(localEcho);  // just observe for name and color changes
		participantControl.setLocalEcho(localEcho);
	      }
	      if (localEcho != null) {
		Telepointer tp = new Telepointer(localEcho);
		tp.setShowName(showNames.getState());
		telepointers.put(localEcho.getIdentifier(), tp);
	      }
	    } else {
	      if (localParticipant != null) {
		Telepointer tp = (Telepointer)telepointers.remove(localParticipant.getIdentifier());
		if (tp != null) {
		  Participant part = tp.getParticipant();
		  part.deleteObserver(tp);
		  localParticipant.deleteObserver(part);
		}
	      }
	    }
	  }
	  repaint();

	} 
*/
	else if (parent.getLabel().equals("View")) {
	    if (label.equals("Disable Design Mode")) {
		getFrame().setDesignTime(false);
	        getFrame().getCurrentComponent().getParent().repaint();
	    } else if (label.equals("Enable Design Mode")) {
		getFrame().setDesignTime(true);
	        getFrame().getCurrentComponent().getParent().repaint();
	    } else if (label.equals("Hide Invisible Beans")) {
		getFrame().setHideInvisibleBeans(true);
	    } else if (label.equals("Show Invisible Beans")) {
		getFrame().setHideInvisibleBeans(false);
	    }

	    resetViewMenu();

	} else {
	    // The only place left is the "events" submenu.
	    // Create an event hookup from a source to a target.
	    doEventHookup(evt);
	}
    }

    /**
     * Schedule a menu item to be executed asynchronously in the BeanBox's
     * menu handling thread.
     */
 
    public synchronized void queueMenuItem(ActionEvent evt) {
 
        if (events == null) {
            events = new Vector();
        }
        events.addElement(evt);
        notifyAll();
        if (eventThread == null) {
            eventThread = new Thread(this);
            eventThread.start();
        }
    }

    public void doRemoteInsert(Object bean, Rectangle bounds) {
	MouseEvent placementEvent = new MouseEvent(this,
	    MouseEvent.MOUSE_PRESSED, System.currentTimeMillis(), 0,
     	    bounds.x, bounds.y, 0, false);
       insertBean(bean, placementEvent, false);
    }

    /**
     * Insert a given Component instance into the current BeanBox.
     */

    public void doInsert(Object bean) {

        // Change the cursor to indicate an "insert".  (We may already
	// have done this, but that's OK.)

	getFrame().setCursor(crosshairCursor);

	// Wait for a mouse down event.
	MouseEvent evt = null;
	synchronized (mouseDown) {
	    try {
	        mouseDown.wait();
		evt = mouseDownEvent;
		insertBean(bean, mouseDownEvent, true);
	    } catch (Exception ex) {
		System.err.println("Insert thread interruped");
		return;
	    }
	}

	nuggetListeners.nuggetAdded(bean, 
	    new Rectangle(evt.getX(), evt.getY(), 0, 0));

    }

    private void insertBean(Object bean, MouseEvent evt, boolean select) {
	// Revert to the default cursor.
	getFrame().setCursor(defaultCursor);

	// PLI
	getFrame().showStatus("");

	// If its a BeanBox, color it a darker shade than the current.
	if (bean instanceof BeanBox) {
	    BeanBox bb = (BeanBox)bean;
	    //bb.resize(400,200);
	    bb.setSize(400,200);
	    int r = getBackground().getRed();
	    int g = getBackground().getGreen();
	    int b = getBackground().getBlue();
	    Color newColor = new Color((r*9)/10, (g*9)/10, (b*9)/10);
	    bb.setBackground(newColor);
	}

	// Create a Wrapper for the child
	Wrapper child = new Wrapper(bean); 
	NuggetInfo.getNuggetInfo(bean).setWrapper(child);
	  
	// Insert and reshape the child component.
	add(child, 0);
	int childWidth = child.preferredSize().width;
	int childHeight = child.preferredSize().height;

	if (childWidth == 0)
	    childWidth = 50;
	if (childHeight == 0)
	    childHeight = 50;

	int x = evt.getX(); // - childWidth/2;
	int y = evt.getY(); // - childHeight/2;
	child.setBounds(x, y, childWidth, childHeight);

	debug("checking for instance of Applet");
	if (Beans.isInstanceOf(bean, Applet.class)) {
	  debug("Bean is instance of Applet");
	  ClassLoader cl = SimpleClassLoader.ourLoader;
	  Applet apl = (Applet) Beans.getInstanceOf(bean, Applet.class);
	  assignAppletStub( apl, cl, bean.getClass());
	  apl.init();
	  apl.start();
	}

	if (select) {
	    // Make the child the focus.
            getFrame().setCurrentComponent(child);
	    getFrame().scroller.repaint();

	    // If this is a stretchable component, let the user begin
	    // resizing it immediately.
	    if (Beans.isInstanceOf(bean, Stretchable.class) &&
		! Beans.isInstanceOf(bean, UnStretchable.class)) {
                startResize(child, mouseDownEvent.getX(), 
		    mouseDownEvent.getY(), 
                    Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
	    }
	}
    }

    //----------------------------------------------------------------------

    // Push a new event onto our internal event queue and wakeup
    // our event handler thread.
    private synchronized void forkEvent(Event evt) {
	if (events == null) {
	    events = new Vector();
	}
	events.addElement(evt);
	notify();
	if (eventThread == null) {
	    eventThread = new Thread(this);
	    eventThread.start();
	}
    }

    // Main method for our internal event handling thread.
    // This guy lets us minimize the amount of work we do in the AWT event 
    // thread, which reduces the risk of deadlock.
    public void run() {
	for (;;) {
	    // Wait for an event.
	    ActionEvent evt;
	    synchronized (this) {
		while (events.size() == 0) {
		    try {
		        wait();
		    } catch (InterruptedException ix) {
		    }
		}
		evt = (ActionEvent) events.elementAt(0);
		events.removeElementAt(0);
	    }
	    // now process the event.
	    doMenuItem(evt);
	}
    }

    //----------------------------------------------------------------------

    /**
     * Someone has done a mouse button down on the "wrapper" around the bean.
     * This starts a move that will be terminated when they release the
     * mouse button.  We will draw a rubbed-banded box to indicate the
     * current move position.
     */

    public void startMove(Wrapper child, int x, int y) {
	// If this is an attempt to move our own wrapper, just ignore the
	// request.
	if (getParent() == child)
	    return;
	moveChild = child;
	moveStartX = child.location().x + x;
	moveStartY = child.location().y + y;
	child.setRubberBanding(true);
    }

    /**
     * Figure out the on-screen box to be drawn to represent the proposed
     * new move location of a bean we're moving.
     */

    Rectangle getMoveBox(int mx, int my) {
	int x = moveChild.location().x;
	int y = moveChild.location().y;
	int w = moveChild.size().width;
	int h = moveChild.size().height;
	x = x + mx - moveStartX;	
	y = y + my - moveStartY;
	return new Rectangle(x, y, w, h);
    }

    /**
     * Complete a move started with "startMove".
     */
     void finishMove(int mx, int my) {
	Rectangle box = getMoveBox(mx, my);
	doMove(box);

	nuggetListeners.nuggetMoved(moveChild.getBean(), moveChild.getBounds());

	moveChild = null;
	repaint();
    }

    public void remoteFinishMove(int mx, int my, int mw, int mh) {
    	doMove(new Rectangle(mx, my, mw, mh));

    	moveChild = null;
    	repaint();
    }

    private void doMove(Rectangle box) {
	Dimension size = getSize();
	if (box.x + box.width > size.width)
	    box.x = size.width - box.width;
	else if (box.x < 0)
	    box.x = 0;

	if (box.y + box.height > size.height)
	    box.y = size.height - box.height;
	else if (box.y < 0)
	    box.y = 0;

	//if ((box.x < size.width - 5) && (box.y < size.height - 5) &&
	//    (box.x >= 0 ) && (box.y >= 0)) {
	moveChild.setBounds(box.x, box.y, box.width, box.height);
	moveChild.setRubberBanding(false);
	deleteRubberBox();
	getFrame().scroller.repaint();
    }

    //----------------------------------------------------------------------

    /**
     * Someone has done a mouse button down on a corner of the "wrapper"
     * around the bean.  This starts a resize sequence that will be
     * terminated when they release the mouse button.  We will draw a 
     * rubbed-banded box to indicate the proposed resize shape..
     */

    void startResize(Wrapper child, int x, int y, Cursor cursor) {
	resizeStartX = child.location().x + x;
	resizeStartY = child.location().y + y;
	resizeChild = child;
	resizeCursor = cursor;
	child.setRubberBanding(true);
    }

    /**
     * Figure out the box to draw to imndicate the result of an
     * in-progress resize.  This is slightly complicated because
     * it deopends on which corner they picked-up to begin the
     * resize.
     */
    Rectangle getResizeBox(int mx, int my, boolean recurse) {

	int x = resizeChild.location().x;
	int y = resizeChild.location().y;
	int w = resizeChild.size().width;
	int h = resizeChild.size().height;
	if (resizeCursor == nwResizeCursor) {
	    w = (x + w) - mx;
	    h = (y + h) - my;
	    x = mx;
	    y = my;
        } else if (resizeCursor == swResizeCursor) {
	    w = (x + w) - mx;
	    h = my -y;
	    x = mx;
        } else if (resizeCursor == neResizeCursor) {
	    w = mx - x;
	    h = (y + h) - my;
	    y = my;
        } else if (resizeCursor == seResizeCursor) {
	    w = mx - x;
	    h = my - y;
	}
	Rectangle resizeBoxBounds = new Rectangle(x, y, w, h);
	resizeChild.setBounds(resizeBoxBounds);
	repaint();
	if (! recurse)
	    return resizeBoxBounds;

	if ((w < 1) && (h < 1)) {
	    resizeChild.setBounds(resizeBoxBounds);
	    if (resizeCursor == nwResizeCursor)
		resizeCursor = seResizeCursor;
	    else if (resizeCursor == seResizeCursor)
		resizeCursor = nwResizeCursor;
	    else if (resizeCursor == neResizeCursor)
		resizeCursor = swResizeCursor;
	    else if (resizeCursor == swResizeCursor)
		resizeCursor = neResizeCursor;
	    return getResizeBox(mx, my, false);
	}
	else if (w < 1) {
	    resizeChild.setBounds(resizeBoxBounds);
	    if (resizeCursor == nwResizeCursor)
		resizeCursor = neResizeCursor;
	    else if (resizeCursor == neResizeCursor)
		resizeCursor = nwResizeCursor;
	    else if (resizeCursor == seResizeCursor)
		resizeCursor = swResizeCursor;
	    else if (resizeCursor == swResizeCursor)
		resizeCursor = seResizeCursor;
	    return getResizeBox(mx, my, false);
	}
	else if (h < 1) {
	    resizeChild.setBounds(resizeBoxBounds);
	    if (resizeCursor == nwResizeCursor)
		resizeCursor = swResizeCursor;
	    else if (resizeCursor == neResizeCursor)
		resizeCursor = seResizeCursor;
	    else if (resizeCursor == seResizeCursor)
		resizeCursor = neResizeCursor;
	    else if (resizeCursor == swResizeCursor)
		resizeCursor = nwResizeCursor;
	    return getResizeBox(mx, my, false);
	}
/*
	if (w < 10) {
	    w = 10;
	}
	if (h < 10) {
	    h = 10;
	}
*/
	return new Rectangle(x, y, w, h);
    }

    /** 
     * Complete a resize begin with startResize.
     */
    void finishResize(int mx, int my) {
	Rectangle box = getResizeBox(mx, my, false);
	Dimension size = getSize();
	if (box.x + box.width > size.width)
	    box.x = size.width - box.width;
	else if (box.x < 0)
	    box.x = 0;

	if (box.y + box.height > size.height)
	    box.y = size.height - box.height;
	else if (box.y < 0)
	    box.y = 0;

	resizeChild.setBounds(box.x, box.y, box.width, box.height);

	// PLI -- Just using the MOVE event for this for now...
	nuggetListeners.nuggetMoved(resizeChild.getBean(), 
	   resizeChild.getBounds());

	resizeChild.setRubberBanding(false);
	resizeChild = null;
	deleteRubberBox();
    }

    //----------------------------------------------------------------------

    private void drawRubberBox(Rectangle box) {
	Graphics g = getGraphics();
	if (g == null) {
	    return;
	}
	Dimension size = getSize();
	if (box.x + box.width > size.width)
	    box.x = size.width - box.width;
	else if (box.x < 0)
	    box.x = 0;

	if (box.y + box.height > size.height)
	    box.y = size.height - box.height;
	else if (box.y < 0)
	    box.y = 0;

	// First remove any existing rubber box.
	deleteRubberBox();
	// Now draw the new rubber box.
	g.setColor(Color.red);
	g.drawRect(box.x, box.y, box.width, box.height);	
	oldRubberBox = box;
    }

    private void deleteRubberBox() {
	if (oldRubberBox != null) {
	    Graphics g = getGraphics();
	    if (g == null) {
	        return;
	    }
	    g.setColor(getBackground());
	    g.drawRect(oldRubberBox.x, oldRubberBox.y, 
			oldRubberBox.width, oldRubberBox.height);	
	    oldRubberBox = null;
	}
    }

    //----------------------------------------------------------------------
    public Dimension minimumSize() {
	return new Dimension(2000, 1000);
    }
    public Dimension preferredSize() {
	return minimumSize();
    }

    //----------------------------------------------------------------------

    void resetViewMenu() {
	BeanBoxFrame frame = getFrame();
	Menu viewMenu = getFrame().getMenuBar().getMenu(2);
	viewMenu.removeAll();
/*
	if (Beans.isDesignTime()) {
	    viewMenu.add(new MenuItem("Disable Design Mode"));
	} else {
	    viewMenu.add(new MenuItem("Enable Design Mode"));
	}
	if (getFrame().getHideInvisibleBeans()) {
	    viewMenu.add(new MenuItem("Show Invisible Beans"));
	} else {
	    viewMenu.add(new MenuItem("Hide Invisible Beans"));
	}
*/
	if (Beans.isDesignTime()) {
            addMenuItem(frame, viewMenu, new MenuItem("Disable Design Mode"));
        } else {
            addMenuItem(frame, viewMenu, new MenuItem("Enable Design Mode"));
        }
        if (getFrame().getHideInvisibleBeans()) {
            addMenuItem(frame, viewMenu, new MenuItem("Show Invisible Beans"));
        } else {
            addMenuItem(frame, viewMenu, new MenuItem("Hide Invisible Beans"));
        }
    }

    //----------------------------------------------------------------------

    /**
     * Figure out the top-level BeanBoxFrame that contains the current
     * BeanBox.
     */

    public synchronized BeanBoxFrame getFrame() {
	if (ourFrame != null) {
	    return ourFrame;
	}
	Component c = this;
	while (c != null) {
	    if (c instanceof BeanBoxFrame) {
		ourFrame = (BeanBoxFrame) c;
		return ourFrame;
	    }
	    c = c.getParent();
	}
	// This should never happen
	throw new Error("Couldn't find frame ?!?");
    }

    //----------------------------------------------------------------------

    // Log an error.

    void error(String message, Exception ex) {
	String mess = message + ":\n" + ex;
	System.err.println(message);
	ex.printStackTrace();
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(getFrame(), mess);

    }

    void error(String message) {
	System.err.println(message);
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(getFrame(), message);

    }

    /**
     * Finds the beanbox in which the given bean is located. Returns null
     * if the bean is not a descendant of any BeanBox. 
     */
    public static BeanBox findBeanBox(Component bean) {
        while ((bean != null) && (! (bean instanceof BeanBox)))
            bean = bean.getParent();
/*
	if (bean == null)
	    return getFrame().getTopBox();
	else
*/
      	    return (BeanBox)bean;
    }

    private static boolean debug = false;

    static void debug(String msg) {
	if (debug) {
	    System.err.println("BeanBox:: "+msg);
	}
    }

    //----------------------------------------------------------------------
 
    transient BeanBoxFrame ourFrame;

    transient Component connectionSource;
    transient int oldX1 = -1;
    transient int oldX2;
    transient int oldY1;
    transient int oldY2;
    transient Rectangle oldRubberBox;
  
    transient Hashtable methodMap;   // maps MenuItems to Methods
    transient Hashtable eventMap;   // maps Menus to EventSetDescriptors
    transient Class customizerClass;

    private static Hashtable serNames = new Hashtable();

    // If moveChild is non-null, we're moving that child
    transient Wrapper moveChild;
    transient int moveStartX;
    transient int moveStartY;

    // If resizeChild is non-null, we're resizing that child
    transient Wrapper resizeChild;
    transient Cursor resizeCursor;
    transient int resizeStartX;
    transient int resizeStartY;

    private final static int padX = 10;
    private final static int padY = 10;

    private final static String defaultStoreFile = "beanbox.tmp";

    // We keep a private event queue and a thread to process them.
    transient private Thread eventThread;
    transient private Vector events;

    // mouseDown gets notified when we get a mouse down event.
    private transient Object mouseDown = new Object();
    private transient MouseEvent mouseDownEvent;

    // Shorthands for the cursors.
    private static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

}



/**
 * Auxiliary class to deliver the InputStream
 */

class MyProducer implements InputStreamProducer {
    private String name;

    public MyProducer(String fileName) {
	this.name = fileName;
    }

    public synchronized InputStream getInputStream() {
	try {
	    return new FileInputStream(name);
	} catch (Exception ex) {
	    return null;
	}
    }
 private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}
