
/**
 * A BeanBoxFrame acts as the top-level "frame" that contains a BeanBox.
 * <p>
 * The BeanBoxFrame manages the frame's menubar and keeps track of which
 * bean currently has the focus.
 * <p>
 * Note that there is an asumption that there is only one BeanBoxFrame
 * per application, so various method/fields are static.
 */

package sieve.quarry;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

import vt.cs.widgets.*;

public class BeanBoxFrame extends Frame implements LayoutManager, Runnable,
    ActionListener {
    private static String tmpDir = "tmp";
    private static boolean doShowTimes = false; // This is for debug code?
    private static String clipDir = "tmp";
    private static String clipFile = "beanBoxClip.ser";
    private static String clipResource = "beanBoxClip";
    private static String versionId = "Sieve v1.0 - April 1997";

    private Thread focusThread;
    private Component nextFocus;

    protected Label statusLabel = new Label("Status: ", Label.LEFT);

    protected Hashtable atts = new Hashtable();
    private Hashtable propertyPanelCache = new Hashtable();

    public static void main(String argv[]) {
	System.setSecurityManager(new QuarrySecurityManager());
	BeanBoxFrame fr = new BeanBoxFrame(argv);
	fr.show();
	Frame toolBoxFrame = new Frame("ToolBox");
	toolBoxFrame.setLayout(new BorderLayout());
	toolBoxFrame.add(fr.getToolBox());
	toolBoxFrame.pack();
	//toolBoxFrame.setSize(new Dimension(400, 400));
	toolBoxFrame.show();
    }

    public BeanBoxFrame() {
	this(new String[0]);
    }

    public BeanBoxFrame(String[] argv) {
	super("Sieve Quarry");
        Timer tim = new Timer();

	if ((argv != null) && (argv.length > 0)) {
	    for (int i=0; i<argv.length; i++) {
	        int index = argv[i].indexOf("=");
	        if (index != -1) {
	            // parse for an attribute
	      	    atts.put(argv[i].substring(0, index),
		    argv[i].substring(index+1, argv[i].length()));
	    	} 
		else if (argv[0].equals("-version")) {
	            System.out.println("BeanBox version: "+versionId);
	            System.exit(0);
	        } 
		else {
	            atts.put(argv[i], "dummy");
	    	}
	    }
	}
	String checkString = (String)atts.get("s");
	if (checkString != null)
	    atts.put("standalone", "dummy");

	Beans.setDesignTime(true);

	if (showTimes()) {
	    System.err.println("total startup time => " + tim.elapsed());
	}

	//PLI setLayout(null);
	setLayout(new BorderLayout());
	setBackground(Color.lightGray);
	setForeground(Color.black);
	setMenuBar(new MenuBar());

	// Timing note: the setFont causes AWT to initializie itself,
	// so a large chunk of time happens here.
	setFont(new Font("Dialog", Font.PLAIN, 12));

	if (showTimes()) {
	    System.err.println("starting AWT => " + tim.elapsed());
	    tim.reset();
	}

	toolBox = new ToolBox(20, 20, this);

	if (showTimes()) {
	    System.err.println("new Toolbox => " + tim.elapsed());
	    tim.reset();
	}

	//topBox = new BeanBox(atts);
	topBox = new QuarryBox(atts);
	topBox.setBackground(Color.lightGray);
	topWrapper = new Wrapper(topBox);
	topWrapper.setBackground(Color.lightGray);
	topWrapper.resize(topWrapper.minimumSize());

	scroller = new MURadarPane(RadarPane.SCROLLBARS_AS_NEEDED, topBox);
	//	topWrapper.resize(1000, 1000);
	scroller.add(topWrapper);
	scroller.setDrawCursors(true);
	scroller.setDrawLabels(true);
	add("Center", scroller);
	debug("adding statusLabel");
	add("South", statusLabel);
	validate();

	if (showTimes()) {
	    System.err.println("new BeanBox => " + tim.elapsed());
	    tim.reset();
	}

	doSetCurrentFocus(topWrapper);
	topBox.resize(100,100);
	//	setLayout(this);

	setBounds(170, 20, 400, 550);

        propertyPanel = new PropertyPanel();
        propertyPanel.setBeanBox(topBox);
        propertySheet = new Frame("Properties");
	propertySheet.setLayout(new BorderLayout());
        propertySheet.add(propertyPanel);
        propertyPanel.setTarget(topBox);
	propertyPanel.doLayout();
	propertySheet.setSize(new Dimension(200, 200));
        //propertySheet.pack();
        propertySheet.show();

	toolBox.show();

	// Create a thread to handle focus changes.
	focusThread = new Thread(this);
	focusThread.start();
    }

    private boolean inBeanBox(Component c) {
	while (c != null) {
	    if (c instanceof BeanBox) {
		return true;
	    }
            c = c.getParent();
	}
	return false;
    }

    public void showStatus(String msg) {
    	statusLabel.setText("Status: "+msg);
    }

    public boolean handleEvent(Event evt) {

	// We'd like to use GOT_FOCUS here, but because we can't update
	// the current toolkit focus (see setCurrentFocus below) we 
	// have to fake it with MOUSE_DOWN instead.  Sigh.

	if (evt.id == Event.MOUSE_DOWN) {
	    // If the target is inside a holder change focus.
	    // This test is necessary to avoid refocusing onto dialog boxes.
	    Component target = (Component) evt.target;
System.out.println(" Got mousedown in frame. target = " + target);
	    if (inBeanBox(target)) {
		// System.err.println("evt = " + evt);
	        setCurrentComponent(target);
	    }
	}

	if (evt.id == Event.WINDOW_DESTROY) {
	  getTopBox().shutdown(0);
	  System.exit(0);
	}

	// Forward menu related events.
	if (evt.target instanceof MenuItem && evt.clickCount >= 0) {
	    evt.clickCount = -1;
	    getCurrentComponent().postEvent(evt);
            return true;
	}

	return false;
    }

    public void actionPerformed(ActionEvent evt) {
        // Menu actions come here,
        // We forward them to the current BeanBox.
        getCurrentBeanBox().queueMenuItem(evt);
    }


     public String getParameter(String name) {
       return (String)atts.get(name);
     }

    Object getCurrentBean() {
	return currentFocus.getBean();
    }

    Component getCurrentComponent() {
	return currentFocus.getChild();
    }

    public BeanBox getCurrentBeanBox() {
	Component c = getCurrentComponent();
	for (;;) {
	    if (c == null) {
	        System.err.println("No BeanBox in focus???");
		return null;
	    }
	    if (c instanceof BeanBox) {
		return (BeanBox) c;
	    }
	    c = c.getParent();
	}
    }


   void setCurrentComponent(Component focus) {
	// Null means focus on the top-level beanbox.
	if (focus == null) {
	    focus = topWrapper;
	}
	// Wakeup our internal thread to do the focus change.
	synchronized (this) {
	    nextFocus = focus;
	    notify();
	}	
    }

    // Internal thread to handle focus changes.  We use a separate
    // thread for this to offload work from the event thread in order
    // to avoid deadlocks.
    public void run() {
      debug("BeanBoxFrame.run started");
	for (;;) {
	    Component bean;
	    synchronized (this) {
		while (nextFocus == null) {
		    try {
		        wait();
		    } catch (Exception ex) {
			System.err.println("Unexpected interrupt in focus thread");
		    }
		}
		bean = nextFocus;
		nextFocus = null;
	    }
	    doSetCurrentFocus(bean);
	}
    }

    private void doSetCurrentFocus(Component focus) {
	Wrapper target = Wrapper.findWrapper(focus);

	if (target == currentFocus) {
	    return;
	}
	
	// Deactivate the old Wrapper.
	if (currentFocus != null) {
	    currentFocus.deactivate();
	}

	currentFocus = target;

        // Activate the new Wrapper.
	currentFocus.activate();

	// Find the nearest surrounding BeanBox and use its menubar.
	for (Component c = target.getChild(); c != null; c = c.getParent()) {
	    if (c instanceof BeanBox) {
		BeanBox hdr = (BeanBox) c;
		hdr.updateMenuBar(getMenuBar());
		break;
	    }
	}

	// Point the property sheet at the new focus.
	if (propertySheet != null) {
/*
	    propertyPanel.setTarget(getCurrentBean());
	    propertySheet.resize(propertyPanel.getSize().width,
		    propertyPanel.getSize().height + 40);
*/

	    // Begin experimental caching code...
	    if (propertyPanel != null)
		propertySheet.remove(propertyPanel);

	    Object bean = getCurrentBean();
            propertyPanel = (PropertyPanel) propertyPanelCache.get(bean);
	    boolean needLayout = false;
	    if (propertyPanel == null) {
		propertyPanel = new PropertyPanel(bean);
		propertyPanelCache.put(bean, propertyPanel);
		needLayout = true;
	    }

/*
	    if (propertyPanel.getTargetHasCustomizer())
		propertySheet.resize(200, 50);
	    else {
*/
	    if (! propertyPanel.getTargetHasCustomizer()) {
	        propertySheet.add(propertyPanel);
		if (needLayout)
		    propertyPanel.doLayout();
		propertySheet.pack();
	    }
	    // End experimental caching code...

	}
    }

    public void setCustomizer(Customizer c) {
	if (propertySheet != null) {
	    propertyPanel.setCustomizer(c);
	}
    }

    public void dispose() {
      getTopBox().shutdown(0);
      System.exit(0);
    }

    public static String getClipDir() {
	return clipDir;
    }

    public static String getClipFileName() {
	if (clipDir != null) {
	    return clipDir+java.io.File.separator+clipFile;
	} else {
	    return clipFile;
	}
    }

    public static String getClipResource() {
	return clipResource;
    }

    public static String getTmpDir() {
	return tmpDir;
    }

    //----------------------------------------------------------------------
    // Layout related stuff.
    //----------------------------------------------------------------------

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
	return new Dimension(400,300);
    }

    public Dimension minimumLayoutSize(Container parent) {
	return new Dimension(100,100);
    }

    public void layoutContainer(Container parent) {
	Dimension d = parent.size();
	Insets ins = parent.getInsets();
	int width = d.width - (ins.left + ins.right);
	int height = d.height - (ins.top + ins.bottom);
/*
	topWrapper.setBounds(ins.left, ins.top, width, height);
	topWrapper.invalidate();
	topWrapper.layout();
*/
	scroller.setBounds(ins.left, ins.top, width, height);
	System.out.println("scroller.invalidate");
	scroller.invalidate();
	scroller.layout();
    }

    public BeanBox getTopBox() {
	return topBox;
    }   

    public Wrapper getTopWrapper() {
	return topWrapper;
    }   

    public ToolBox getToolBox() {
	return toolBox;
    }

    protected void setToolBox(ToolBox tb) {
	toolBox = tb;
    }

    public Frame getPropertySheet() {
	return propertySheet;
    }

    public static boolean showTimes() {
	return doShowTimes;
    }

    public boolean getHideInvisibleBeans() {
	return hideInvisibleBeans;
    }

    public void setHideInvisibleBeans(boolean hide) {
	hideInvisibleBeans = hide;
	Wrapper.showInvisibleBeans(!hide);
    }

    public void setDesignTime(boolean designMode) {
	Beans.setDesignTime(designMode);
	if (designMode) {
 	    toolBox.setVisible(true);
	    propertySheet.setVisible(true);
	} else {
	    toolBox.setVisible(false);
	    propertySheet.setVisible(false);
	}
    }

    //----------------------------------------------------------------------

    private BeanBox topBox;
    private Wrapper topWrapper;
    public MURadarPane scroller;

    private Wrapper currentFocus;

    private Frame propertySheet;
    private PropertyPanel propertyPanel;

    private ToolBox toolBox;

    private boolean hideInvisibleBeans;    

    private static final boolean debug = false;
    private void debug(String message) {
      if (debug)
        System.err.println(getClass().getName()+": "+message);
    }
}
