/*
 * This file should go away shortly. This functionality is now provided
 * by the PropertyPanel class. -- PLI 4/3/97
 */
package sieve.quarry;

import java.beans.*;
import java.lang.reflect.*;
import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;
import sieve.crew.*;

public class PropertySheet extends Frame {

    PropertySheet(Object target, int x, int y) {
	super("PropertySheet");
	setLayout(null);
	setBackground(Color.lightGray);	
	move(x,y);
	setTarget(target);
	show();
    }

    synchronized void setTarget(Object targ) {

	target = targ; 
	removeAll();

        try {
	    BeanInfo bi = Introspector.getBeanInfo(target.getClass());
	    properties = bi.getPropertyDescriptors();
	} catch (IntrospectionException ex) {
	    error("PropertySheet: Couldn't introspect", ex);
	    return;
	}

	setLayout(null);

	editors = new PropertyEditor[properties.length];
	values = new Object[properties.length];
	views = new Component[properties.length];
	labels = new Label[properties.length];

	// Create an event adaptor.
	EditedAdaptor adaptor = new EditedAdaptor(this);

	labelWidth = 100;
	viewWidth = 120;
	int height = 35;

	for (int i = 0; i < properties.length; i++) {

	    // Don't display hidden or expert properties.
	    if (properties[i].isHidden() || properties[i].isExpert()) {
		continue;
	    }

	    String name = properties[i].getDisplayName();
	    Class type = properties[i].getPropertyType();
	    Method getter = properties[i].getReadMethod();
	    Method setter = properties[i].getWriteMethod();

	    // PLI: Hack up a slightly better looking display name
	    // if the display name was the same as the prop name
	    // and the name isn't something in all caps (like "URL").
	    if (name.equals(properties[i].getName()) &&
		(! name.toUpperCase().equals(name))) {
		StringBuffer buf = new StringBuffer(name);

		// Capitalize the first character
		if (Character.isLowerCase(buf.charAt(0))) {
		    buf.setCharAt(0, Character.toUpperCase(buf.charAt(0)));
		}

		// Add Spaces before any other capital letters.
		int len = name.length();
		for (int ch = 1; ch < len; ch++) {
		    if (Character.isUpperCase(buf.charAt(ch))) {
			buf.insert(ch, ' ');
			ch++;
			len++;
		    }
		}
		name = buf.toString();
	    }

	    // Only display read/write properties.
	    if (getter == null || setter == null) {
		continue;
	    }
	
	    Component view = null;

	    try {
		Object args[] = { };
		Object value = getter.invoke(target, args);
	        values[i] = value;

	        PropertyEditor editor = null;
	        Class pec = properties[i].getPropertyEditorClass();
		if (pec != null) {
		    try {
			editor = (PropertyEditor)pec.newInstance();
		    } catch (Exception ex) {
			// Drop through.
		    }
		}
		if (editor == null) {
		    editor = PropertyEditorManager.findEditor(type);
		}
	        editors[i] = editor;

	        // If we can't edit this component, skip it.
	        if (editor == null) {
		    continue;
	        }

	        editor.setValue(value);
	        editor.addPropertyChangeListener(adaptor);

		// Now figure out how to display it...
		if (editor.isPaintable() && editor.supportsCustomEditor()) {
		    view = new PropertyCanvas(this, editor);
		} else if (editor.getTags() != null) {
		    view = new PropertySelector(editor);
		} else if (editor.getAsText() != null) {
		    String init = editor.getAsText();
		    view = new PropertyText(editor);
		} else {
		    System.err.println("Skipping non-displayable property " + name);
		    continue;
		}

	    } catch (Exception ex) {
		System.err.println("Skipping property " + name + " : " + ex);
		ex.printStackTrace();
		continue;
	    }

	    labels[i] = new Label(name, Label.RIGHT);
	    add(labels[i]);
	    int w = labels[i].preferredSize().width;
	    if (w > labelWidth) {
		labelWidth = w;
	    }

	    views[i] = view;
	    add(views[i]);

	    w = view.preferredSize().width;
	    if (w > viewWidth) {
		viewWidth = w;
	    }
	    int h = view.preferredSize().height;
	    if (h < 30) {
		h = 30;
	    }

	    height += (h + hpad);
	}

	int width = labelWidth + viewWidth + 10;
	if (width > 300) {
	    width = 300;
	}

	resize(width,height);
	doLayout();
	processEvents = true;
    }


    public void layout() {
	int width = size().width;

	int y = insets().top + 10;

	if (views == null || labels == null) {
	    return;
	}

	for (int i = 0; i < properties.length; i++) {
	    if (labels[i] == null || views[i] == null) {
		continue;
	    }

	    labels[i].setBounds(0, y+5, labelWidth-8, 25);

	    int h = views[i].preferredSize().height;
	    if (h < 30) {
		h = 30;
	    }
	    int w = views[i].preferredSize().width;
	    if (w < 105) {
		w = 105;
	    }
	    if (w > viewWidth) {
		w = viewWidth;
	    }
	    views[i].setBounds(labelWidth, y, viewWidth, h);
	    y += (h + hpad);
	}

	y += 5 + insets().bottom;
    }


    synchronized void setCustomizer(Customizer c) {
	if (c != null) {
	    c.addPropertyChangeListener(new EditedAdaptor(this));
	}
    }

    public boolean handleEvent(Event evt) {
	if (evt.id == Event.ACTION_EVENT) {
	    Component parent = getParent();
	    if (parent != null) {
	        parent.postEvent(new Event(this, Event.MOUSE_DOWN, null));
	    }
	} else if (evt.id == Event.WINDOW_DESTROY) {
	    System.exit(0);
	}
	return (false);
    }

    synchronized void wasModified(PropertyChangeEvent evt) {

	if (!processEvents) {
	    return;
	}

	if (evt.getSource() instanceof PropertyEditor) {
	    PropertyEditor editor = (PropertyEditor) evt.getSource();
	    for (int i = 0 ; i < editors.length; i++) {
	        if (editors[i] == editor) {
		    PropertyDescriptor property = properties[i];
	            Object value = editor.getValue();
	            values[i] = value;
//System.out.println("** In PropertySheet: setting property " + 
//property.getDisplayName() + " to " + value); 
		    Method setter = property.getWriteMethod();
		    try {
		        Object args[] = { value };
		        args[0] = value;
		        setter.invoke(target, args);
		    } catch (java.lang.reflect.InvocationTargetException ex) {
		      if (ex.getTargetException() instanceof PropertyVetoException) {
			warning("Vetoed; reason is: " + ex.getTargetException().getMessage());
		      }
		      else
		        error("InvocationTargetException while updating " + 
					property.getName(), ex);
		    } catch (Exception ex) {
		        error("Unexpected exception while updating " + 
					property.getName(), ex);
	            }
	            if (views[i] != null && views[i] instanceof PropertyCanvas) {
		        views[i].repaint();
	            }
                    if (target instanceof Component) {
                      BeanBox box = BeanBox.findBeanBox((Component)target);
                      if ((box != null) && (box instanceof QuarryBox)) {
                        ((QuarryBox)box).notifyParticipants(
			  NuggetMessage.MODIFY,
                          new NuggetChangeInfo(target, property.getName(),
                          value));
                      }
                    }

		    break;
		}
	    }
	}

	// Now re-read all the properties and update the editors
	// for any other properties that have changed.
	for (int i = 0; i < properties.length; i++) {
	    Object o;
	    try {
	        Method getter = properties[i].getReadMethod();
	        Object args[] = { };
	        o = getter.invoke(target, args);
	    } catch (Exception ex) {
		o = null;
		continue;
	    }
	    if ((o != null) && (o == values[i] || o.equals(values[i]))) {
	        // The property is equal to its old value.
		continue;
	    }
	    values[i] = o;
	    // Make sure we have an editor for this property...
	    if (editors[i] == null) {
		continue;
	    }
	    // The property has changed!  Update the editor.
	    // PLI editors[i].setValue(o);
	    if (views[i] != null) {
		views[i].repaint();
	    }
	}

	// Make sure the target bean gets repainted.
	if (Beans.isInstanceOf(target, Component.class)) {
	    ((Component)(Beans.getInstanceOf(target, Component.class))).repaint();
	}
    }


    private void warning(String s) {
	new ErrorDialog(this, "Warning: " + s);
    }


    //----------------------------------------------------------------------
    // Log an error.

    private void error(String message, Exception ex) {
	String mess = message + ":\n" + ex;
	System.err.println(message);
	ex.printStackTrace();
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(this, mess);

    }

    //----------------------------------------------------------------------
    private Object target;
    private PropertyDescriptor properties[];
    private PropertyEditor editors[];
    private Object values[];
    private Component views[];
    private Label labels[];

    private boolean processEvents;
    private static int hpad = 4;
    private int labelWidth = 100;
    private int viewWidth = 110;
}
