package sieve.quarry;

import sieve.quarry.event.*;
import sieve.crew.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import SharedData.*;

public class QuarryBox extends BeanBox implements QuarryLinkListener,
    QuarryNuggetListener {

    public static final int  defaultPort = 8889;
    public static final String defaultHost = "simon.cs.vt.edu";
    public static final String defaultSessionName = "Sieve";
    private Hashtable telepointers = new Hashtable();
 
    private String localHostName = "UnknownHost";

    private boolean cursorMoved = false;

    private NuggetControl nuggetControl = null;
    private ParticipantControl participantControl = null;
    private Session session = null;
    private Channel controlChannel = null;
    private Participant localParticipant = null;
    private ParticipantDialog participantDialog = null;

    MenuItem editParticipant = new MenuItem("Edit Participant Info...");
    Menu telepointerMenu = new Menu("Telepointers");
    public CheckboxMenuItem showNames = 
	new CheckboxMenuItem("Show Participant Names", true);
    CheckboxMenuItem showRemotePointers =  
	new CheckboxMenuItem("Show Remote Pointers", true);
    CheckboxMenuItem showLocalPointer = 
	new CheckboxMenuItem("Show Local Pointer", false);

    public QuarryBox(Hashtable atts) {
	super(atts);
	addQuarryLinkListener(this);
	addQuarryNuggetListener(this);

	try {
            localHostName = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException uhex) {
            debug("UnknownHostException when trying to get local host.");
        }
   
        if (atts.get("standalone") == null) {
            // Set up Session
 
            int port = defaultPort;
            String portString = (String)atts.get("port");
            if (portString != null)
                port = Integer.parseInt(portString);
 
            String host = (String)atts.get("host");
            if (host == null)
                host = defaultHost;
 
            String sessionName = (String)atts.get("sessionName");
            if (sessionName == null)
                sessionName = defaultSessionName;
 
            createSession(host, port, sessionName);
          }

    }

    public synchronized void updateMenuBar(MenuBar bar) {
	super.updateMenuBar(bar);
	if (bar.getMenuCount() == 4) {
	    if (atts.get("standalone") == null) {
		BeanBoxFrame frame = getFrame();
                Menu m = new Menu("Share");
                addMenuItem(frame, m, editParticipant);
 
                addMenuItem(frame, telepointerMenu, showNames);
                addMenuItem(frame, telepointerMenu, showRemotePointers);
                addMenuItem(frame, telepointerMenu, showLocalPointer);
 
                //m.add(telepointerMenu);
                addMenuItem(frame, m, telepointerMenu);
                bar.add(m);
            }
	}
    }

    void doMenuItem(ActionEvent evt) {
        MenuItem mi = (MenuItem)evt.getSource();
        Menu parent = (Menu)mi.getParent();
        String label = mi.getLabel();

System.out.println("mi = " + mi + " parent = " + parent);

	if (parent.getLabel().equals("Share")) {
	    if (label.equals("Edit Participant Info...")) 
                editUser();
	}
	else if (parent == telepointerMenu ) {
	    if (mi == showNames) {
		for (Enumeration e = telepointers.elements(); 
		    e.hasMoreElements(); ) {
		    Telepointer tp = (Telepointer)e.nextElement();
              	    tp.setShowName(showNames.getState());
		}
	    }
	    else if (mi == showRemotePointers) {
		for (Enumeration e = telepointers.elements(); 
                    e.hasMoreElements(); ) {
                    Telepointer tp = (Telepointer)e.nextElement();
		    tp.setVisible(showRemotePointers.getState());
                }
	    }
	    else if (mi == showLocalPointer) {
		if (showLocalPointer.getState()) {
		    // Create telepointer that uses an "echo" of the 
		    // localParticipant
		    Participant localEcho = null;
		    if (participantControl != null) {
			localEcho = participantControl.getLocalEcho();
		    }
		    if ((localEcho == null) && (localParticipant != null)) {
			localEcho = new Participant(localParticipant);
			// just observe for name and color changes
			localParticipant.addObserver(localEcho);
			participantControl.setLocalEcho(localEcho);
		    }
		    if (localEcho != null) {
                	Telepointer tp = new Telepointer(localEcho);
                	tp.setShowName(showNames.getState());
                	telepointers.put(localEcho.getIdentifier(), tp);
              	    }
		}
		else { // turn local t-ptr off
		    if (localParticipant != null) {
			Telepointer tp = (Telepointer)telepointers.remove(
			    localParticipant.getIdentifier());
			if (tp != null) {
			    Participant part = tp.getParticipant();
			    part.deleteObserver(tp);
			    localParticipant.deleteObserver(part);
			}
		    }
		}
	    }
	    repaint();
	}
	else
	    super.doMenuItem(evt);

    }

    private void createSession(String host, int port, String sessionName) {
    	String sessionURL = 
		"coll:"+host+":"+port+"/socket/Session/"+sessionName;
 
      	try {
            session = SessionFactory.createSession(sessionURL);
        } 
 	catch (Exception e) {
            System.out.println("Error creating session: "+e);
            e.printStackTrace();
        }

        if (session != null) {
            participantControl = new ParticipantControl(session, this);
            nuggetControl = new NuggetControl(session, this);
	    nuggetControl.setParticipantControl(participantControl);
        }
    }

    public void addNotify() {
        super.addNotify();
        if (atts.get("standalone") == null) {
      	    editUser();
      	    participantControl.notifyParticipants(ParticipantMessage.CREATE,
        	localParticipant);
    	} 
	else
      	    createLocalParticipant();
    }

    private void createLocalParticipant() {
    	if (participantControl != null) {
      	    localParticipant = new Participant("NoName@"+localHostName,
                ParticipantDialog.getRandomColor());

      	    participantControl.setLocalParticipant(localParticipant);
    	} 
	else {
            localParticipant = new Participant("", Color.black);

      	    vt.cs.widgets.MURadarPane muRadarPane = null;

            if (muRadarPane == null)
                muRadarPane = this.getFrame().scroller;
 
            muRadarPane.addLocalViewPort(localParticipant);
	}
    }

    public void editUser() {
    	if (localParticipant == null)
      	    createLocalParticipant();
 
    	if (participantDialog == null) {
      	    Component comp = this;
      	    while (!(comp instanceof Frame))
           	comp = comp.getParent();
      	    participantDialog = 
		new ParticipantDialog((Frame)comp, localParticipant);
    	}
 
    	participantDialog.show();
    	//    getFrame().scroller.setLocalUserColor(localParticipant.getColor());

    	if (participantControl != null)
      	    participantControl.notifyParticipants(ParticipantMessage.MODIFY,
            	localParticipant);
    }

    //------------- MURadarPaneParent interface --------
    public void updateBounds(Rectangle bounds){ // PLI -- synching
    	if (localParticipant == null)
      	    createLocalParticipant();
 
    	localParticipant.setBounds(bounds);
 
    	if (participantControl != null)
      	    participantControl.notifyParticipants(ParticipantMessage.MOVE,
              	localParticipant);
    }
    //------------- end MURadarPaneParent interface --------

    public void setCursorMoved(boolean value) { 
        cursorMoved = value;
        if ((cursorMoved) && (participantControl != null))
      	    participantControl.notifyCursor();
    }

    public synchronized boolean getCursorMoved() {
    	return cursorMoved;
    }

    public synchronized void mouseMoved(MouseEvent evt) {
	super.mouseMoved(evt);
        int x = evt.getX();
        int y = evt.getY();
 
        // There should be a better way to do this. To make links and
        // telepointers work properly, we have to translate from the
        // coordinate space of an arbitrarily nested component to
        // the BeanBox's coordinate space. Yuck.
        if (evt.getComponent() != this) {
            Component last = evt.getComponent();
            Component curr = last.getParent();
            while ((last != this) && (curr != null)) {
                x += last.getLocation().x;
                y += last.getLocation().y;
                last = curr;
                curr = last.getParent();
            }
        }
 
        if (participantControl != null) {
            // update remote telepointers
            localParticipant.setLocation(new Point(x, y));
            setCursorMoved(true);
        }

    }

    void shutdown(int exitValue) {
    	// close channel connections, etc.
    	if (participantControl != null)
      	    participantControl.shutdown();
    	if (nuggetControl != null)
      	    nuggetControl.shutdown();
 
    	System.exit(exitValue);
    }

    public void addTelepointer(String id, Telepointer tp) {
    	telepointers.put(id, tp);
    }
    public Telepointer removeTelepointer(String id) {
    	return (Telepointer)telepointers.remove(id);
    }


    public synchronized void linkAdded(LinkEvent e) {

	if (nuggetControl != null) {
	    LinkInfo li = LinkInfo.getLinkInfo(e.getLinkSource(),
		e.getLinkDestination());
	    nuggetControl.notifyParticipants(NuggetMessage.LINK_CREATE, li);
	}
    }

    public synchronized void linkRemoved(LinkEvent e) {
	if (nuggetControl != null) {
	    LinkInfo li = LinkInfo.getLinkInfo(e.getLinkSource(),
		e.getLinkDestination());
	    nuggetControl.notifyParticipants(NuggetMessage.LINK_REMOVE, li);
	}
    }

    public void nuggetAdded(NuggetEvent e) {
        if (nuggetControl != null) {
            NuggetInfo ni = NuggetInfo.getNuggetInfo(e.getNugget());
            ni.setBounds(e.getBounds());
	    if (localParticipant != null)
		ni.setCreatorId(localParticipant.getIdentifier());
            nuggetControl.notifyParticipants(NuggetMessage.CREATE, ni);
        }
    }

    public void nuggetMoved(NuggetEvent e) {
        if (nuggetControl != null) {
            NuggetInfo ni = NuggetInfo.getNuggetInfo(e.getNugget());
            ni.setBounds(e.getBounds());
            nuggetControl.notifyParticipants(NuggetMessage.MOVE, ni);
        }
    }

    public void nuggetRemoved(NuggetEvent e) {
    }


    public void paint(Graphics g) {
	super.paint(g);
	Rectangle clip = g.getClipRect();
        for (Enumeration e = telepointers.elements(); e.hasMoreElements();){
            Telepointer tp = (Telepointer)e.nextElement();
            Rectangle cr = tp.getBounds();
            if (cr.intersects(clip)) {
          	Graphics cg = g.create(cr.x, cr.y, cr.width, cr.height);
          	try {
            	    tp.paint(cg);
          	} 
		finally {
            	    cg.dispose();
          	}
	    }
        }
    }

    /*
     * PropertyPanel still uses this, but probably shouldn't...
     */
    public void notifyParticipants(int type, Object data) {
      	if (nuggetControl != null)
            nuggetControl.notifyParticipants(type, data);
    }


}

