/**
 * Report on the introspection information for a given TableView class.
 */

package sieve.quarry;

import java.beans.*;
import java.lang.reflect.*;

public class TableViewReport {

    public static void report(Object bean) {

        Class beanClass = bean.getClass();
	try {
	    BeanInfo bi = Introspector.getBeanInfo(beanClass);
            Object args[] = { };

	    putln("CLASS: " + beanClass.getName());

	    PropertyDescriptor props[] = bi.getPropertyDescriptors();
            String name = null;
	    for (int i = 0; i < props.length && (name == null); i++) {
		PropertyDescriptor pd = props[i];
                if (pd.getName().equals("name")) {
                    Method getter = pd.getReadMethod();
		    name = getter.invoke(bean, args).toString();
                }
            }
            if (name != null) 
                putln("NAME: " + name);
            else
                putln("");

	    for (int i = 0; i < props.length; i++) {
		PropertyDescriptor pd = props[i];
                Method putter = pd.getWriteMethod();
                Method getter = pd.getReadMethod();
                if ((putter != null) && (getter != null)
                    && (! pd.isHidden()) && (! pd.isExpert())
                    && (! (pd instanceof IndexedPropertyDescriptor))
                    && (! pd.getName().equals("name"))) {
		    //putFlags(pd);
	            put(pd.getName(), 17);
		    put(" ");
	            put("" + pd.getPropertyType(), 27);
		    put(" ");

		    put(getter.invoke(bean, args).toString());

	            //putMethod(pd.getReadMethod());
		    //put("/");
	            //putMethod(pd.getWriteMethod());
	            putln("");
/*
		    if (pd instanceof IndexedPropertyDescriptor) {
		        IndexedPropertyDescriptor ipd =
					(IndexedPropertyDescriptor) pd;
		        put("    ");
		        put("  (indexed)", 17);
		        put(" ");
	                put("" + ipd.getIndexedPropertyType(), 27);
		        put(" ");
	                putMethod(ipd.getIndexedReadMethod());
		        put("/");
	                putMethod(ipd.getIndexedWriteMethod());
	                putln("");
                    }
*/
		}
	    }
            if (bean instanceof TableViewPanel) {
	        putln("------- TableView inside of Panel -------> ");
                report(((TableViewPanel)bean).tableView);
            }
	    putln("");
	} catch (IntrospectionException ex) {
	    System.out.println("Caught introspection exception: " + ex);
	    ex.printStackTrace();
	}
        catch (Exception e) {
	    e.printStackTrace();
        }
    }

    private static void putMethod(Method m) {
	if (m == null) {
	    put("");
	} else {
	    put(m.getName());
  	}
    }

    private static void putFlags(FeatureDescriptor fd) {
	if (fd.isHidden()) {
	    put("H");
	} else {
	    put(" ");
	}
	if (fd.isExpert()) {
	    put("E");
	} else {
	    put(" ");
	}
	if (fd instanceof IndexedPropertyDescriptor) {
	    put("[");
	} else {
	    put(" ");
	}
	put(" ");
    }

    private static void put(String s) {
	System.out.print(s);
    }

    private static void put(String s, int size) {
	System.out.print(s);
	for (int i = s.length(); i < size; i++) {
	    System.out.print(" ");
	}
    }

    private static void putln(String s) {
	put(s, s.length());
	System.out.println("");
    }

}


