
/*
 * This wrapper class exists simply to add a black border around
 * the currenty active component.
 */

package sieve.quarry;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.beans.*;
import java.util.*;
import vt.cs.widgets.Scalable;

import sieve.sluice.*;
import sieve.crew.*;
import sieve.util.*;

public class Wrapper extends Container implements Serializable, Scalable,
    MouseListener, MouseMotionListener, PropertyChangeListener {

    public Wrapper(Object bean) {
        this.bean = bean;
	this.rawBean = bean;
	active = false;
	setLayout(new BorderLayout());
	if (Beans.isInstanceOf(bean, Component.class)) {
            child = (Component)Beans.getInstanceOf(bean, Component.class);
	} else {
	    invisibleWrappers.addElement(this);
	    child = getCustomizer(bean);

	}
	add("Center", child);

	child.move(borderWidth, borderWidth);
	//	addContainerListener(new WrapperListener());
	listenForMice(true);
    }

/* This needs work...
    TipText tt = null;
*/
    public void addNotify() {
	super.addNotify();
	Component c = this;
	while ((c != null) && ! (c instanceof BeanBoxFrame))
	    c = c.getParent();
    	if ((c != null) && ((BeanBoxFrame) c).getHideInvisibleBeans()) 
	    setVisible(false);
/*
	tt = new TipText(TipText.findFrame(this), this, 
	    "Right-click and release to start link", true);
*/
    }


    /**
     * get the wrapped bean.
     */
    public Object getBean() {
	return bean;
    }

    public Object getRawBean() {
    	return rawBean;
    }

    /**
     * get the AWT component used to represent the wrapped bean.
     */
    public Component getChild() {
	return child;
    }

    /**
     * Static method to map from an AWT component to the associated Wrapper.
     */
    public static Wrapper findWrapper(Component comp) {
	for (;;) {
	    if (comp == null) {
	        throw new Error("component has no wrapper!?");
	    }
	    if (comp instanceof Wrapper) {
		return (Wrapper) comp;
	    }
	    if (comp instanceof BeanBoxFrame) {
		return ((BeanBoxFrame)comp).getTopWrapper();
	    }
	    comp = comp.getParent();
	}
    }

    public void layout() {
	child.layout();
	sizeToFit();
	child.repaint();
    }

    private void sizeToFit() {
	if (childHasStupidPreferredSize()) {
	    return;
	}
	Dimension d = preferredSize();
	resize(d.width, d.height);
    }


    public void setActive(boolean isActive) {
	active = isActive;
	repaint();
    }

    public Dimension preferredSize() {
	Dimension childSize = child.getPreferredSize();
	if (childHasStupidPreferredSize()) {
	    childSize = child.getSize();
	}
	int width = childSize.width + (2 * borderWidth);
	int height = childSize.height + (2 * borderWidth);
	return new Dimension(width, height);
    }

    private boolean childHasStupidPreferredSize() {
	// We do a special check for demented behaviour from empty Panels
	// for the sake of applets.  If an applet does not explicitly
	// specify a dimension, then we allow it to be resized arbitrarily
	// rather than limiting it to the tiny size returned by the FlowLayout
	if (child instanceof Panel) {
	    Container cont = (Container)child;
	    LayoutManager lay = cont.getLayout();
	    if (cont.getComponentCount() == 0 && lay instanceof FlowLayout) {
		FlowLayout flow = (FlowLayout) lay;
		Dimension cd = child.getPreferredSize();
		Dimension fd = flow.preferredLayoutSize(cont);
		if (cd.width == fd.width && cd.height == fd.height) {
		    return true;
		}
	    }
	}
	return false;
    }

    // Note that to avoid deadlocks, setBounds is *not* synchronized
    public void setBounds(int x, int y, int width, int height) {
	super.setBounds(x, y, width, height);
	child.setBounds(borderWidth, borderWidth, 
		width-(2*borderWidth), height-(2*borderWidth));
	child.validate();
    }

    public void setColorSource(ColorSource cs) {
	this.colorSource = cs;
    }

    // Note that to avoid deadlocks, paint is *not* synchronized
    public void paint(Graphics g) {
	Color myColor = Color.darkGray;
	if (colorSource != null)
	    myColor = colorSource.getColor();

        // Move this out somehow. Maybe a "WrapperPainter" interface?
	if (Beans.isInstanceOf(bean, Stretchable.class) &&
	    ! Beans.isInstanceOf(bean, UnStretchable.class)) {
/*
	if (Beans.isInstanceOf(bean, Component.class))  &&
            ! Beans.isInstanceOf(bean, Canvas.class) &&
            ! Beans.isInstanceOf(bean, Panel.class)) {
*/
	    if (active) {
	        int lowX = borderWidth;
	 	int highX = size().width - borderWidth;
		int lowY = borderWidth;
		int highY = size().height - borderWidth;
		g.setColor(Color.black);
		g.fillRect(0, 0, borderWidth, borderWidth);
		g.fillRect(0, highY, borderWidth, borderWidth);
		g.fillRect(highX, highY, borderWidth, borderWidth);
		g.fillRect(highX, 0, borderWidth, borderWidth);
	    }
	}

	else 
	if (active && Beans.isDesignTime() && !(child instanceof BeanBox)) {
	    int hashWidth = 4;
	    int width = size().width;
	    int height = size().height;

	    // We'll use a slanting polygon to draw a hashed area.
	    Polygon poly = new Polygon();
	    for (int i = 0; i < 4; i++) {
		poly.addPoint(0,0);
	    }
   	    poly.ypoints[2] = height + 1;
   	    poly.ypoints[3] = height + 1;

	    for (int x = 0; x < (width+height); x += hashWidth) {
		// draw alternate dark gray and light gray stripes.
		if (((x/hashWidth)%2) == 0) {
	           g.setColor(myColor);
		} else {
	            g.setColor(Color.lightGray);
		}

		poly.xpoints[0] = x;
		poly.xpoints[1] = x + hashWidth + 1;
		poly.xpoints[2] = x - height + hashWidth + 1;
		poly.xpoints[3] = x - height;
		g.fillPolygon(poly);
	    }
	}
	else {
	    g.setColor(myColor);
	    g.drawRect(borderWidth-1, borderWidth-1, 
	        size().width-(2 * (borderWidth-1)), 
	        size().height-(2 * (borderWidth-1)));
	    if (! (child instanceof BeanBox))
	        g.drawRect(0, 0, size().width-1, size().height-1);
	}
	super.paint(g);
    }

    public synchronized void activate() {
	active = true;
	repaint();
    }
 
    public synchronized void deactivate() {
	active = false;
	repaint();
    }

    // Copied from the April'97 BDK. (What they basically did is renamed
    // handleEvent...)
    public synchronized void doMouseStuff(MouseEvent evt) {

	if (!active || !Beans.isDesignTime()) {
	    return;
	}

	Component parent = getParent();
	if (parent != null && parent instanceof BeanBoxFrame) {
	    return;
	}

	int x = evt.getX();
	int y = evt.getY();
	int id = evt.getID();

	int lowX = borderWidth;
	int highX = size().width - borderWidth;
	int lowY = borderWidth;
	int highY = size().height - borderWidth;

	if (evt.getSource() == this && !rubberBanding &&
	       (id == MouseEvent.MOUSE_ENTERED || 
		id == MouseEvent.MOUSE_MOVED)) {

	    // Check if we're on the border.	
	    if (x < lowX || x > highX || y < lowY || y > highY) {
		// We're on the border.  If we're in a corner set the cursor
		// to indicate a resize, otherwise a move.
		lowX += resizeDelta;
		lowY += resizeDelta;
		highX -= resizeDelta;
		highY -= resizeDelta;

	        if (x < lowX && y < lowY) {
		    setCursor(nwResizeCursor);
		    needToRestoreCursor = true;
	        } else if (x < lowX && y > highY) {
		    setCursor(swResizeCursor);
		    needToRestoreCursor = true;
	        } else if (x > highX && y < lowY) {
		    setCursor(neResizeCursor);
		    needToRestoreCursor = true;
	        } else if (x > highX && y > highY) {
		    setCursor(seResizeCursor);
		    needToRestoreCursor = true;
		} else {
		    setCursor(moveCursor);
		    needToRestoreCursor = true;
		}
	    }

	} 
        else if (evt.getSource() != this && 
	       (id == MouseEvent.MOUSE_ENTERED || 
		id == MouseEvent.MOUSE_MOVED) &&
		needToRestoreCursor) {

	    setCursor(defaultCursor);
	    needToRestoreCursor = false;

	} 
    }

    // Mouse listener methods for target bean.
    // We also report all mouse events to our containing BeanBox.
    public void mouseClicked(MouseEvent evt) {
        doMouseStuff(evt);
        getBeanBox().mouseClicked(evt);
    }
 
    public void mousePressed(MouseEvent evt) {
	if (active && Beans.isDesignTime() && (getParent() == null ||
	    !(getParent() instanceof BeanBoxFrame))) {

	    int x = evt.getX();
	    int y = evt.getY();
            if (! ((evt.getSource() == child  || evt.getSource() == this) &&
	        Beans.isInstanceOf(getParent(), BeanBoxFrame.class))) {
	        // Check if we need to resize or move.
	        Cursor cursor = getCursor();
	        if (cursor == nwResizeCursor || 
		    cursor == swResizeCursor ||
		    cursor == neResizeCursor ||
		    cursor == seResizeCursor) {
		    getBeanBox().startResize(this, x, y, cursor);
	            setCursor(defaultCursor);
	            needToRestoreCursor = false;
	        }    
	        else  { //if (cursor == moveCursor) {
		    // Start a link if the left mouse button is pressed
                    if (evt.getModifiers() == Event.META_MASK) {
                        new LinkThread(getBeanBox());
		    }
	            else {
		        getBeanBox().startMove(this, x, y);
	                setCursor(defaultCursor);
	                needToRestoreCursor = false;
                    }
	        } 
	    }
	}
        getBeanBox().mousePressed(evt);
    }
 
    public void mouseReleased(MouseEvent evt) {
        doMouseStuff(evt);
        getBeanBox().mouseReleased(evt);
    }
    public void mouseEntered(MouseEvent evt) {
        doMouseStuff(evt);
        getBeanBox().mouseEntered(evt);
    }
 
    public void mouseExited(MouseEvent evt) {
	if (active && Beans.isDesignTime() && (getParent() == null ||
	    !(getParent() instanceof BeanBoxFrame)) 
	    && evt.getSource() == this && needToRestoreCursor) {
	    setCursor(defaultCursor);
	    needToRestoreCursor = false;
	}
        getBeanBox().mouseExited(evt);
    }
 
    public void mouseDragged(MouseEvent evt) {
        doMouseStuff(evt);
        getBeanBox().mouseDragged(evt);
    }
 
    public void mouseMoved(MouseEvent evt) {
        doMouseStuff(evt);
        getBeanBox().mouseMoved(evt);
    }

    /**
     * @deprecated  Provided for backwards compatibility only.
     * We support old-style event handling as a backward compatibility
     * feature when we're wrapping a bean using the old event model.
     */
   
    public boolean handleEvent(Event evt) {
        Component source = (Component) evt.target;
        MouseEvent me;
        int x = evt.x;
        int y = evt.y;
        if (source == child) {
           // remap coordinate to child's space.
            x = x - borderWidth;
            y = y - borderWidth;
        }
        if (evt.id == Event.MOUSE_DOWN) {
            me = new MouseEvent(source, MouseEvent.MOUSE_PRESSED,
                        0, 0, x, y, 0, false);
            mousePressed(me);
        } else if (evt.id == Event.MOUSE_UP) {
            if (sawMouseDown) {
                me = new MouseEvent(source, MouseEvent.MOUSE_CLICKED,
                        0, 0, x, y, 0, false);
                mouseClicked(me);
            }
            sawMouseDown = false;
            me = new MouseEvent(source, MouseEvent.MOUSE_RELEASED,
                        0, 0, x, y, 0, false);
            mouseReleased(me);
        } else if (evt.id == Event.MOUSE_MOVE) {
            me = new MouseEvent(source, MouseEvent.MOUSE_MOVED,
                        0, 0, x, y, 0, false);
            mouseMoved(me);
        } else if (evt.id == Event.MOUSE_ENTER) {
            sawMouseDown = false;
            me = new MouseEvent(source, MouseEvent.MOUSE_ENTERED,
                        0, 0, x, y, 0, false);
            mouseEntered(me);
        } else if (evt.id == Event.MOUSE_EXIT) {
            sawMouseDown = false;
            me = new MouseEvent(source, MouseEvent.MOUSE_EXITED,
                        0, 0, x, y, 0, false);
            mouseExited(me);
        } else if (evt.id == Event.MOUSE_DRAG) {
            me = new MouseEvent(source, MouseEvent.MOUSE_DRAGGED,
                        0, 0, x, y, 0, false);
            mouseDragged(me);
        }
	else
	    return false;
        return true;
    }

    //----------------------------------------------------------------------
    private static Hashtable eventModelCache = new Hashtable();
 
    // Check if our child needs to use the old AWT event mnodel.
 
    private synchronized boolean useNewEventModel() {
        // Check our cache first.
        Boolean b = (Boolean) eventModelCache.get(child.getClass());
        if (b != null) {
            return b.booleanValue();
        }
 
        // We check whether the bean has any public methods that take
        // old AWT event objects.  If so, we assume it wants the
        // old event model, otherwise we assume it wants the new.
        boolean useNew = true;
 
        try {
            Class clz = child.getClass();
            while (useNew && clz != null) {
 
                java.lang.reflect.Method methods[] = clz.getDeclaredMethods();
 
                for (int i = 0; i < methods.length; i++) {
                    java.lang.reflect.Method m = methods[i];
                    int mods = m.getModifiers();
                    if (!java.lang.reflect.Modifier.isPublic(mods)) {
                        // Skip non=public methods.
                        continue;
                    }
                    Class params[] = m.getParameterTypes();
                    if (params.length > 0 && params[0] == java.awt.Event.class)
{
                        // First arg is java.awt.Event.  We assume this is an
                        // old-style AWT event handler method.
                        useNew = false;
                        break;
                    }
                }
                clz = clz.getSuperclass();
                if (clz.getName().indexOf("java.") == 0) {
                   // We've reached a java.* class, so we're done.
                    break;
                }
            }
        } catch (Exception ex) {
            System.err.println("Wrapper.useOldEventModel caught: " + ex);
        }
 
        eventModelCache.put(child.getClass(), new Boolean(useNew));
        return useNew;
    }
 
 
    void listenForMice(boolean enable) {
        // If the child uses the old event mode we rely on receiving events
        // through handleEvent.  Otherwise we use the new event model and
        // register explicit listeners for mouse events.
        if (!useNewEventModel()) {
// Another hack (for B&P?) -- PLI
this.addMouseListener(this);
this.addMouseMotionListener(this);
            return;
        }
        if (enable) {
            this.addMouseListener(this);
            this.addMouseMotionListener(this);
// Hack...  -- PLI
if (! (child instanceof Panel) && ! (child instanceof Canvas)) {
            child.addMouseListener(this);
            child.addMouseMotionListener(this);
}
        } else {
            this.removeMouseListener(this);
            this.removeMouseMotionListener(this);
            child.removeMouseListener(this);
            child.removeMouseMotionListener(this);
        }
    }
 
    //----------------------------------------------------------------------

    private Component getCustomizer(Object bean) {
	Container cust = new Panel();
	cust.setLayout(new BorderLayout());
	Component customizer = null;
	try {
            BeanInfo bi = Introspector.getBeanInfo(this.bean.getClass());
            BeanDescriptor bd = bi.getBeanDescriptor();
            Class customizerClass = bd.getCustomizerClass();
            Label header = new Label(bd.getDisplayName());
            header.setAlignment(Label.CENTER);
            cust.add("North", header);
 
            if (customizerClass == null) {
		// Property panel will listen for changes, so we
		// don't have to
                customizer = new PropertyPanel(this.bean);
            }
            else {
                customizer = (Component)customizerClass.newInstance();
                ((Customizer)customizer).setObject(this.bean);
                ((Customizer)customizer).addPropertyChangeListener(this);
            }
            cust.add("Center", customizer);
	    // invalidate();
            Dimension size = customizer.getPreferredSize();
            //cust.resize(size.width, size.height);
            if (cust.getParent() != null)
                cust.getParent().layout();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
	return cust;
    }

    public void propertyChange(PropertyChangeEvent evt) {
      	BeanBox box = getBeanBox(); // BeanBox.findBeanBox(this);
      	if ((box != null) && (box instanceof QuarryBox)) {
            ((QuarryBox)box).notifyParticipants(NuggetMessage.MODIFY,
           	new NuggetChangeInfo(bean, evt.getPropertyName(),
           	evt.getNewValue()));
        }
    }

    //----------------------------------------------------------------------

    final BeanBox getBeanBox() {
	Component parent = getParent();
	if (parent instanceof BeanBox) {
	    return (BeanBox)parent;
	}
	// Hmm, our parent isn't a BeanBox.  We may be the
	// top-level wrapper.
	// PLI -- Removed parent check
	//if (parent instanceof BeanBoxFrame && child instanceof BeanBox) {
	if (child instanceof BeanBox) {
	    return (BeanBox) child;
	}
	// Otheriwse we're hopelessly confused.
	throw new Error("can't find BeanBox from wrapper");
    }

    void setRubberBanding(boolean banding) {
	rubberBanding = banding;
    }


    /**
     * show or hide all the "invisible beans".
     */
    static void showInvisibleBeans(boolean show)  {
	for (int i = 0; i < invisibleWrappers.size(); i++) {
	    ((Wrapper)invisibleWrappers.elementAt(i)).setVisible(show);
	}
    }

    private Component child;
    private Object bean;
    private Object rawBean; // in case a TV is converted to a TVPanel
    private transient boolean active;
    private transient boolean needToRestoreCursor;

    private ColorSource colorSource = null;

    private transient boolean rubberBanding;

    private final static int borderWidth = 5;
    private final static int resizeDelta = 8;    

    // Shorthands for the cursors.
    private static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private static Cursor moveCursor = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

    private static Vector invisibleWrappers = new Vector();
    private transient boolean sawMouseDown;

    public void print(Graphics g) {
    	super.print(g);
    	paint(g);
    }

    // ------------ Scalable methods ---------------
    public void paintScaled(Graphics g, Dimension size) {
        Object bean = getBean();
    	if (bean instanceof Scalable)
           ((Scalable)bean).paintScaled(g, size);
    }

    public boolean scaleSelf() {
      	Object bean = getBean();
      	if (bean instanceof Scalable)
            return ((Scalable)bean).scaleSelf();
      	else
            return false;
    }
    // ------------ end Scalable methods ---------------


    private static final boolean debug = false;
    private void debug(String message){
      if (debug)
        System.err.println(getClass().getName()+": "+message);
    }
}



class WrapperListener implements ContainerListener {


  WrapperListener() {
  }

  public void componentAdded(ContainerEvent e) {
    debug("componentAdded: "+e);
  }

  public void componentRemoved(ContainerEvent e) {
    debug("componentRemoved: "+e);
  }


  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }

}
