package sieve.sluice;

/**
 * The <CODE>TableEditor</CODE> interface represents the minimal
 * level of functionality that must be provided by a class that
 * allows editing of a table. Currently no record or field removal
 * methods are specified, though it is anticipated that these will
 * be added in the future.
 * <P>
 * (<I>At some point we will write a support class that wraps
 * a simple <CODE>TableEditor</CODE> object and provides a richer
 * set of access methods</I>.)
 */
public interface TableEditor {

    /**
     * Inserts a block of records into the table. 
     * The <CODE>data</CODE> is ordered by <CODE>[field][record]</CODE> and
     * must contain entries for all fields. 
     * Fields appear in the same order as returned by TableView.getFieldNames().
     * The first record in <CODE>data</CODE> will appear in the record
     * specified by <CODE>atRecord</CODE>.
     */
    public void insert(Object[][] data, int atRecord)
        throws InterchangeException;

    /**
     * Replaces a block of records in the table. 
     * The <CODE>data</CODE> is ordered by <CODE>[field][record]</CODE> and
     * must contain entries for all fields. 
     * Fields appear in the same order as returned by TableView.getFieldNames().
     * The first record in <CODE>data</CODE> will appear in the record
     * specified by <CODE>atRecord</CODE>.
     */
    //public void replace(Object[][] data, int atRecord)
    //    throws InterchangeException;
 
    /**
     * Appends a block of records to the table. 
     * The <CODE>data</CODE> is ordered by <CODE>[field][record]</CODE> and
     * must contain entries for all fields. 
     * Fields appear in the same order as returned by TableView.getFieldNames().
     * (<I>This method will likely be removed when a wrapper class has
     * been implemented, as it can simply call <CODE>insert</CODE></I>.)
     */
    public void append(Object[][] data) throws InterchangeException;

    /**
     * Adds a field to the table. Note that some <CODE>TableEditor</CODE>
     * implementations may not allow fields to be added  if there is
     * data in the table.
     */
    public void addField(FieldDescriptor fd)
        throws InterchangeException;

    /**
     * Removes a range of records from the table. Read-only table
     * implementations may throw a ReadOnlyException.
     */
    public void remove(int startingAt, int numToRemove) 
	throws InterchangeException;

    /**
     * Removes a field from the table. Read-only table implementations 
     * may throw a ReadOnlyException.
     */
    public void removeField(String fieldName) throws InterchangeException;

}

