package sieve.sluice.codebook;

import java.util.*;
import java.io.*;

public class SGMLParser {
       
    /**
     * Extracts tags from SGML files.
     * TODO: Write a real parser with lex & yacc
     * Returns an "EOF" tag upon reaching end of file, or
     * an "END_TAGNAME" tag for an ending tag.
     */
    public static Tag getNextTag(LineNumberInputStream is) 
        throws IOException, SyntaxException {
	
        Tag returnTag = null;
        StreamTokenizer st = new StreamTokenizer(is);
        st.resetSyntax();
        st.wordChars('a', 'z');
        st.wordChars('A', 'Z');
        st.wordChars('0', '9');
        st.whitespaceChars(0, 32);
        st.quoteChar(34);
	/*
        st.ordinaryChar('/'); // Otherwise treated as a comment character
        //st.ordinaryChar('!');
        st.wordChars('0', '9');	// Temporary fix--treat all numbers as words
	*/

        while (st.nextToken() != '<') // Assumes EOF won't happen inside a tag
            if (st.ttype == st.TT_EOF) return new Tag("EOF");

        if (st.nextToken() == '/') {    // It's an ending tag
            if (st.nextToken() != st.TT_WORD)
                throw new SyntaxException(is.getLineNumber());
            returnTag = new Tag("END_" + st.sval.toUpperCase());
            if (st.nextToken() != '>')
                throw new SyntaxException(is.getLineNumber());
        }
        else if (st.ttype == '!') {
            returnTag = new Tag("EXCLAMATION_TAG");
            // ignore exclamation pt tags for now--just read to end
            while (st.nextToken() != '>');
        }
        else {      // It's a beginning tag
            if (st.ttype != st.TT_WORD)
                throw new SyntaxException(is.getLineNumber());
            returnTag = new Tag(st.sval.toUpperCase());     // Get name
            while (st.nextToken() != '>')  { // Get attlist for tag
                if (st.ttype != st.TT_WORD)
                    throw new SyntaxException(is.getLineNumber());
                String attname = st.sval.toUpperCase();
                if (st.nextToken() != '=') 
                    throw new SyntaxException(is.getLineNumber());
                if (st.nextToken() == st.TT_NUMBER) {
		    System.out.println("WARNING: Got a number token");
                    returnTag.attlist.put(attname, "" + st.nval);
		}
                else if (st.sval == null) 
                    throw new SyntaxException(is.getLineNumber());
                else returnTag.attlist.put(attname, st.sval);
            }
        }
        return returnTag;
    }
}



/**
 * Spits out token types as it tokenizes.  Used for debugging.
 */
class DebugTokenizer extends StreamTokenizer {
    public DebugTokenizer(InputStream is) { super(is); }
    public int nextToken() throws IOException {         // Overrides parent version
        System.out.println("ttype is " + super.nextToken());
        return ttype;
    }
}


