package sieve.sluice.views;

import sieve.sluice.*;
import sieve.sluice.event.*;

/**
 * FieldXYView tried to subclass FieldNameSelectionView but ended up needing to
 * override too much. It is a view of a CodebookFinder table which maintains a single
 * field as the independent variable (x) and all other fields as dependent variables (y).
 * In reality, the only way it does this is to use a FieldNameSelectionView to keep the
 * y-field selections and a String to record the x-field. Objects listening
 * to the table ask for the fields then do a get() on those fields. 
 * FieldXYView has a customizer widget.
 *
 * @see sieve.sluice.DefaultTableView
 * @see sieve.sluice.codebook.CodebookFinder
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */

public class FieldXYView extends DefaultTableView implements TableView, TableViewListener {
	
	FieldNameSelectionView	fnsv = null;	//keeps track of y-field selections
	String					xField = null;

	public FieldXYView(TableView t) {
		super(t);	//this keeps track of the original table
		fnsv = new FieldNameSelectionView();	
		try {fnsv.addSourceView(t);}
		catch(TooManySourcesException e) {}
	}

	public FieldXYView(TableView t, String[] fds) {
		super(t);	//this keeps track of the original table
		fnsv = new FieldNameSelectionView();	
		try {fnsv.addSourceView(t);}
		catch(TooManySourcesException e) {}
		setFields(fds);
	}

	public FieldXYView() {
		super();
	}

	public void setXField(String x) {
		xField = new String(x);
		eventHandler.fireTableViewEvent(new TableViewEvent(this));
	}

	public void setYFields(String[] fds) {
		if(fnsv != null) {
			fnsv.setFields(fds);
			eventHandler.fireTableViewEvent(new TableViewEvent(this));
		}	//otherwise, NoSourceException, but similar classes don't throw for this method
	}

	public void setFields(String[] fds) {
		//this sets first to X and rest to Y. If only 1, sets it to both X and Y
		if((fds != null) && (fds.length > 0)) {
			String[] tmp = null;
			int g=fds.length-1;
			if(g>0) {	//then there are at least 2 fields 
				tmp = new String[g];
				for(int y=0;y<g; y++)
					tmp[y] = fds[y+1];
			}
			else {	//g==0, set same for x and y
				tmp = new String[1];
				tmp[0] = fds[0];
			}
			xField = new String(fds[0]);
			fnsv.setFields(tmp);
		}
		else {
			xField = null;
			fnsv.setFields(new String[0]);
		}
		eventHandler.fireTableViewEvent(new TableViewEvent(this));	//this will make listeners get the changes
	}

	public String getXFieldName() {
		return xField;
	}

	public String[] getYFieldNames() {
		return fnsv.getFields();
	}

	public String[] getFieldNames() {
	//not sure this is exactly what we want -- if y fields chosen but no x field,
	//we won't show any as being selected (this is useful on initialization, where 
	//we need this to return null)
		String[] ys = null;	
		if(fnsv != null) 
			ys = fnsv.getFields();
		String[] ans = null;
		if(xField != null) {
			if (ys != null) {
				int h= ys.length+1;
				ans = new String[h];
				for(int j=1;j<h;j++) 
					ans[j] = ys[j-1];
			}
			else 
				ans = new String[1];
			ans[0] = xField;
			return ans;
		}
		else
			return null;
	}

	public String[] getFields() {
		return getFieldNames();
	}

	public String[] getAvailableFieldNames() throws NoSourceException {
		if(source == null)
			throw new NoSourceException("FieldXYView has no source");
		else
			return source.getFieldNames();
	}

	public FieldNameSelectionView getYTable() {
		return fnsv;
	}

	public void addSourceView(TableView t) throws TooManySourcesException {
		super.addSourceView(t);
		fnsv = new FieldNameSelectionView();
		try {fnsv.addSourceView(t);}
		catch(TooManySourcesException e) {}
	}

	public void removeSourceView(TableView source) {
		super.removeSourceView(source);
		fnsv = null;
	}


}
