package sieve.sluice.views;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sieve.sluice.*;

/**
 * Customizer widget for FieldXYView. This takes lots of code from FieldNameSelectionViewCustomizer
 * but couldn't extend it directly. Has java.awt.Choice for selecting a single x-field
 * and a java.awt.List for single or multi-select of y-fields. Orders x as the first
 * field and y's as the remaining fields in its target table. 
 *
 * @see sieve.sluice.views.FieldXYView
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */


public class FieldXYViewCustomizer extends Panel 
  implements Customizer, TableViewListener, ItemListener, PropertyChangeListener {

	private		boolean			isMultiple = true;
	private		Panel			p1 = new Panel();
	private		Panel			p2 = new Panel();
	protected	Choice			xCh = null;
	protected	FieldXYView		target = null;	
    protected	List			fieldList;
    protected	String[] 		fields;
    protected	PropertyChangeSupport support = new PropertyChangeSupport(this);

    public static final String NOT_CONNECTED_MSG = "(Not Connected)";


	//------- CONSTRUCTORS -------------------------------------
	public FieldXYViewCustomizer(boolean multi) {
	/**
	 * if <code>multi</code> is false, creates customizer with a single-select 
	 * List for y-fields
	 */
		isMultiple = multi;
		setLayout(new BorderLayout()); 
		setBackground(Color.white);
	}

	public FieldXYViewCustomizer() {
	/**
	 * creates customizer with a multi-select List for y-fields
	 */
		setLayout(new BorderLayout()); 
		setBackground(Color.white);
	}

	public void setObject(Object obj) {
	/**
	 * sets the table to be customized
	 */
		target = (FieldXYView) obj;
		target.addTableViewListener(this); 
		fieldList = new List(8, isMultiple);	
		fieldList.addItemListener(this);
		xCh = new Choice();	
		xCh.addItemListener(this);
		xCh.addItem(NOT_CONNECTED_MSG);
		setLayout(new BorderLayout());
		
		updateList();
		
		p1.setLayout(new BorderLayout());
		Label xLab = new Label("Choose Independent Variable (x-axis)");
		xLab.setBackground(Color.white);
		p1.add("North",xLab);
		p1.add("Center",xCh);

		p2.setLayout(new BorderLayout());
		Label yLab = new Label();
		yLab.setBackground(Color.white);
		if(isMultiple) 
			yLab.setText("Choose Dependent Variables (y-axis)");
		else
			yLab.setText("Choose Dependent Variable (y-axis)");
		p2.add("North",yLab);
		p2.add("Center",fieldList);

		add("North", p1);
		add("Center",p2);
		setSize(getPreferredSize());
	}

	protected void updateList() {
	/**
	 * gets the field names from the target table for the lists. Called by
	 * setObject and tableViewChanged. Customizer does not listen for table
	 * view changes unless explicitly set by container.
	 */
	   	String[] newFields = null;
		try { newFields = target.getAvailableFieldNames(); }
        catch (Exception e) {e.printStackTrace(); } 
		int i=0;					
		xCh.removeAll();	//have to remove all regardless, since can't use replaceItem like in List
		if (newFields == null) {
			fieldList.removeAll();
			fieldList.addItem(NOT_CONNECTED_MSG);
			fieldList.setEnabled(false);
			xCh.addItem(NOT_CONNECTED_MSG);
			xCh.setEnabled(false);
		}
		else {
			String desc;
			if (newFields.length == 0) {
				fieldList.removeAll();			
				fieldList.addItem("No Fields");	
				fieldList.setEnabled(false);
				xCh.removeAll();
				xCh.addItem("No Fields");	
				xCh.setEnabled(false);
			}
			else {
				fieldList.setEnabled(true);
				xCh.setEnabled(true);
				for (i=0; i < newFields.length; i++) {
					desc = null;
					try { desc = target.source.getFieldDescriptor(newFields[i]).getDescription(); }
					catch (Exception e) {}

				    if ((desc == null) || desc.equals(newFields[i]))
			           desc = newFields[i];
				    else
					   desc = desc + " (" + newFields[i] + ")";

					if (i >= fieldList.getItemCount()) 
			            fieldList.addItem(desc);
					else if (!desc.equals(fieldList.getItem(i))) 
						fieldList.replaceItem(desc, i); 
					xCh.addItem(desc);	//no replaceItem() for Choice class
				}
			}//else (newFields.length != 0)

			fields = newFields;
			String[] currentFields = target.getFields(); //if fields is null, this should be too!
			if (currentFields != null) {
				for (i=0; i < currentFields.length; i++) {
				    for (int j = 0; j < fields.length; j++) {
						if (currentFields[i].equals(fields[j])) {
							if(i==0)
								xCh.select(j);	//guessing these are in order. If behaves unexpectedly,
							else {				// then we'll wait until list is set, then find the first selected in it
								if (!fieldList.isIndexSelected(j)) //not sure why we care if it already is...
								    fieldList.select(j);
							}
							j = fields.length; // Break out
						}
					}
				}
			}
		}
		setSize(getPreferredSize());
	}


	public void itemStateChanged(ItemEvent ie) {
	/**
	 * If a field is selected, check to see if same X and Y; if so deselect.
	 * If a field is either selected or deselected, update the table with the change.
	 */
		if( (ie.getSource() == fieldList) || (ie.getSource() == xCh) ) {//if either of our components
			if (ie.getStateChange() == ItemEvent.SELECTED)  { //if something selected
				int which = xCh.getSelectedIndex();
				if((fieldList.getItemCount() > 1) && (fieldList.isIndexSelected(which)) )  //(fieldList.getItem(which).compareTo(xCh.getSelectedItem()) == 0)) {	
					fieldList.deselect(which);	//don't allow same x and y (doesn't generate another item event
			}
			resetFields();
		}
	}


	private void resetFields() {
	/**
	 * gets the selections, translates to actual field names if necessary, and 
	 * calls target.setFields
	 */
        /*
         * Since we may have field descriptions (rather than field names) in the List widget,
		 * translate indices here (per Philip's code in FieldNameSelectionViewCustomizer). 
		 * This is a bit more work than using setXField and setYFields, but it will catch all
		 * changes (for example, a select in x which also causes a deselect in y).
         */
		if(fields != null) {
	     int[] selection = fieldList.getSelectedIndexes();
			String[] selectedFields;
			if(xCh.getSelectedIndex() > (-1)) {
				if(selection != null) {
					selectedFields = new String[selection.length + 1];
					for (int i = 1; i <= selection.length; i++) 
						selectedFields[i] = fields[selection[i-1]];
				}
				else
					selectedFields = new String[1];
				selectedFields[0] = fields[xCh.getSelectedIndex()];
				//support.firePropertyChange("fields", target.getFields(), selectedFields);
				target.setFields(selectedFields);	//in theory, we have set the fields in the proper x,y order to do the get()
			}
		}
    }

   public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals("fields")) {
			String[] selectedFields = (String[])evt.getNewValue();
			if (selectedFields != null) {
				for (int i = 0; i < fields.length; i++) {
					fieldList.deselect(i);
					if(fields[i].equals(selectedFields[0]))
						xCh.select(i);
					else {
						for (int j = 1; j < selectedFields.length; j++) {
							if (fields[i].equals(selectedFields[j]))
								fieldList.select(i);
						}
					}
				}
			}
		}
   }	//don't we want to resetFields() ? or does that cause looping? (maybe we get an item event...)
		//or fire a property change?


   public Dimension getPreferredSize() {
	   //Choice likes to jump out of its container if the container is too small.
	   //Here we make sure it's big enough
		return new Dimension(Math.max(p1.getPreferredSize().width,p2.getPreferredSize().width), p2.getPreferredSize().height*2);
   } 

   public void addPropertyChangeListener(PropertyChangeListener l) {
		support.addPropertyChangeListener(l);
   }

   public void removePropertyChangeListener(PropertyChangeListener l) {
		support.removePropertyChangeListener(l);
   }


   public void tableViewChanged(TableViewEvent event) {
		updateList();
   }


}
