package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;
import java.lang.*; 

/**
 *  A NonReplaceSampleView shrinks the size of the table by randomly
 *  sampling k records (without replacemnt) from the full body of samples
 **/

public class NonReplaceSampleView extends SampleView implements TableView {
    public NonReplaceSampleView(TableView source, int size) {
       super(source, new NonReplaceSampler(source, size));
    }
}

class NonReplaceSampler implements Sampler {

    private int       size     = 0;
    private int       viewSize = 0;
    private int       nextRec  = 0; 
    private TableView source   = null;
    int[]   records;

    public NonReplaceSampler(TableView inSource, int inSize)
    {
        int i=0;

        // Set the tableview source, sample size and total # of records
        source   = inSource;
        size     = inSize;
        try
        {
          viewSize = source.getRecordCount();
        }
        catch (Exception e)
        {
          // Not sure what should be done if this fails... 
	  e.printStackTrace(); 
        }

        // create and initialize the array of record numbers;
        records = new int[viewSize];
        for(i=0;i<viewSize;i++)
            records[i] = i;

        // Now shuffle those numbers a bit..
        Random generator = new Random();
        for(i=0;i<viewSize-1;i++)
        {

          int swapPos = Math.abs(generator.nextInt()) % (viewSize - i) + i;
          int temp         = records[i];
          records[i]       = records[swapPos];
          records[swapPos] = temp; 

        }

    }

    public int getRecordNumber(int requestedRecord)
    throws NoSuchRecordException
    {
       // if out of range throw the excpetion
       if ((requestedRecord < 0) || (requestedRecord >= size))
          throw new NoSuchRecordException(Long.toString(requestedRecord));

       // Otherwise simply return the next table value that cooresponds to it.
       return records[requestedRecord]; 
    }

    public int size()
    {
      return size;
    }

    public boolean recordIsInSample(int record)
    {
       for(int i=0;i<size;i++)
          if (records[i] == record) return true;

       return false; 
    }

}
