package sieve.sluice.views.predicates;

import java.lang.reflect.*;
import java.util.Vector;
import java.util.Hashtable;

public abstract class Predicate {
    Object predicatesObject = null;
    Method predicateMethod = null;

    private static Hashtable predicatesClasses = null;
    private static Hashtable predicatesObjects = null;

    String name = null;

    static {
	predicatesClasses = new Hashtable();
	predicatesObjects = new Hashtable();
	predicatesClasses.put(Object.class, 
	    sieve.sluice.views.predicates.ObjectPredicates.class);
	predicatesClasses.put(Number.class, 
	    sieve.sluice.views.predicates.NumberPredicates.class);
	predicatesClasses.put(String.class, 
	    sieve.sluice.views.predicates.StringPredicates.class);
	predicatesClasses.put(java.util.Date.class, 
	    sieve.sluice.views.predicates.DatePredicates.class);
    }

    public static void registerPredicatesClass(Class forClass, 
	Class predsClass) {
	try {
	    predicatesObjects.put(forClass, predsClass.newInstance());
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	predicatesClasses.put(forClass, predsClass);
    }
    
    public Predicate(Class aClass, String name) {
	this.predicatesObject = getPredicatesObject(aClass);
	this.name = name;
	String methodName = getMethodName(name);
	try {
            Method[] methods = predicatesObject.getClass().getMethods();
            for (int i = 0; i < methods.length && predicateMethod == null; i++)
                if (methods[i].getName().equals(methodName))
		    predicateMethod = methods[i];
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Predicate(Class aClass, Method predicateMethod) {
	this.predicatesObject = getPredicatesObject(aClass);
	this.predicateMethod = predicateMethod;
	this.name = getDisplayName(predicateMethod.getName());
    }

    public static Predicate getPredicateFromName(Class aClass, String name) {
	Object predicatesObject = getPredicatesObject(aClass);
	Method predicateMethod = null;
	String methodName = getMethodName(name);
	try {
            Method[] methods = predicatesObject.getClass().getMethods();
            for (int i = 0; i < methods.length && predicateMethod == null; i++){
                if (methods[i].getName().equalsIgnoreCase(methodName))
		    predicateMethod = methods[i];
	    }
        }
        catch (Exception e) {
            e.printStackTrace();
	}
	if (predicateMethod == null)
	    return null;
	else if (predicateMethod.getParameterTypes().length == 1)
	    return new UnaryPredicate(aClass, predicateMethod);
	else
	    return new BinaryPredicate(aClass, predicateMethod);
    }

    public String getName() {
	return name;
    }

    private static Object getPredicatesObject(Class aClass) {
	// See if we've created a predicates object before...
        ObjectPredicates obj = (ObjectPredicates)predicatesObjects.get(aClass);
	if (obj != null)
	    return obj;

	// If not, first check the table of classes, then look for one
	// the hard way by constructing a class name
        while ((obj == null) &&
            (!aClass.getName().equals("java.lang.Object"))) {
	    Class predClass = (Class)predicatesClasses.get(aClass);
	    if (predClass == null) {
                try {
                    String name = aClass.getName();
                    name = "sieve.sluice.views.predicates." +
                        name.substring(name.lastIndexOf('.') + 1) + 
			"Predicates";
                    predClass = Class.forName(name);
                }
                catch (Exception e) {
                    aClass = aClass.getSuperclass();
		    predClass = null;
                }
	    }
	    if (predClass != null) {
		try {
            	    obj = (ObjectPredicates) predClass.newInstance();
		}
            	catch (Exception e) {
                    aClass = aClass.getSuperclass();
		    obj = null;
                }
	    }
        }
        if (obj == null)
            obj = new ObjectPredicates();

	predicatesObjects.put(aClass, obj);
        return obj;
    }

    /* ------------ Methods for retrieving display names ------------ */
    private static String[] getPredicateNames(Class aClass, int numArgs) {
	Object predObj = getPredicatesObject(aClass);
	String[] predNames = null;
        try {
            Method[] methods = predObj.getClass().getMethods();
            Vector predicates = new Vector();
 
            // Look for methods that return Boolean
            for (int i = 0; i < methods.length; i++) {
                String name = methods[i].getName();
		if (! name.equals("equals")) { // Filter out Object.equals(obj)
                    String returnType = methods[i].getReturnType().getName();
                    if ((returnType.equals("boolean") ||
                         returnType.equals("java.lang.Boolean")) &&
                        (methods[i].getParameterTypes().length == numArgs))
                        predicates.addElement(name);
		}
            }
	    predNames = new String[predicates.size()];
	    for (int i = 0; i < predNames.length; i++) {
		predNames[i] = getDisplayName((String)predicates.elementAt(i));
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	return predNames;
    }

    public static String[] getBinaryPredicateNames(Class aClass) {
        return getPredicateNames(aClass, 2);
    }
    public static String[] getUnaryPredicateNames(Class aClass) {
        return getPredicateNames(aClass, 1);
    }


    /* --------------- Methods for name manipulation --------------- */

    public static String getMethodName(String displayName) {
	String methodName = displayName;

	// Don't do anything to all-caps display names
        if (! methodName.toUpperCase().equals(methodName)) {
            StringBuffer buf = new StringBuffer(methodName);

            buf.setCharAt(0, Character.toLowerCase(buf.charAt(0)));

            // Remove spaces
            int len = methodName.length();
            for (int ch = 1; ch < len; ch++) {
                if (Character.isSpace(buf.charAt(ch))) {
                    for (int j = ch; j < len - 1; j++)
                        buf.setCharAt(j, buf.charAt(j + 1));
            	    buf.setCharAt(ch, Character.toUpperCase(buf.charAt(ch)));
                    ch--;
                    len--;
                }
            }
            buf.setLength(len);
            methodName = buf.toString();
        }
	return methodName;
    }

    public static String getDisplayName(String methodName) {

	StringBuffer buf = new StringBuffer(methodName);

        buf.setCharAt(0, Character.toUpperCase(buf.charAt(0)));

	int len = methodName.length();
        for (int ch = 1; ch < len; ch++) {
            if (Character.isUpperCase(buf.charAt(ch))) {
                buf.insert(ch, ' ');
                ch++;
                len++;
            }
        }

	return buf.toString();
    }

}

