package sieve.sluice.weblog;

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.SimpleDateFormat;
import sieve.sluice.*;
import sieve.sluice.NoSuchFieldException;

public class TableImpl extends Table {
    String[]	fields	= {
	"Host name", "User name", 
	"Date", "Operation", "Document",
	"Protocol", "Protocol version",
	"Return code", "Size", 
	"Referer", "Browser",
	"Browser version", "Browser details"};
    String[]	types	= {
	"java.lang.String", "java.lang.String", 
	"java.util.Date", "java.lang.String",
	"java.lang.String", "java.lang.String",
	"java.lang.String", "java.lang.Integer",
	"java.lang.Integer", "java.lang.String",
	"java.lang.String", "java.lang.String", 
	"java.lang.String"};

    int	records	= 0;

    Hashtable	idx	= null;

    URL		log	= null;

    /* Example: 29/Oct/1996:13:19:09 -0500 */
    SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MMM/yyyy:HH:mm:ss");

    static Hashtable 	returnCodes = null;

    static {
        returnCodes = new Hashtable();
        // From RFC 1945 (HTTP 1.0) and IETF June '96 draft for HTTP 1.1
        returnCodes.put(new Integer(100), "Continue"); // 1.1
        returnCodes.put(new Integer(101), "Switching Protocol"); // 1.1
        returnCodes.put(new Integer(200), "OK");
        returnCodes.put(new Integer(201), "Created");
        returnCodes.put(new Integer(203), 
	    "Non-Authoritative Information"); //1.1
        returnCodes.put(new Integer(202), "Accepted");
        returnCodes.put(new Integer(204), "No Content");
        returnCodes.put(new Integer(205), "Reset Content"); // 1.1
        returnCodes.put(new Integer(206), "Partial Content"); // 1.1
        returnCodes.put(new Integer(300), "Multiple Choices"); // 1.1
        returnCodes.put(new Integer(301), "Moved Permanently");
        returnCodes.put(new Integer(302), "Moved Temporarily");
        returnCodes.put(new Integer(303), "See Other"); // 1.1
        returnCodes.put(new Integer(304), "Not Modified");
        returnCodes.put(new Integer(305), "Use Proxy"); // 1.1
        returnCodes.put(new Integer(400), "Bad Request");
        returnCodes.put(new Integer(401), "Auth Forbidden");
        returnCodes.put(new Integer(402), "Payment Required"); // 1.1
        returnCodes.put(new Integer(403), "Forbidden");
        returnCodes.put(new Integer(404), "Not Found");
        returnCodes.put(new Integer(405), "Method Not Allowed"); // 1.1
        returnCodes.put(new Integer(406), "Not Acceptable"); // 1.1
        returnCodes.put(new Integer(407), 
	    "Proxy Authentication Required"); // 1.1
        returnCodes.put(new Integer(408), "Request Time-out"); // 1.1
        returnCodes.put(new Integer(409), "Conflict"); // 1.1
        returnCodes.put(new Integer(410), "Gone"); // 1.1
        returnCodes.put(new Integer(411), "Length Required"); // 1.1
        returnCodes.put(new Integer(412), "Precondition Failed"); // 1.1
        returnCodes.put(new Integer(413), "Request Entity Too Large"); // 1.1
        returnCodes.put(new Integer(414), "Request-URI Too Large"); // 1.1
        returnCodes.put(new Integer(415), "Unsupported Media Type"); // 1.1
        returnCodes.put(new Integer(500), "Internal Server Error");
        returnCodes.put(new Integer(501), "Not Implemented");
        returnCodes.put(new Integer(502), "Bad Gateway");
        returnCodes.put(new Integer(503), "Service Unavailable");
        returnCodes.put(new Integer(504), "Gateway Time-out"); // 1.1
        returnCodes.put(new Integer(505), "HTTP Version not supported"); // 1.1
    }

    public TableImpl(String url) throws InterchangeException {
	setLocation(url);
    }

    public TableImpl() throws InterchangeException {
        // setLocation("http://simon.cs.vt.edu/~begolej/access_log");
        setLocation("/data/ei_log");
    }


    /* ---------------  Property methods --------------- */
    public String getTableType() {
        return "Web Server Log";
    }

    public void setLocationURL(URL log) {
	this.log = log;
	idx = new Hashtable();
	for (int i = 0; i < fields.length; i++) {
	    idx.put(fields[i], new Integer(i));
	} 
        records = 0;
        eventHandler.fireTableViewEvent(new TableViewEvent(this));
     }	

     public URL getLocationURL() {
	return log;
     }

     /* ---------------  Table methods --------------- */
     public void insert(Object[][] data, int atRecord) throws 
        InterchangeException {
	throw new ReadOnlyException();
     }

     public void append(Object[][] data) throws InterchangeException {
	throw new ReadOnlyException();
     }

     public void addField(FieldDescriptor fd) throws InterchangeException {
	throw new ReadOnlyException();
     }

     public void remove(int firstRecord, int lastRecord) throws
         InterchangeException {
         throw new ReadOnlyException();
     }

     public void removeField(String fieldName) throws InterchangeException {
         throw new ReadOnlyException();
     }

     /* -------------  TableView methods ------------- */
     public TableView table() {
	return this;
     }

     public TableDescriptor getTableDescriptor() {
          // For now we do nothing...
          return new TableDescriptor();
     }

    public String[] getFieldNames() {
	return fields;
    }

    public FieldDescriptor getFieldDescriptor(String field) throws 
        InterchangeException{
	Integer i = (Integer) idx.get(field);
	    if (i == null) 
	        throw new NoSuchFieldException(field);
        FieldDescriptor fd = new FieldDescriptor(field);
        if (field.equals("Return code")) {
            fd.setLabels(returnCodes);
        }
        if (field.equals("Size") || field.equals("Date"))
            fd.setLevel(FieldDescriptor.LEVEL_INTERVAL);
	else
            fd.setLevel(FieldDescriptor.LEVEL_NOMINAL);
        fd.setType(types[i.intValue()]);
        return fd;
    }


    public int getRecordCount() {
	if (records == 0) {
	    try {
		DataInputStream stream = new DataInputStream(
		    new BufferedInputStream(log.openStream()));
		String line = null;
		while ((line = stream.readLine()) != null)
		    records++;
	    } 
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
	return records;
    }

    public Object[][] get(String[] requestedFields, int startingAt,
	int numberOfRecords) throws InterchangeException {
	
	int 		records = getRecordCount();
	Object[][] 	result	= null;

	if (startingAt >= records)
	    throw new NoSuchRecordException(Long.toString(startingAt));
	else if (startingAt + numberOfRecords > records)
	    throw new NoSuchRecordException(Long.toString(records));
	else {
	    result = new Object[requestedFields.length][(int)numberOfRecords];

	    int[] map = new int[fields.length];
	    for (int i = 0; i < fields.length; i++)  map[i] = -1;
	    for (int i = 0; i < requestedFields.length; i++) {
	        Integer pos = (Integer)(idx.get(requestedFields[i]));
	        if (pos == null)
	  	    throw new NoSuchFieldException(requestedFields[i]);
	        map[pos.intValue()] = i;
	    }

	    try {
	        DataInputStream stream = new DataInputStream(
		    new BufferedInputStream(log.openStream()));
	        String 		line = null;
	        StringTokenizer	tok = null;
	        int 		counter = 0;
	        while (counter < startingAt) {
		    line = stream.readLine();
		    counter++;
	        }
	        counter = 0;
	        while (counter < numberOfRecords) {
		    line = stream.readLine();
		    tok = new StringTokenizer(line);
		    String token = null;
 		    Object obj = null;
		    int tokenNumber = 0;

		    while (tok.hasMoreTokens()) {
		        token = tok.nextToken(" ");
                        obj = null; 

		        /* User name */
		        if (tokenNumber == 1)  
			    token = tok.nextToken("-");

		        /* Date */
		        else if (tokenNumber == 2) {
			    token = token.substring(1) + tok.nextToken("]");
			    try {
			        obj = dateFormat.parse(stripQuotes(token));
                            }
                            catch (Exception e) {} // Ignore parse errors
                        }

		        /* Operation */
		        else if (tokenNumber == 3)
			    token = token.substring(1);

		        /* Protocol and version  or browser and version */
		        else if ((tokenNumber == 5) || (tokenNumber == 10)) {
			    if (map[tokenNumber] != -1) {
			        int stopIdx = token.indexOf("/");
			        if (stopIdx == -1) stopIdx = token.length();
			        if (token.startsWith("\""))
				    result[map[tokenNumber]][(int)counter] = 
 				        stripQuotes(token.substring(1,stopIdx));
			        else {
				    result[map[tokenNumber]][(int) counter] = 
				        stripQuotes(token.substring(0, stopIdx));
			        }
			    }
			    tokenNumber++;
			    token = token.substring(token.indexOf("/") + 1, 
			        token.length());
		        }
                        /* Return code and size */
                        else if ((tokenNumber == 7) || (tokenNumber ==8)) {
                            obj = new Integer(stripQuotes(token));
                        }
		        /* Browser details */
		        else if ((tokenNumber == 12) && (tok.hasMoreTokens())) {
			    token = token + " " + tok.nextToken("\"");
		        }
						
		        if (map[tokenNumber] != -1) {
                            if (obj == null)
				obj = stripQuotes(token);
			    result[map[tokenNumber]][(int)counter] = obj;
                        }

		        tokenNumber++;
		    }

		    counter++;
	        }
	    }
	    catch(Exception e) {
	        e.printStackTrace();
	    }
	}
	return result;
    }

    String stripQuotes(String token) {
	if (token.endsWith("\""))
	    if (token.length() > 1)
		token = token.substring(0, token.length() - 1);
	    else
	        /* Would null be more appropriate? */
		return "(unknown)";

	if (token.startsWith("\""))
	    token = token.substring(1);
	return token;
    }

    public Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException {
	return get(fields, startingAt, getRecordCount() - startingAt);
    }
}

