package sieve.util;


import java.awt.*;

// Put this in vt.cs.widgets?
public class Arrow extends Component implements Shape {
    Point 	start 		= null;
    Point 	end 		= null;
    Polygon 	oldPolygon 	= null;
    Polygon 	polygon 	= null;
    int		arrowHeadLength = 10;
    int		arrowHeadWidth  = 10;

    public Arrow(Point start, Point end, int arrowHeadLength, 
	int arrowHeadWidth) {
	this.start = start;
	this.end = end;
	this.arrowHeadLength = arrowHeadLength;
	this.arrowHeadWidth = arrowHeadWidth;
    }

    public void setStartPoint(Point start) {
	if ((start != null) && (! start.equals(this.start))) {
	    this.start = start;
	    repaint();
	}
    }
    public Point getStartPoint() {
	return start;
    }

    public void setEndPoint(Point end) {
	if ((end != null) && (! end.equals(this.end))) {
	    this.end = end;
	    repaint();
	}
    }
    public Point getEndPoint() {
	return end;
    }

    public void setArrowHeadLength(int arrowHeadLength) {
	if (arrowHeadLength != this.arrowHeadLength) {
	    this.arrowHeadLength = arrowHeadLength;
	    repaint();
	}
    }
    public int getArrowHeadLength() {
	return arrowHeadLength;
    }

    public void setArrowHeadWidth(int arrowHeadWidth) {
	if (arrowHeadWidth != this.arrowHeadWidth) {
	    this.arrowHeadWidth = arrowHeadWidth;
	    repaint();
	}
    }
    public int getArrowHeadWidth() {
	return arrowHeadWidth;
    }

    public Rectangle getBounds() {
	return super.getBounds();
    }

    public void paint(Graphics g) {
        // Erase the old polygon
        // Can't just do a clearRect 'cause it would wipe out other links, too
        if (oldPolygon != null) {
            erase(g);
        }

	// Paint the new one
	g.setColor(getForeground());
	polygon = paintArrow(g, start, end, arrowHeadLength, arrowHeadWidth);

        // Set the oldPolygon for later erasure
        oldPolygon = polygon;
    }

    public static Polygon paintArrow(Graphics g, Point start, Point end,
	int arrowHeadLength, int arrowHeadWidth) {
	Polygon polygon = null;
	Point	arrowStart = null;
        int   dist = GraphicsUtil.getDistance(start, end);
        if (dist < 2 * arrowHeadLength)
            arrowStart = new Point((end.x + start.x)/2,
                (end.y + start.y)/2);
        else
            arrowStart = new Point(end.x -
                arrowHeadLength * (end.x-start.x)/dist,
                end.y -
                arrowHeadLength * (end.y-start.y)/dist);
 
        double theta;
        if (arrowStart.y == end.y)
            theta = 0;
        else
            theta = Math.atan((end.x - arrowStart.x)/
                (double)(arrowStart.y - end.y)) + Math.PI/2;
 
        // Copied from Andy
        int[] xArray = new int[5];
        xArray[0] = arrowStart.x;
        xArray[1] = arrowStart.x - (int)(arrowHeadWidth * Math.sin(theta));
        xArray[2] = end.x;
        xArray[3] = arrowStart.x + (int)(arrowHeadWidth * Math.sin(theta));
        xArray[4] = arrowStart.x;
        int[] yArray = new int[5];
        yArray[0] = arrowStart.y;
        yArray[1] = arrowStart.y + (int)(arrowHeadWidth * Math.cos(theta));
        yArray[2] = end.y;
        yArray[3] = arrowStart.y - (int)(arrowHeadWidth * Math.cos(theta));
        yArray[4] = arrowStart.y;
        polygon = new Polygon(xArray, yArray, xArray.length);
 
  
        // Draw the polygon
        g.drawLine(start.x, start.y, arrowStart.x, arrowStart.y);
        g.fillPolygon(polygon);
	return polygon;
    }
 
    public void erase(Graphics g) {
        g.setColor(getParent().getBackground());
        g.fillPolygon(oldPolygon);
    }

    public boolean contains(int x, int y) {
        if (polygon != null)
            return polygon.contains(x, y);
        else
            return false;
    }


}

