package sieve.util;


import java.awt.*;

/**
 * Class for basic graphics utilities.
 */
public class GraphicsUtil {

    static int getDistance(Point p1, Point p2) {
        return (int) Math.sqrt(
            (p1.x - p2.x) * (p1.x - p2.x) +
            (p1.y - p2.y) * (p1.y - p2.y));
    }

    static Point getIntersection(Rectangle bounds, Point lineStart,
        Point lineEnd) {
        Point inside = null;
        Point outside = null;
        Point intersection = null;
 
        if (bounds.contains(lineEnd)) {
            inside = lineEnd;
            outside = lineStart;
        }
        if (bounds.contains(lineStart)) {
            if (inside != null) { // Line is completely within the rect
                return null;
            }
            outside = lineEnd;
            inside = lineStart;
        }
        if (inside == null) { // Line is completely outside the rect
            return null;
        }
        if (inside.x == outside.x) { // Vertical line
            if (outside.y <= bounds.y) // Outside pt is north
                 intersection = new Point(outside.x, bounds.y);
            else
                 intersection = new Point(outside.x, bounds.y + bounds.height);
        }
        else if (inside.y == outside.y) { // Horizontal line
            if (outside.x <= bounds.x) // Outside pt is west
                 intersection = new Point(bounds.x, outside.y);
            else
                 intersection = new Point(bounds.x + bounds.width, outside.y);
        }
        else {
            double m = - (((double)inside.y - (double)outside.y)/
                ((double)inside.x - (double)outside.x));
            double b = (double)inside.y - m * (double)inside.x;
            double y =  m * (double)bounds.x + b;
            if ((y >= bounds.y) && (y <= bounds.y + bounds.height)) { // e-w
                if (outside.x <= bounds.x) // Outside pt is west
                     intersection = new Point(bounds.x, 2 * bounds.y +
                        bounds.height - (int)y);
                else
                     intersection = new Point(bounds.x + bounds.width, (int)y);
            }
            else { // n-s line
                double x = ((double)bounds.y - b)/m;
                if (outside.y <= bounds.y) // Outside pt is north
                    intersection = new Point(2 * bounds.x + bounds.width -
                        (int)x, bounds.y);
                else
                    intersection = new Point((int)x, bounds.y + bounds.height);
            }
        }
 
        return intersection;
    }


}

