package sieve.whiteboard;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import com.sun.java.swing.*; 

public class WBLabel extends JTextField implements KeyListener, Runnable {
    private PropertyChangeSupport support = new PropertyChangeSupport(this);
    private Dimension mySize = null;
    private long wakeUpTime = 0;
    private FontMetrics fm = null;

    public WBLabel() {
	mySize = new Dimension(200, 20);
	addKeyListener(this);
/*
	Thread life = new Thread(this);
	life.setDaemon(true);
	life.start();
*/
    }

    public synchronized void run() {
	while (true) {
	    setEditable(System.currentTimeMillis() > wakeUpTime);
	    try {
	        wait((long)Math.max(0, wakeUpTime - 
		    System.currentTimeMillis()));
	    }
	    catch(InterruptedException e) { }
	}
    }

    public void setText(String text) {
	wakeUpTime = System.currentTimeMillis() + 5000;
	synchronized(this) {
	    notifyAll();
	}
	super.setText(text);
    }

    public void removeNotify() {
    }

    public String getText() {
	return super.getText();
    }

    public void paint(Graphics g) {
	g.setFont(getFont());
	g.setColor(getForeground());
	fm = g.getFontMetrics();
	super.paint(g);
    }

    private void findSize() {
	if (fm != null) {
	    Dimension d = getSize();
	    mySize = new Dimension(fm.stringWidth(getText()) + 10, d.height);
	    setSize(mySize);
	}
    }

    // ------------- KeyListener methods -------------
    public void keyTyped(KeyEvent e) {
	support.firePropertyChange("text", null, getText());
	// findSize();
    }
    public void keyPressed(KeyEvent e) { }
    public void keyReleased(KeyEvent e) { }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        support.addPropertyChangeListener(listener);
    }
 
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        support.removePropertyChangeListener(listener);
    }

    public void setSize(Dimension size) {
/*
	if (fm != null)
	    size.height = fm.getHeight();
*/
	if ((size.width != 0) && (size.height != 0)) 
            this.mySize = size;
        super.setSize(size);
	repaint();
    }
    public void setSize(int w, int h) {
/*
	if (fm != null)
	    h = fm.getHeight();
*/
	if ((w != 0) && (h != 0)) 
            this.mySize = new Dimension(w, h);
        super.setSize(w, h);
	repaint();
    }
    public void setBounds(int x, int y, int w, int h) {
/*
	if (fm != null)
	    h = fm.getHeight();
*/
	if ((w != 0) && (h != 0)) 
            this.mySize = new Dimension(w, h);
        super.setBounds(x, y, w, h);
	repaint();
    }
    public void setBounds(Rectangle r) {
/*
	if (fm != null)
	    r.height = fm.getHeight();
*/
	if ((r.width != 0) && (r.height != 0)) 
            this.mySize = new Dimension(r.width, r.height);
        super.setBounds(r);
	repaint();
    }

    public Dimension getMinimumSize() {
        return mySize;
    }
    public Dimension minimumSize() {
        return mySize;
    }
    public Dimension preferredSize() {
        return mySize;
    }
    public Dimension getPreferredSize() {
        return mySize;
    }
}

