/*  
 * $Id: SharedTextWrapper.java,v 1.2 1998/01/02 20:51:56 isenhour Exp $
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sieve.whiteboard.text;

import java.awt.*;
import java.util.*;
import com.sun.java.swing.*;
import com.sun.java.swing.text.*;

/**
 *
 * @version 	$Revision: 1.2 $
 * @author James "Bo" Begole
 */

public class SharedTextWrapper extends JPanel {
  SharedTextArea textArea = null;

  public SharedTextWrapper() {
    setLayout(new BorderLayout());
    textArea = new SharedTextArea();
    JScrollPane scroller = 
      new JScrollPane(textArea, 
		      ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
		      ScrollPaneConstants. HORIZONTAL_SCROLLBAR_ALWAYS);
    add("Center", scroller);
  }

  public SharedTextArea getSharedTextArea() {
    return textArea;
  }

  private String lastCommand;
  public void setTextCommand(String command) {
    lastCommand = command;    
    if (command != null) {
      if (command.startsWith("insert")) {
	// do insertion
	// example format: "insert|0|This is a test"
	StringTokenizer st = new StringTokenizer(command, "|");
	String token = st.nextToken(); // "insert"
	token = st.nextToken(); // int offset
	int offset = Integer.parseInt(token);
	token = st.nextToken(); // the string to insert
	SharedDocument doc = (SharedDocument)textArea.getDocument();
	try {
	  doc.remoteInsertString(offset, token, null);
	} catch (BadLocationException ble) {
	  // ignore it, but report it
	  System.out.println("offset="+offset+", str="+token);
	  System.out.println("Error inserting text: "+ble);
	  ble.printStackTrace();
	}
      } else if (command.startsWith("remove")) {
	// do removal
	// example format: "remove|4|6"
	StringTokenizer st = new StringTokenizer(command, "|");
 	String token = st.nextToken(); // "insert"
	token = st.nextToken(); // int offset
	int offset = Integer.parseInt(token);
	token = st.nextToken(); // int length
	int length = Integer.parseInt(token);
	SharedDocument doc = (SharedDocument)textArea.getDocument();
	try {
	  doc.remoteRemove(offset, length);
	} catch (BadLocationException ble) {
	  // ignore it, but report it
	  System.out.println("offset="+offset+", length="+length);
	  System.out.println("Error removing text: "+ble);
	  ble.printStackTrace();
	}
      }
    }
  }
  public String getTextCommand() {
    return lastCommand;
  }

  public Dimension getPreferredSize() {
    Dimension size = getSize();
    if ((size.width < myMinSize.width) && (size.height<myMinSize.height))
      return getMinimumSize();
    else if (size.width < myMinSize.width)
      return new Dimension(myMinSize.width, size.height);
    else if (size.height < myMinSize.height)
      return new Dimension(size.width, myMinSize.height);
    else 
      return size;
  }

  Dimension myMinSize = new Dimension(200, 300); 
  public Dimension getMinimumSize() {
    return myMinSize;
  }

}
