package sieve.workbench;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

/**
 * DefaultTool is the tool used to create objects for which no other
 * tool can be found. The behavior of this tool mimics the behavior
 * of the BeanBox: the new object is placed on the Workbench at the
 * location that the mouse is released.
 */
public class DefaultTool extends SimpleTool implements Tool {
    Workbench wb = null;
    Class     cl = null;

    public DefaultTool(Workbench wb, Class cl) {
	this.wb = wb;
	this.cl = cl;
    }

    public DefaultTool() { }

    public void init(Workbench wb, Class cl) {
	this.wb = wb;
	this.cl = cl;
    }

    /**
     * Subclasses which need the default UI for creating components
     * (i.e., "click to create") can just override this method to
     * return a particular kind of WorkbenchComponent.
     */
    protected WorkbenchComponent getWorkbenchComponent(Workbench wb,
	Class cl) {
	try {
	  // I'm using Beans.instantiate() rather than cl.newInstance here,
	  // even though -- since we have a Class object -- we know
	  // enough for newInstance to work properly. Beans.instantiate,
	  // however, provides at least minimal support for Applets.
	  return new DefaultWrapper(wb, 
	    Beans.instantiate(cl.getClassLoader(), cl.getName()));
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	return null;
    }


    public void mouseReleased(MouseEvent e) { 
	Cursor c = wb.getCursor();
	wb.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	WorkbenchComponent wrapper = getWorkbenchComponent(wb, cl);
	wrapper.getView().setLocation(e.getPoint());
	wb.add(wrapper.getView());
	wb.setCursor(c);
	wb.componentAdded(wrapper, this);
    }

    public Cursor getPreferredCursor() {
	return null;
    }

}



