package sieve.workbench;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.Serializable;

public class SimpleLink extends MouseAdapter implements Link, Serializable {

    private WorkbenchComponent source	= null;
    private WorkbenchComponent dest	= null;
    private Workbench 	   wb		= null;
    private UniqueID	   id		= null;
    private SimpleLinkComponent arrow 	= null;

    public SimpleLink(Workbench wb, WorkbenchComponent source) throws 
	LinkException {
	init(wb, source, null, null);
    }

    public SimpleLink() {}

    public void init(Workbench wb, WorkbenchComponent source, 
        WorkbenchComponent dest, Serializable state) throws LinkException {
	this.wb = wb;
   	if (! (source instanceof DefaultWrapper))
	    throw new LinkException("Destination incompatible");
	this.source = (DefaultWrapper)source;
	arrow = new SimpleLinkComponent(source.getView());
	if (dest != null)
	    setDestComponent(dest);
	id = new UniqueID(this);
      	arrow.addMouseListener(this);
    }

    public Serializable getState() {
	return null;
    }

    public Component getView() {
	return arrow;
    }

    /**
     * Returns the unique ID associated with this link.
     */
    public UniqueID getUniqueID() {
	return id;
    }

    /**
     * Sets the unique ID associated with this link.
     */
    public void setUniqueID(UniqueID id) {
	this.id = id;
    }

    /**
     * Return the component from which the link was started.
     */
    public WorkbenchComponent getSourceComponent() {
	return source;
    }
 
    /**
     * Return the component to which the link was drawn.
     */
    public WorkbenchComponent getDestComponent() {
	return dest;
    }
    
    public void remove() {
	doUnlink();
	arrow.remove();
    }

    public void setDestComponent(WorkbenchComponent c) throws LinkException {
	// Should be able to remove this now...
   	if (! (c instanceof DefaultWrapper))
	    throw new LinkException("Destination incompatible");
	if (c == null)
	    throw new LinkException("Null destination component");

	dest = c;
	link(source, dest);

	arrow.setDestComponent(dest.getView());
    }

    public void unlink(WorkbenchComponent source, WorkbenchComponent dest) 
	throws LinkException {}

    public void link(WorkbenchComponent source, WorkbenchComponent dest) 
	throws LinkException {
	arrow.setColor(Color.gray);
    }

    public void setColor(Color c) {
	arrow.setColor(c);
    }
    public Color getColor() {
	return arrow.getColor();
    }

    private void doUnlink() {
	try {
	    unlink(source, dest);
	}
	catch(LinkException e) {
	}
	if (dest != null) {
	    wb.linkRemoved(this);
	    dest = null;
 	}
    }

    public void mouseClicked(MouseEvent e) {
	arrow.remove();
      	doUnlink();
    	wb.repaint();
    }

    //  MouseMotionListener methods ------------

    public  void mouseMoved(MouseEvent e) { 
	arrow.setColor(Color.red);
	arrow.setTempEnd(e.getX(), e.getY());
    }
    public  void mouseDragged(MouseEvent e) {}
}


