package sieve.workbench;

/**
 * A Tool is responsible for instantiating a new component. Notes:
 * <UL>
 * <LI> Tool object are the <B>ONLY</B> objects which may instantiate
 *      WorkbenchComponents.
 * <LI> Tools <B>MUST</B> have empty constructors. Initialization is
 *      performed by calling the init() method.
 * <LI> MouseEvents are the <B>ONLY</B> inputs that a Tool may use
 *      to construct a WorkbenchComponent.
 * <LI> Tools <B>MUST NOT</B> register themselves with the Workbench (or
 *      any other objects) as MouseListeners or MouseMotionListeners. 
 *      If appropriate, this will be done by an external object.
 * <LI> Tools should only be acquired through the ToolFactory.
 * </UL>
 */
import java.awt.event.*;
import java.awt.*;

public interface Tool extends MouseListener, MouseMotionListener {

    /**
     * Initializes the tool with a workbench and the class of the object
     * to be constructed.
     */
    public void init(Workbench wb, Class cl);

    /**
     * The cursor that this tool prefers to have displayed while it is
     * active. The crosshair cursor will be used if this returns null.
     */
    public Cursor getPreferredCursor();

}

