package sieve.workbench;

import java.util.*;
import java.beans.Beans;

/**
 * The ToolFactory is preferred method of obtaining tool objects for
 * constructing WorkbenchComponents. Classes which have associated Tool
 * objects (e.g., whiteboard components) should register themselves
 * in static initialization code.
 */
public class ToolFactory {
    // Eventually we may need one per WB
    static private ToolFactory instance = null; 

    private  Hashtable tools = null;
    private  Vector failedToolClasses = null;

    private ToolFactory() {
	tools = new Hashtable();
	failedToolClasses = new Vector();
    }

    /**
     * Returns a reference to a ToolFactory object.
     */
    public static ToolFactory getToolFactory() {
	if (instance == null)
	    instance = new ToolFactory();
	return instance;
    }

    /**
     * Registers the tool to be used to create the specified class.
     */
    public void registerTool(String componentClassName, Class toolClass) {
	tools.put(componentClassName, toolClass);
    }

    /**
     * Registers the tool to be used to create the specified class.
     */
    public void registerTool(Class componentClass, Class toolClass) {
	registerTool(componentClass.getName(), toolClass);
    }

    private Tool getToolFromClassName(Class cl) {
	Tool tool = null;
	if (! failedToolClasses.contains(cl)) {
	    try {
	        String toolName = cl.getName() + "Tool";
	        tool = (Tool)Beans.instantiate(cl.getClassLoader(), toolName);
	    }
	    catch(Exception e) { 
		failedToolClasses.addElement(cl);
	    }
	}
	return tool;
    }

    /**
     * Returns a tool for constructing an object with the specified class.
     */
    public Tool getTool(Class componentClass, Workbench wb) {
	Class toolClass = null;
	Tool tool = null;
	Class superClass = componentClass;

	// If there isn't a tool registered for this class, search its
	// superclasses.
	while ((superClass != null) && (toolClass == null)) {
	    toolClass = (Class)tools.get(superClass.getName());

	    // If there was nothing explicitly registered, try appending
	    // "Tool" to the class name.
	    if (toolClass == null) {
		tool = getToolFromClassName(superClass);

		// If that doesn't work, check the interfaces...
		if (tool == null) {
		    Class[] ints = superClass.getInterfaces();

		    for(int i = 0; (i < ints.length) && (tool == null) &&
			(toolClass == null); i++) {
	    		toolClass = (Class)tools.get(ints[i].getName());
	    		if (toolClass == null) 
			    tool = getToolFromClassName(ints[i]);
		    }
		}
	    }
		
	    if (tool != null)
		toolClass = tool.getClass();
	    superClass = superClass.getSuperclass();
	}

	// If no tool was found, use the default one with BeanBox-ish 
	// behavior
	if (toolClass == null)
	    toolClass = DefaultTool.class;
	try {
	    if (tool == null)
	        tool = (Tool)toolClass.newInstance();
	    tool.init(wb, componentClass);
	    tools.put(componentClass, toolClass); // Not sure about this...
	    return tool;
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

}

