package sieve.workbench.collab;
 
import java.util.*;
import sieve.workbench.*;

/**
 * A ComponentRegistry maps UniqueIDs to WorkbenchComponents, Links and 
 * eventually Participants (?) for a given Workbench.
 */
public class ComponentRegistry {
    private static Hashtable instances = new Hashtable();

    private Hashtable components = new Hashtable();
    private Hashtable links = new Hashtable();

    /**
     * There is exactly one ComponentRegistry per Workbench. This
     * method returns a reference to that instance for the specified
     * Workbench.
     */
    public static ComponentRegistry getComponentRegistry(Workbench wb) {
	ComponentRegistry instance = (ComponentRegistry) instances.get(wb);
	if (instance == null) {
	    instance = new ComponentRegistry();
	    instances.put(wb, instance);
	}
	return instance;
    }

    /**
     * Returns the WorkbenchComponent with the given ID.
     */
    public WorkbenchComponent getWorkbenchComponent(UniqueID id) {
	return (WorkbenchComponent) components.get(id);
    }

    /**
     * Registers a WorkbenchComponent. The WorkbenchComponent's 
     * getUniqueID() method is used to find the id.
     */
    public void registerWorkbenchComponent(WorkbenchComponent wbc) {
	components.put(wbc.getUniqueID(), wbc);
    }
    /**
     * Removes a WorkbenchComponent from the registry. The WorkbenchComponent's 
     * getUniqueID() method is used to find the id.
     */
    public void removeWorkbenchComponent(WorkbenchComponent wbc) {
	components.remove(wbc.getUniqueID());
    }

    /**
     * Returns the Link with the given ID.
     */
    public Link getLink(UniqueID id) {
	return (Link) links.get(id);
    }

    /**
     * Registers a Link. The Link's getUniqueID() method is used to find the
     * id.
     */
    public void registerLink(Link link) {
	links.put(link.getUniqueID(), link);
    }

    /**
     * Removes a Link from the registry. The Link's 
     * getUniqueID() method is used to find the id.
     */
    public void removeLink(Link link) {
	links.remove(link.getUniqueID());
    }

}

