package sieve.workbench.collab;

import sieve.workbench.collab.msg.Message;
import java.rmi.*;

/**
 * This is the interface that clients can use to add, remove, and find
 * out about channels associated with a Server object.
 * Currently, clients should not use this interface directly,
 * but instead use the WorkbenchChannelManager class.
 * 
 * @see WorkbenchChannelManager
 */
public interface ServerInterface extends Remote {
    /**
     * Adds the specified session if it doesn't already exist. 
     *
     * @return The URL for the session
     */
    public String addSession(String session) throws RemoteException;

    /**
     * Returns a list of all available session names.
     */
    public String[] getSessions() throws RemoteException;

    /**
     * Adds the specified channel if it doesn't already exist. Listeners
     * are notified when this happens.
     */
    public void addChannel(String session, String channel) 
	throws RemoteException;

    /**
     * Removes the specified channel if it exists. Listeners
     * are notified when this happens.
     */
    public void removeChannel(String session, String channel) 
	throws RemoteException;

    /**
     * Returns the names of all channels avaiable on the given session.
     */
    public String[] getChannels(String session) throws RemoteException;

    /**
     * Returns all stored messages for the given channel. These
     * messages represent all interesting activity on the channel up
     * to the time that this call is made.
     */
    public Message[] getStoredMessages(String session, String channel) 
	throws RemoteException;

    /**
     * Adds a listener for session and channel creation/deletion
     */
    public void addServerListener(ServerListener l) throws RemoteException;

    /**
     * Removes a listener for session and channel creation/deletion
     */
    public void removeServerListener(ServerListener l) throws RemoteException;

}




