package sieve.workbench.collab;

import java.util.*;
import java.rmi.*;

/**
 * Utility class for use by the Server to handle registration and 
 * notification of ServerListeners. 
 */
public class ServerListenerSupport {
    private Server source = null;
    private Vector listeners = new Vector();

    public ServerListenerSupport(Server source) {
	this.source = source;
    }

    public synchronized void addServerListener(ServerListener l) {
	listeners.addElement(l);
    }

    public synchronized void removeServerListener(ServerListener l) {
	listeners.removeElement(l);
    }

    public void sessionAdded(String sessionName) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	for (int i = 0; i < targets.size(); i++) {
	    try {
	       ((ServerListener)targets.elementAt(i)).sessionAdded(sessionName);
	    }
	    catch (RemoteException e) {
		removeServerListener((ServerListener)targets.elementAt(i));
	    }
	}
    }

    public void channelAdded(String sessionName, String channelName) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	for (int i = 0; i < targets.size(); i++) {
	    try {
	       ((ServerListener)targets.elementAt(i)).channelAdded(sessionName,
		    channelName);
	    }
	    catch (RemoteException e) {
		removeServerListener((ServerListener)targets.elementAt(i));
	    }
	}
    }

}

