package sieve.workbench.collab;

import sieve.workbench.*;
import sieve.workbench.collab.msg.Message;
import java.awt.event.*;
import java.io.*;


public class SingleUserSupport {
	private Workbench				wb = null;
	private Controller				controller = null;
	private ReplayController		replay = null;
	private LocalRecorderClient		localClient = null;
	private Broadcaster				localBroadcaster = null;

/**
 * SingleUserSupport provides functionality for a user to save and restore workspaces in 
 * single-user mode (and to save an restore local versions in collaborative mode). It 
 * parallels the collaborative-mode message passing sequences by creating local copies of 
 * the broadcaster, recorder and controller.
 * <IMG SRC="wbarch.gif">
 *
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 *
 */

	public SingleUserSupport(Workbench w) {
		wb = w;
		controller = new Controller(wb);
		replay = new ReplayController(controller);
		localClient = new LocalRecorderClient(); 
		localBroadcaster = new Broadcaster(wb, controller, localClient);
		localBroadcaster.startBroadcasting();
	}

	public synchronized void restoreWorkbench() {
	    // All hell breaks loose if we start driving the workbench
	    // with old messages without turning off broadcasting.
	    localBroadcaster.stopBroadcasting();

	    // Get all of the old messages
	    Message[] msgs = localClient.getRecorder().getMessages();
            for (int i = 0; i < msgs.length; i++) 
                msgs[i].deliver(controller);	//replay or the real one? My guess is that here 
												//we are restoring the current session, so we use the "real" controller
	    // Restart broadcasting
        localBroadcaster.startBroadcasting();
	}

	//will need handle to this if user wants to load a locally-saved session.
	//public Controller getSingleUserController() {
	//	return controller;
	//}

	public void loadSnapshot(String fileName) { //name must be .ser file
		if(fileName != null) {
			try {
				File f = new File(fileName);
				if(f.exists()) {
					ObjectInputStream fin = new ObjectInputStream(new BufferedInputStream(new FileInputStream(f)));
					ChannelRecorder recorder = (ChannelRecorder)fin.readObject();			
					Message[] msgs = recorder.getMessages();
					for(int i=0; i<msgs.length; i++) {
						msgs[i].deliver(replay);
						localClient.send(msgs[i]);	//broadcaster won't send these since originated at controllor. Bypass. May want to rethink this later.
					}
				}
			}
			catch(Exception e) {e.printStackTrace();}
		}
	}

	public void saveSnapshot(String fileName) { //name must be .ser file
		if(fileName != null) {
			try {
				File f = new File(fileName);
				ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(f));
				out.writeObject(localClient.getRecorder());
				out.close();
			}
			catch(Exception e) {e.printStackTrace();}
		}

	}

}

