package sieve.workbench.collab;

import sieve.workbench.collab.msg.Message;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import SharedData.*;
import java.util.*;

public class WorkbenchChannelManager extends UnicastRemoteObject
    implements ServerListener {
    private static Hashtable instances = new Hashtable();

    private ServerInterface server = null;
    private String session = null;
    private String sessionURL = null;
    private Session sessionObject = null;

    private String host = null;
    private WorkbenchChannelListenerSupport listeners = null;

    public synchronized static WorkbenchChannelManager 
	getManager(String host, String session) {
	WorkbenchChannelManager mgr = 
	    (WorkbenchChannelManager) instances.get(session);
	if (mgr == null) {
	    try {
		mgr = new WorkbenchChannelManager(host, session);
		instances.put(session, mgr);
	    }
	    catch(Exception e) {
		System.out.println("WARNING: Exception on Manager creation ");
	  	e.printStackTrace();
	    }
	}
	return mgr;
    }
    
    /**
     * This method may be used to see if the WorkbenchServer is currently
     * accessible.
     */
    public static boolean isServerUp(String host) {
	try {
	    Registry r = 
		LocateRegistry.getRegistry(host, Server.REGISTRY_PORT);    
	    ServerInterface server = (ServerInterface) r.lookup(Server.NAME);
	    return (server != null);
	}
	catch(Exception e) {}
	return false;
    }

    protected WorkbenchChannelManager(String host, String session) throws
	RemoteException, NotBoundException {
	this.session = session;
	this.host = host;
	listeners = new WorkbenchChannelListenerSupport(this);
	try {
	    Registry r = 
		LocateRegistry.getRegistry(host, Server.REGISTRY_PORT);    
	    server = (ServerInterface) r.lookup(Server.NAME);
	    sessionURL = server.addSession(session);
// System.out.println("ChanMan getting session at: " + sessionURL);
	    sessionObject = SessionFactory.createSession(sessionURL);
	    server.addServerListener(this);
	}
	catch(Exception e) {
	    System.out.println("Error creating session. " + 
		"The manager will still work, but no session/channel " + 
		"support will be available.");
	    e.printStackTrace();
	}
    }

    public Session getSession() {
	if (sessionObject == null) {
	    try {
	        sessionObject = SessionFactory.createSession(sessionURL);
	    }
	    catch(Exception e) {
	        System.out.println("Error creating session: ");
	        e.printStackTrace();
	    }
	}
	return sessionObject;
    }

    public String getHost() {
	return host;
    }

    public void addChannel(String channel) {
	if (server != null) {
	    try {
	        server.addChannel(session, channel);
	    }
	    catch (Exception e) {
	        e.printStackTrace();
	    }
	}
    }

    public void removeChannel(String channel) {
	if (server != null) {
	    try {
	        server.removeChannel(session, channel);
	    }
	    catch (Exception e) {
	        e.printStackTrace();
	    }
	}
    }

    public String[] getAvailableChannels() {
	String[] names = new String[0];
	if (server != null) {
	    try {
	        names = server.getChannels(session);
	    }
	    catch (Exception e) {
	        e.printStackTrace();
	    }
	}
	return names;
    }

    public Message[] getStoredMessages(String channel) {
	Message[] msgs = new Message[0];
	if (server != null) {
	    try {
	        msgs = server.getStoredMessages(session, channel);
	    }
	    catch (Exception e) {
	        e.printStackTrace();
	    }
	}
	return msgs;
    }

    // --------- ServerListener interface methods ---------
    /**
     * Called when a new session is added on the server.
     */
    public void sessionAdded(String sessionName) throws RemoteException {
	// Don't care...
    }

    /**
     * Called when a new channel is added in a session.
     */
    public void channelAdded(String sessionName, String channelName)
	throws RemoteException {
	if (sessionName.equals(session))
	    listeners.channelAdded(channelName);
System.out.println("Got channel added. session = " + sessionName + " channel = " + channelName);
    }

    /**
     * Called when a session is removed.
     */
    public void sessionRemoved(String sessionName)
	throws RemoteException {
	// Don't care... (eventually check if its our session)
    }

    /**
     * Called when a channel is removed.
     */
    public void channelRemoved(String sessionName, String channelName)
	throws RemoteException {
	if (sessionName.equals(session))
	    listeners.channelRemoved(channelName);
//System.out.println("Got channel removed. session = " + sessionName + " channel = " + channelName);
    }


    // ---------------- Listener Support ----------------
    /**
     * Adds a WorkbenchChannelListener. These objects will be notified
     * when channels available through this WorkbenchChannelManager instance
     * are added or removed.
     */
    public void addWorkbenchChannelListener(WorkbenchChannelListener l) {
	listeners.addWorkbenchChannelListener(l);
    } 

    /**
     * Removes a WorkbenchChannelListener. 
     */
    public void removeWorkbenchChannelListener(WorkbenchChannelListener l) {
	listeners.removeWorkbenchChannelListener(l);
    } 
}

