package sieve.workbench.event;

import sieve.workbench.*;
import java.util.*;

/**
 * Utility class for use by Workbench objects (such as the Workbench) to 
 * handle registration and notification of ToolListeners. 
 */
public class ToolEventSupport {
    private Workbench source = null;
    private Vector listeners = new Vector();

    public ToolEventSupport(Workbench source) {
	this.source = source;
    }

    public synchronized void addToolListener(ToolListener tl) {
	listeners.addElement(tl);
    }

    public synchronized void removeToolListener(ToolListener tl) {
	listeners.removeElement(tl);
    }

    public void toolChanged(Tool oldTool) {
	ToolEvent e = new ToolEvent(source, source.getCurrentTool(),
	    oldTool, source.isToolRetained(), null);

	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	for (int i = 0; i < targets.size(); i++) {
	    ((ToolListener)targets.elementAt(i)).toolChanged(e);
	}
    }

    public void toolUsed(WorkbenchComponent c) {
	ToolEvent e = new ToolEvent(source, source.getCurrentTool(),
	    null, source.isToolRetained(), c);

	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	for (int i = 0; i < targets.size(); i++) {
	    ((ToolListener)targets.elementAt(i)).toolUsed(e);
	}
    }

    public void toolRetainedStateChanged() {
	ToolEvent e = new ToolEvent(source, source.getCurrentTool(),
	    null, source.isToolRetained(), null);

	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	for (int i = 0; i < targets.size(); i++) {
	    ((ToolListener)targets.elementAt(i)).toolRetainedStateChanged(e);
	}
    }

}



