package sieve.workbench.event;

import sieve.workbench.*;
import java.util.*;
import java.beans.PropertyChangeEvent;

/**
 * Utility class for use by the Workbench to handle registration and 
 * notification of WorkbenchListeners. 
 */
public class WorkbenchListenerSupport {
    private Workbench source = null;
    private Vector listeners = new Vector();

    public WorkbenchListenerSupport(Workbench source) {
	this.source = source;
    }

    public synchronized void addWorkbenchListener(WorkbenchListener l) {
	listeners.addElement(l);
    }

    public synchronized void removeWorkbenchListener(WorkbenchListener l) {
	listeners.removeElement(l);
    }

    public void componentAdded(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentAdded(e);
	}
    }

    public void componentSelected(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentSelected(e);
	}
    }

    public void componentDeselected(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentDeselected(e);
	}
    }


    public void componentRemoved(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentRemoved(e);
	}
    }

    public void componentBoundsChanged(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentBoundsChanged(e);
	}
    }

    public void componentZOrderChanged(WorkbenchComponent c) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentZOrderChanged(e);
	}
    }

    public void componentEdited(WorkbenchComponent c, EditEvent pce) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	WorkbenchComponentEvent e = new WorkbenchComponentEvent(source, c, pce);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).componentEdited(e);
	}
    }

    public void linkStarted(Link l) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	LinkEvent e = new LinkEvent(source, l);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).linkStarted(e);
	}
    }

    public void linkAdded(Link l) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	LinkEvent e = new LinkEvent(source, l);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).linkAdded(e);
	}
    }

    public void linkRemoved(Link l) {
	Vector targets;
	synchronized(this) {
	    targets = (Vector)listeners.clone();
	}
	LinkEvent e = new LinkEvent(source, l);

	for (int i = 0; i < targets.size(); i++) {
	    ((WorkbenchListener)targets.elementAt(i)).linkRemoved(e);
	}
    }

}


